Package: scTensor
Type: Package
Title: Detection of cell-cell interaction from single-cell RNA-seq
        dataset by tensor decomposition
Version: 2.22.0
Authors@R: c(person("Koki", "Tsuyuzaki", role = c("aut", "cre"),
                email = "k.t.the-answer@hotmail.co.jp"),
            person("Kozo", "Nishida", role = "aut",
                email = "kozo.nishida@gmail.com"))
Depends: R (>= 4.1.0)
Imports: methods, RSQLite, igraph, S4Vectors, plotly, reactome.db,
        AnnotationDbi, SummarizedExperiment, SingleCellExperiment,
        nnTensor (>= 1.1.5), ccTensor (>= 1.0.2), rTensor (>= 1.4.8),
        abind, plotrix, heatmaply, tagcloud, rmarkdown, BiocStyle,
        knitr, AnnotationHub, MeSHDbi (>= 1.29.2), grDevices, graphics,
        stats, utils, outliers, Category, meshr (>= 1.99.1), GOstats,
        ReactomePA, DOSE, crayon, checkmate, BiocManager, visNetwork,
        schex, ggplot2
Suggests: testthat, LRBaseDbi, Seurat, scTGIF, Homo.sapiens
Description: The algorithm is based on the non-negative tucker decomposition (NTD2) of nnTensor.
License: Artistic-2.0
biocViews: DimensionReduction, SingleCell, Software, GeneExpression
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/scTensor
git_branch: RELEASE_3_23
git_last_commit: ec309af
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 00:36:52 UTC; biocbuild
Author: Koki Tsuyuzaki [aut, cre],
  Kozo Nishida [aut]
Maintainer: Koki Tsuyuzaki <k.t.the-answer@hotmail.co.jp>

import(methods)
import(SingleCellExperiment)
import(BiocStyle)
import(knitr)
import(RSQLite)
import(AnnotationHub)

importFrom(BiocManager, install)
importFrom(AnnotationDbi, toTable)
importFrom(igraph, graph.data.frame, bipartite_mapping, max_bipartite_match, graph.empty, add.vertices, add.edges, "E<-", "V<-", V, E, layout_with_dh, plot.igraph, get.edgelist, get.vertex.attribute, edge.attributes)
importFrom(reactome.db, reactomeEXTID2PATHID)
importFrom(MeSHDbi, select)
importFrom(S4Vectors, metadata, "metadata<-")
importFrom(plotly, plot_ly)
importFrom(SummarizedExperiment, assay, assays, "assay<-", "assays<-")
importFrom(rTensor, as.tensor, modeSum, rs_unfold, cs_unfold)
importFrom(abind, abind)
importFrom(plotrix, gradient.rect)
importFrom(heatmaply, heatmaply)
importFrom(tagcloud, smoothPalette, tagcloud)
importFrom(rmarkdown, render)

importFrom(grDevices, colorRampPalette, dev.off, png, rgb)
importFrom(graphics, par, plot, text, legend)
importFrom(utils, browseURL, txtProgressBar, setTxtProgressBar)
importFrom(outliers, grubbs.test, chisq.out.test)
importFrom(nnTensor, NMF, NTD, recTensor)
importFrom(ccTensor, MultiCX)
importFrom(Category, hyperGTest)

importFrom(meshr, meshHyperGTest)
import(GOstats)
importFrom(ReactomePA, enrichPathway)
importFrom(DOSE, enrichDGN, enrichDO, enrichNCG)

importFrom(crayon, bold, blue, green)
importFrom(checkmate, assertClass, assertList)
importFrom(stats, median, cor, dist, na.omit, quantile, p.adjust, hclust, cutree, rbinom, rnbinom, var, sd)
importFrom("utils", "data")
importFrom("grDevices", "hcl")

importFrom(visNetwork, visNetwork)
importFrom(schex, make_hexbin, plot_hexbin_feature)
importFrom(ggplot2, scale_fill_gradient, ggsave)

exportMethods(cellCellSetting, cellCellRanks, cellCellDecomp, cellCellReport, getParam, "setParam<-")
export(cellCellSimulate, newCCSParams)

.annotationhub_taxid <- function(taxid){
    ###################################
    # OrgDb Search
    ###################################
    ahub <- AnnotationHub()
    target1 <- which(mcols(ahub)[, "taxonomyid"] == taxid)
    target2 <- which(mcols(ahub)[, "rdataclass"] %in% c("OrgDb"))
    target <- intersect(target1, target2)
    ah_ids <- rev(rownames(mcols(ahub))[target])
    hit <- FALSE
    for(i in seq_along(ah_ids)){
        ah_id <- ah_ids[i]
        ah <- ahub[[ah_id]]
        check <- length(which(AnnotationDbi::columns(ah) == "ENTREZID")) == 1
        if(check){
            hit <- TRUE
            break
        }
    }
    if(hit){
        ah
    }else{
        ###################################
        # EnsDb Search
        ###################################
        target3 <- which(mcols(ahub)[, "rdataclass"] %in% c("EnsDb"))
        target <- intersect(target1, target3)
        ah_ids <- rev(rownames(mcols(ahub))[target])
        hit <- FALSE
        for(i in seq_along(ah_ids)){
            ah_id <- ah_ids[i]
            ah <- ahub[[ah_id]]
            check <- length(which(AnnotationDbi::columns(ah) == "ENTREZID")) == 1
            if(check){
                hit <- TRUE
                break
            }
        }
        if(hit){
            ah
        }else{
            NULL
        }
    }
}

.annotationhub <- list(
    "Hsa" = function(){
        query(AnnotationHub(), c("OrgDb", "Homo sapiens", "org.Hs.eg.db.sqlite"))[[1]]
    },
    "Mmu" = function(){
        query(AnnotationHub(), c("OrgDb", "Mus musculus", "org.Mm.eg.db.sqlite"))[[1]]
    },
    "Ath" = function(){
        query(AnnotationHub(), c("OrgDb", "Arabidopsis thaliana", "org.At.tair.db.sqlite"))[[1]]
    },
    "Rno" = function(){
        query(AnnotationHub(), c("OrgDb", "Rattus norvegicus", "org.Rn.eg.db.sqlite"))[[1]]
    },
    "Bta" = function(){
        query(AnnotationHub(), c("OrgDb", "Bos taurus", "org.Bt.eg.db.sqlite"))[[1]]
    },
    "Cel" = function(){
        query(AnnotationHub(), c("OrgDb", "Caenorhabditis elegans", "org.Ce.eg.db.sqlite"))[[1]]
    },
    "Dme" = function(){
        query(AnnotationHub(), c("OrgDb", "Drosophila melanogaster", "org.Dm.eg.db.sqlite"))[[1]]
    },
    "Dre" = function(){
        query(AnnotationHub(), c("OrgDb", "Danio rerio", "org.Dr.eg.db.sqlite"))[[1]]
    },
    "Gga" = function(){
        query(AnnotationHub(), c("OrgDb", "Gallus gallus", "org.Gg.eg.db.sqlite"))[[1]]
    },
    "Pab" = function(){
        query(AnnotationHub(), c("OrgDb", "Pongo abelii", "org.Pongo_abelii.eg.sqlite"))[[1]]
    },
    "Xtr" = function(){
        query(AnnotationHub(), c("OrgDb", "Xenopus", "Silurana", "org.Xenopus_\\(Silurana\\)_tropicalis.eg.sqlite"))[[1]]
    },
    "Ssc" = function(){
        query(AnnotationHub(), c("OrgDb", "Sus scrofa", "org.Ss.eg.db.sqlite"))[[1]]
    }
)

.SinglePlot <- function(x){
    plot(1, 1, col="white", ann=FALSE, xaxt="n", yaxt="n", axes=FALSE)
    par(ps=100)
    text(1, 1, x, col="red")
}

.shrink2 <- function(x, thr=7){
    block <- strsplit(as.character(x) , " ")[[1]]
    l <- length(block)
    nc <- vapply(block, nchar, 0L)
    if(l >= 2){
        out <- block[1]
        for(i in 2:l){
            end <- length(out)
            tmp <- block[i]
            if(nchar(out[end])+nchar(tmp) <= thr){
                out[end] <- paste(c(out[end], tmp), collapse=" ")
            }else{
                out[end+1] <- tmp
            }
        }
    paste(out, collapse="\n")
    }else{
        x
    }
}

.setColor <- function(col){
    if(col == "reds"){
        c("#FFF5F0", "#FEE0D2", "#FCBBA1", "#FC9272", "#FB6A4A", "#EF3B2C",
            "#CB181D", "#A50F15", "#67000D")
    }else if(col == "blues"){
        c("#F7FBFF", "#DEEBF7", "#C6DBEF", "#9ECAE1", "#6BAED6", "#4292C6",
            "#2171B5", "#08519C", "#08306B")
    }else if(col == "greens"){
        c("#F7FCF5", "#E5F5E0", "#C7E9C0", "#A1D99B", "#74C476", "#41AB5D",
            "#238B45", "#006D2C", "#00441B")
    }else if(col == "many"){
        c("#E41A1C", "#377EB8", "#4DAF4A", "#984EA3", "#FF7F00", "#FFFF33",
            "#A65628", "#F781BF", "#1B9E77", "#D95F02", "#7570B3", "#E7298A",
            "#66A61E", "#E6AB02", "#A6761D", "#66C2A5", "#FC8D62", "#8DA0CB",
            "#E78AC3", "#A6D854", "#FFD92F", "#E5C494", "#B3E2CD", "#FDCDAC",
            "#CBD5E8", "#F4CAE4", "#E6F5C9", "#FFF2AE", "#F1E2CC", "#7FC97F",
            "#BEAED4", "#FDC086", "#FFFF99", "#386CB0", "#F0027F", "#BF5B17",
            "#8DD3C7", "#FFFFB3", "#BEBADA", "#FB8072", "#80B1D3", "#FDB462",
            "#B3DE69", "#FCCDE5", "#D9D9D9", "#BC80BD", "#CCEBC5", "#FFED6F",
            "#FBB4AE", "#B3CDE3", "#CCEBC5", "#DECBE4", "#FED9A6", "#FFFFCC",
            "#E5D8BD", "#FDDAEC")
    }else{
        stop("Wrong col is specified in .setColor")
    }
}

.palf <- colorRampPalette(c("#4b61ba", "gray", "#a87963", "red"))

.REACTOMESPC_taxid <- list(
    "7165" = "anopheles",
    "3702" = "arabidopsis",
    "9913" = "bovine",
    "9615" = "canine",
    "6239" = "celegans",
    "9031" = "chicken",
    "9598" = "chimp",
    "7227" = "fly",
    "5811" = "gondii",
    "9606" = "human",
    "10090" = "mouse",
    "9823" = "pig",
    "10116" = "rat",
    "8355" = "xenopus",
    "7955" = "zebrafish"
)

.ggdefault_cols <- function(n){
    hcl(h=seq(15, 375-360/n, length=n)%%360, c=100, l=65)
}

.knowndbs <- c("DLRP", "IUPHAR", "HPMR", "CELLPHONEDB", "SINGLECELLSIGNALR")

.extractLR <- function(sce, lr.evidence, cols){
    # SQLite connection
    con = dbConnect(SQLite(), metadata(sce)$lrbase)
    LR <- dbGetQuery(con, "SELECT * FROM DATA")
    dbDisconnect(con)
    # Extract corresponding rows
    if(lr.evidence[1] == "all"){
        target <- seq(nrow(LR))
    }else{
        targetKnown <- unique(unlist(lapply(.knowndbs, function(x){grep(x, LR$SOURCEDB)})))
        if(lr.evidence[1] == "known"){
            target <- targetKnown
        }else if(lr.evidence[1] == "putative"){
            target <- setdiff(seq(nrow(LR)), targetKnown)
        }else{
            target <- unique(unlist(lapply(lr.evidence, function(x){
                which(x == LR$SOURCEDB)
            })))
            if(length(target) == 0){
                cat("Please specify the valid lr.evidence!\n")
                cat("In this LRBase package, following databases are avilable as lr.evidence.\n")
                print(as.matrix(table(LR$SOURCEDB)))
                stop("\n")
            }
        }
    }
    .uniqueLR(LR[target, cols])
}

.uniqueLR <- function(LR){
    # Filtering
    targetL <- grep("GENEID", LR$GENEID_L, invert = TRUE)
    targetR <- grep("GENEID", LR$GENEID_R, invert = TRUE)
    targetNotNAL <- which(!is.na(LR$GENEID_L))
    targetNotNAR <- which(!is.na(LR$GENEID_R))
    target <- intersect(targetL,
                intersect(targetR,
                    intersect(targetNotNAL, targetNotNAR)))
    .shrinkLR(unique(LR[target, ]))
}

.shrinkLR <- function(LR){
    targetShrink <- setdiff(colnames(LR), c("GENEID_L", "GENEID_R"))
    if(length(targetShrink) != 0){
        left <- unique(LR[, c("GENEID_L", "GENEID_R")])
        for(i in seq_along(targetShrink)){
            if(i == 1){
                right <- .shrinkNonLR(LR, left, targetShrink[i])
            }else{
                right <- cbind(right,
                    .shrinkNonLR(LR, left, targetShrink[i]))
            }
        }
        out <- cbind(left, right)
        colnames(out) <- c("GENEID_L", "GENEID_R", targetShrink)
        out
    }else{
        LR
    }
}

.shrinkNonLR <- function(LR, left, nonLR){
    obj <- list(LR=LR, nonLR=nonLR)
    apply(left, 1, function(x, obj){
        LR = obj$LR
        nonLR = obj$nonLR
        targetL = which(LR$GENEID_L == as.character(x[1]))
        targetR = which(LR$GENEID_R == as.character(x[2]))
        target = intersect(targetL, targetR)
        if(length(target) != 1){
            out <- paste(LR[target, nonLR], collapse="|")
            out <- unique(strsplit(out, "\\|")[[1]])
            paste(setdiff(out, c("-", "")), collapse=" ")
        }else{
            LR[target, nonLR]
        }
    }, obj=obj)
}

.myvisNetwork <- function(g, col=NULL){
    # Edges
    edges = as.data.frame(get.edgelist(g), stringsAsFactors=FALSE)
    colnames(edges) <- c("from", "to")
    edges <- data.frame(edges,
        color=edge.attributes(g)$color,
        width=edge.attributes(g)$width)
    if(!is.null(col)){
        edges <- edges[which(edges$color == col), ]
    }
    # Nodes
    vattr <- get.vertex.attribute(g)
    uniqueid <- unique(c(edges$from, edges$to))
    color <- sapply(uniqueid, function(x){
        pos <- which(vattr$name == x)
        if(length(pos) == 1){
            vattr$color[pos]
        }else{
            "purple"
        }
    })
    size <- sapply(uniqueid, function(x){
        sum(vattr$size[which(vattr$name == x)])
    })
    nodes <- data.frame(
        id=uniqueid,
        type=TRUE,
        color=color,
        size=size*5,
        label=uniqueid
    )
    # Plot
    visNetwork(nodes, edges)
}

.omasize <- function(l, r){
    # 1 - 16
    c(44.7, 44.7, 22.4, 14.9, 11.15,
    8.95, 7.45, 6.4, 5.6, 4.98,
    4.47, 4.06, 3.75, 3.52, 3.25,
    3.05)[max(l, r)]
}

.oma4 <- function(l, r){
    # 1 - 16
    c(131.4, 131.4, 109.2, 101.5, 97.75,
    95.5, 94.1, 93.1, 92.3, 91.7,
    91.2, 90.7, 90.7, 91.70, 91.000,
    91.000)[max(l, r)]
}

.smallTwoDplot <- function(sce, assayNames, geneid, genename, color){
    g <- schex::plot_hexbin_feature(sce, type=assayNames,
        feature=geneid, action="mean",
        xlab="Dim1", ylab="Dim2",
        title=paste0("Mean of ", genename))
    if(color == "reds"){
        g <- g + scale_fill_gradient(low = 'gray', high = 'red')
    }
    if(color == "blues"){
        g <- g + scale_fill_gradient(low = 'gray', high = 'blue')
    }
    g
}

# For previous LRBase (v-1.0.0 - v-1.2.0)
.TAXID <- c(
    "Hsa" = 9606,
    "Mmu" = 10090,
    "Ath" = 3702,
    "Rno" = 10116,
    "Bta" = 9913,
    "Cel" = 6239,
    "Dme" = 7227,
    "Dre" = 7955,
    "Gga" = 9031,
    "Pab" = 9601,
    "Xtr" = 8364,
    "Ssc" = 9823
)

.hyperLinks <- function(ranking, ligandGeneID, receptorGeneID,
    lr, taxid, value, percentage, GeneInfo, pvalue, qvalue, evidence){
    ## helper for vector dividing
    div <- function(x, d=1) {""
        delta <- ceiling(length(x) / d)
        y <- lapply(seq_len(d), function(i){
            as.vector(na.omit(x[((i-1)*delta+1):(i*delta)]))
        })
        return(y)
    }

    embedLink <- function(genename1, geneid1, description1,
                go1, reactome1, mesh1,
                uni1, string1, refex1,
                ea1, sea1, scdb1, panglao1, cmap1,
                genename2, geneid2, description2,
                go2, reactome2, mesh2,
                uni2, string2, refex2,
                ea2, sea2, scdb2, panglao2, cmap2){
        if(description1 != ""){
            description1 <- paste0("Description: ", genename1, description1, "<br>")
        }
        if(description2 != ""){
            description2 <- paste0("Description: ", genename1, description2, "<br>")
        }
        if(go1 != ""){
            go1 <- paste0("GO: ", go1, "<br>")
        }
        if(go2 != ""){
            go2 <- paste0("GO: ", go2, "<br>")
        }
        if(reactome1 != ""){
            reactome1 <- paste0("Reactome: ", reactome1, "<br>")
        }
        if(reactome2 != ""){
            reactome2 <- paste0("Reactome: ", reactome2, "<br>")
        }
        if(uni1 != ""){
            uni1 <- paste0("UniProtKB: ", uni1, "<br>")
        }
        if(uni2 != ""){
            uni2 <- paste0("UniProtKB: ", uni2, "<br>")
        }
        if(string1 != ""){
            string1 <- paste0("STRING: ", string1, "<br>")
        }
        if(string2 != ""){
            string2 <- paste0("STRING: ", string2, "<br>")
        }
        if(refex1 != ""){
            refex1 <- paste0("RefEx: [", genename1, "](", refex1, ")<br>")
        }
        if(refex2 != ""){
            refex2 <- paste0("RefEx: [", genename1, "](", refex2, ")<br>")
        }
        if(ea1 != ""){
            ea1 <- paste0("Expression Atlas: [", genename1, "](", ea1, ")<br>")
        }
        if(ea2 != ""){
            ea2 <- paste0("Expression Atlas: [", genename2, "](", ea2, ")<br>")
        }
        if(sea1 != ""){
            sea1 <- paste0("Single Cell Expression Atlas: [", genename1, "](", sea1, ")<br>")
        }
        if(sea2 != ""){
            sea2 <- paste0("Single Cell Expression Atlas: [", genename2, "](", sea2, ")<br>")
        }
        if(scdb1 != ""){
            scdb1 <- paste0("scRNASeqDB: [", genename1, "](", scdb1, ")<br>")
        }
        if(scdb2 != ""){
            scdb2 <- paste0("scRNASeqDB: [", genename2, "](", scdb2, ")<br>")
        }
        if(panglao1 != ""){
            panglao1 <- paste0("PanglaoDB: [", genename1, "](", panglao1, ")<br>")
        }
        if(panglao2 != ""){
            panglao2 <- paste0("PanglaoDB: [", genename2, "](", panglao2, ")<br>")
        }
        if(cmap1 != ""){
            cmap1 <- paste0("CMap: [", genename1, "](", cmap1, ")<br>")
        }
        if(cmap2 != ""){
            cmap2 <- paste0("CMap: [", genename2, "](", cmap2, ")<br>")
        }
        if(mesh1 != ""){
            mesh1 <- paste0("MeSH: ", mesh1)
        }
        if(mesh2 != ""){
            mesh2 <- paste0("MeSH: ", mesh2)
        }

        # Output
        paste0(
            "[", genename1, "](https://www.ncbi.nlm.nih.gov/gene/",
            geneid1, ")<br>",
            description1, go1, reactome1, uni1, string1, refex1,
            ea1, sea1, scdb1, panglao1, cmap1, mesh1,
            "|",
            "[", genename2, "](https://www.ncbi.nlm.nih.gov/gene/",
            geneid2, ")<br>",
            description2, go2, reactome2, uni2, string2, refex2,
            ea2, sea2, scdb2, panglao2, cmap2, mesh2,
            "|"
        )
    }

    convertGeneName <- function(geneid, GeneInfo){
        if(!is.null(GeneInfo$GeneName)){
            genename <- GeneInfo$GeneName[
                which(GeneInfo$GeneName$ENTREZID == geneid),
                "SYMBOL"][1]
            if(is.null(genename)){
                genename = geneid
            }else{
                if(is.na(genename)){
                    genename = geneid
                }
                if(length(genename) == 0 || genename %in% c("", NA)){
                    genename = geneid
                }
            }
        }else{
            genename = ""
        }
        genename
    }

    convertGeneDescription <- function(geneid, GeneInfo){
        if(!is.null(GeneInfo$Description)){
            description <- GeneInfo$Description[
                which(GeneInfo$Description$ENTREZID ==
                    geneid), "GENENAME"][1]
            if(length(description) == 0 || description %in% c("", NA)){
                description = ""
            }
            description
        }else{
            description = ""
        }
    }

    convertGeneOntology <- function(geneid, GeneInfo){
        if(!is.null(GeneInfo$GO)){
            GO <- unique(unlist(GeneInfo$GO[
                which(GeneInfo$GO$ENTREZID == geneid),
                "GO"]))
            GO <- GO[which(GO != "")]
            GO <- gsub(":", "%3A", GO)
            if(length(GO) != 0){
                GO_loc <- div(seq_along(GO), ceiling(length(GO) / 100))
                GO <- lapply(GO_loc, function(x){
                    mi <- min(x)
                    ma <- max(x)
                    if(ma == 1){
                        paste0("[", mi, "](",
                            "http://amigo.geneontology.org/amigo/search/ontology?q=",
                            paste0(GO[mi:ma], collapse="%20"), ")")
                    }else{
                        paste0("[", mi, "-", ma, "](",
                            "http://amigo.geneontology.org/amigo/search/ontology?q=",
                            paste0(GO[mi:ma], collapse="%20"), ")")
                    }
                })
                GO <- paste(unlist(GO), collapse=" ")
            }else{
                GO <- ""
            }
        }else{
            GO = ""
        }
        GO
    }

    convertReactome <- function(geneid, GeneInfo){
        if(!is.null(GeneInfo$Reactome)){
            Reactome <- unique(unlist(GeneInfo$Reactome[
                which(GeneInfo$Reactome$gene_id ==
                geneid), "DB_ID"]))
            Reactome <- Reactome[which(Reactome != "")]
            if(length(Reactome) != 0){
                Reactome_loc <- div(seq_along(Reactome),
                    ceiling(length(Reactome) / 100))
                Reactome <- lapply(Reactome_loc, function(x){
                    mi <- min(x)
                    ma <- max(x)
                    if(ma == 1){
                    paste0("[", mi, "](",
                        "https://reactome.org/content/query?q=",
                        paste0(Reactome[mi:ma], collapse="+"),
                        "&types=Pathway&cluster=true)")
                    }else{
                    paste0("[", mi, "-", ma, "](",
                        "https://reactome.org/content/query?q=",
                        paste0(Reactome[mi:ma], collapse="+"),
                        "&types=Pathway&cluster=true)")
                    }
                })
                Reactome = paste(unlist(Reactome), collapse=" ")
            }else{
                Reactome = ""
            }
        }else{
            Reactome = ""
        }
        Reactome
    }

    convertMeSH <- function(geneid, GeneInfo){
        if(!is.null(GeneInfo$MeSH)){
            MeSH <- GeneInfo$MeSH[which(GeneInfo$MeSH$GENEID == geneid),
                "MESHID"]
            MeSH <- MeSH[which(MeSH != "")]
            if(length(MeSH) != 0){
                MeSH_loc <- div(seq_along(MeSH), ceiling(length(MeSH) / 100))
                MeSH <- lapply(MeSH_loc, function(x){
                    mi <- min(x)
                    ma <- max(x)
                    if(ma == 1){
                        paste0("[", mi, "](",
                            "https://www.ncbi.nlm.nih.gov/mesh?term=",
                            paste0(MeSH[mi:ma], collapse="%20OR%20"), ")")
                    }else{
                        paste0("[", mi, "-", ma, "](",
                            "https://www.ncbi.nlm.nih.gov/mesh?term=",
                            paste0(MeSH[mi:ma], collapse="%20OR%20"), ")")
                    }
                    })
                MeSH = paste(unlist(MeSH), collapse=" ")
            }else{
                MeSH = ""
            }
        }else{
            MeSH = ""
        }
        MeSH
    }

    convertUniProtKB <- function(geneid, GeneInfo){
        if(!is.null(GeneInfo$UniProtKB)){
            UniProtKB <- unique(unlist(GeneInfo$UniProtKB[
                which(GeneInfo$UniProtKB$ENTREZID == geneid),
                "UNIPROT"]))
            UniProtKB <- UniProtKB[which(UniProtKB != "")]
            if(length(UniProtKB) != 0){
                UniProtKB_loc <- div(seq_along(UniProtKB),
                    ceiling(length(UniProtKB) / 100))
                UniProtKB <- lapply(UniProtKB_loc, function(x){
                    mi <- min(x)
                    ma <- max(x)
                    if(ma == 1){
                        paste0("[", mi, "](",
                            "https://www.uniprot.org/uniprot/?query=",
                            paste0(UniProtKB[mi:ma], collapse="+OR+"),
                            "&sort=score)")
                    }else{
                        paste0("[", mi, "-", ma, "](",
                            "https://www.uniprot.org/uniprot/?query=",
                            paste0(UniProtKB[mi:ma], collapse="+OR+"),
                            "&sort=score)")
                    }
                    })
                UniProtKB <- paste(unlist(UniProtKB), collapse=" ")
            }else{
                UniProtKB <- ""
            }
        }else{
            UniProtKB = ""
        }
        UniProtKB
    }

    # taxidでの生物種フィルタリングが必要
    convertSTRING <- function(geneid, GeneInfo, taxid){
        if(!is.null(GeneInfo$ENSP)){
            ENSP <- unique(unlist(GeneInfo$ENSP[
                which(GeneInfo$ENSP$ENTREZID == geneid),
                "ENSEMBLPROT"]))
            ENSP <- ENSP[which(ENSP != "")]
            if(length(ENSP) != 0 && !is.na(taxid)){
                STRING <- paste0("https://string-db.org/network/",
                    taxid, ".", ENSP)
                STRING <- paste(
                    paste0("[", seq_along(STRING), "](", STRING, ")"),
                    collapse=" ")
            }else{
                STRING <- ""
            }
        }else{
            STRING = ""
        }
        STRING
    }

    convertRefEx <- function(geneid, GeneInfo, taxid){
        if(!is.null(GeneInfo$GeneName)){
            genename <- GeneInfo$GeneName[
                which(GeneInfo$GeneName$ENTREZID == geneid),
                "SYMBOL"][1]
            if(length(genename) != 0){
                ref <- "http://refex.dbcls.jp/genelist.php?gene_name%5B%5D="
                if(taxid == "9606"){
                    paste0(ref, genename, "&lang=en&db=human")
                }else if(taxid == "10090"){
                    paste0(ref, genename, "&lang=en&db=mouse")
                }else if(taxid == "10116"){
                    paste0(ref, genename, "&lang=en&db=rat")
                }else{
                    ""
                }
            }else{
                ""
            }
        }else{
            ""
        }
    }

    # taxidでの生物種フィルタリングが必要
    convertEA <- function(geneid, GeneInfo){
        if(!is.null(GeneInfo$ENSG)){
            ENSG <- GeneInfo$ENSG[
                which(GeneInfo$GeneName$ENTREZID == geneid),
                "ENSEMBL"][1]
            if(length(ENSG) != 0){
                paste0("https://www.ebi.ac.uk/gxa/genes/", tolower(ENSG))
            }else{
                ""
            }
        }else{
            ""
        }
    }

    convertSEA <- function(geneid, GeneInfo){
        if(!is.null(GeneInfo$GeneName)){
            genename <- GeneInfo$GeneName[
                which(GeneInfo$GeneName$ENTREZID == geneid),
                "SYMBOL"][1]
            if(length(genename) != 0){
                paste0("https://www.ebi.ac.uk/gxa/sc/search?species=&q=",
                    genename)
            }else{
                ""
            }
        }else{
            ""
        }
    }

    convertSCDB <- function(geneid, GeneInfo, taxid){
        if(!is.null(GeneInfo$GeneName)){
            genename <- GeneInfo$GeneName[
                which(GeneInfo$GeneName$ENTREZID == geneid),
                "SYMBOL"][1]
            if(length(genename) != 0 && taxid == "9606"){
                paste0("https://bioinfo.uth.edu/scrnaseqdb/",
                    "index.php?r=site/rankGene&gene=",
                    genename,
                    "&check=0")
            }else{
                ""
            }
        }else{
            ""
        }
    }

    convertPANGLAO <- function(geneid, GeneInfo, taxid){
        if(!is.null(GeneInfo$GeneName)){
            genename <- GeneInfo$GeneName[
                which(GeneInfo$GeneName$ENTREZID == geneid),
                "SYMBOL"][1]
            if(length(genename) != 0){
                ref <- "https://panglaodb.se/search.html?query="
                if(taxid == "9606"){
                    paste0(ref, genename, "&species=3")
                }else if(taxid == "10090"){
                    paste0(ref, genename, "&species=2")
                }else{
                    ""
                }
            }else{
                ""
            }
        }else{
            ""
        }
    }

    convertCMAP <- function(geneid, GeneInfo, taxid){
        if(!is.null(GeneInfo$GeneName)){
            genename <- GeneInfo$GeneName[
                which(GeneInfo$GeneName$ENTREZID == geneid),
                "SYMBOL"][1]
            if(length(genename) != 0){
                ref <- "https://clue.io/command?q="
                if(taxid == "9606"){
                    paste0(ref, genename)
                }else{
                    ""
                }
            }else{
                ""
            }
        }else{
            ""
        }
    }

    convertPubMed <- function(geneid1, geneid2, lr){
        target <- intersect(
            which(lr$GENEID_L == geneid1),
            which(lr$GENEID_R == geneid2))
        PubMed <- lr$SOURCEID[target]
        LEN <- length(target) != 0
        NUL <- !is.null(lr$SOURCEID[target])
        HYP <- length(grep("-", PubMed)) == 0
        if(LEN && NUL && HYP){
            PubMed <- unique(strsplit(lr$SOURCEID[target], "\\|")[[1]])
            PubMed_loc <- div(seq_along(PubMed),
                ceiling(length(PubMed) / 100))
            PubMed <- lapply(PubMed_loc, function(x){
                mi <- min(x)
                ma <- max(x)
                if(ma == 1){
                    paste0("[", mi, "](",
                        "https://www.ncbi.nlm.nih.gov/pubmed/?term=",
                        paste0(PubMed[mi:ma], collapse="%20OR%20"), ")")
                }else{
                    paste0("[", mi, "-", ma, "](",
                        "https://www.ncbi.nlm.nih.gov/pubmed/?term=",
                        paste0(PubMed[mi:ma], collapse="%20OR%20"), ")")
                }
            })
            PubMed = paste(unlist(PubMed), collapse=" ")
        }else{
            PubMed = ""
        }
        PubMed
    }

    # ranking
    XYZ <- paste0("|", ranking, "|")
    # Gene Name (Ligand)
    GeneName_L <- convertGeneName(ligandGeneID, GeneInfo)
    # Gene Name (Receptor)
    GeneName_R <- convertGeneName(receptorGeneID, GeneInfo)
    # Description (Ligand)
    Description_L <- convertGeneDescription(ligandGeneID, GeneInfo)
    # Description (Receptor)
    Description_R <- convertGeneDescription(receptorGeneID, GeneInfo)
    # Gene Ontology (Ligand)
    GO_L <- convertGeneOntology(ligandGeneID, GeneInfo)
    # Gene Ontology (Receptor)
    GO_R <- convertGeneOntology(receptorGeneID, GeneInfo)
    # Reactome (Ligand)
    Reactome_L <- convertReactome(ligandGeneID, GeneInfo)
    # Reactome (Receptor)
    Reactome_R <- convertReactome(receptorGeneID, GeneInfo)
    # MeSH (Ligand)
    MeSH_L <- convertMeSH(ligandGeneID, GeneInfo)
    # MeSH (Receptor)
    MeSH_R <- convertMeSH(receptorGeneID, GeneInfo)
    # UniProtKB（Ligand）
    UniProtKB_L <- convertUniProtKB(ligandGeneID, GeneInfo)
    # UniProtKB（Receptor）
    UniProtKB_R <- convertUniProtKB(receptorGeneID, GeneInfo)
    # STRING（Ligand）
    STRING_L <- convertSTRING(ligandGeneID, GeneInfo, taxid)
    # STRING（Receptor）
    STRING_R <- convertSTRING(receptorGeneID, GeneInfo, taxid)
    # RefEx（Ligand）
    RefEx_L <- convertRefEx(ligandGeneID, GeneInfo, taxid)
    # RefEx（Receptor）
    RefEx_R <- convertRefEx(receptorGeneID, GeneInfo, taxid)
    # EA（Ligand）
    EA_L <- convertEA(ligandGeneID, GeneInfo)
    # EA（Receptor）
    EA_R <- convertEA(receptorGeneID, GeneInfo)
    # SEA（Ligand）
    SEA_L <- convertSEA(ligandGeneID, GeneInfo)
    # SEA（Receptor）
    SEA_R <- convertSEA(receptorGeneID, GeneInfo)
    # SCDB（Ligand）
    SCDB_L <- convertSCDB(ligandGeneID, GeneInfo, taxid)
    # SCDB（Receptor）
    SCDB_R <- convertSCDB(receptorGeneID, GeneInfo, taxid)
    # PANGLAO（Ligand）
    PANGLAO_L <- convertPANGLAO(ligandGeneID, GeneInfo, taxid)
    # PANGLAO（Receptor）
    PANGLAO_R <- convertPANGLAO(receptorGeneID, GeneInfo, taxid)
    # CMAP（Ligand）
    CMAP_L <- convertCMAP(ligandGeneID, GeneInfo, taxid)
    # CMAP（Receptor）
    CMAP_R <- convertCMAP(receptorGeneID, GeneInfo, taxid)
    # PubMed (L and R)
    PubMed <- convertPubMed(ligandGeneID, receptorGeneID, lr)

    # Embedding
    paste0(XYZ,
        embedLink(GeneName_L, ligandGeneID, Description_L,
            GO_L, Reactome_L, MeSH_L,
            UniProtKB_L, STRING_L, RefEx_L,
            EA_L, SEA_L, SCDB_L, PANGLAO_L, CMAP_L,
            GeneName_R, receptorGeneID, Description_R,
            GO_R, Reactome_R, MeSH_R,
            UniProtKB_R, STRING_R, RefEx_R,
            EA_R, SEA_R, SCDB_R, PANGLAO_R, CMAP_R
            ),
        "![](figures/Ligand/", ligandGeneID, ".png)", "|",
        "![](figures/Receptor/", receptorGeneID, ".png)", "|",
        round(value, 3), " (", round(percentage, 3), "%)",
        "|", round(pvalue, 3),
        "|", round(qvalue, 3),
        "|", evidence,
        "|", PubMed, "|\n")
}

.HCLUST <- function(x){
    out <- hclust(dist(x), method="ward.D")
    cluster <- cutree(out, 2)
    max1 <- max(x[which(cluster == 1)])
    max2 <- max(x[which(cluster == 2)])
    if(max1 > max2){
        cluster[which(cluster == 1)] <- "selected"
        cluster[which(cluster == 2)] <- "not selected"
    }else{
        cluster[which(cluster == 1)] <- "not selected"
        cluster[which(cluster == 2)] <- "selected"
    }
    cluster
}

.OUTLIERS <- function(x, method="grubbs"){
    ranking = length(x) - rank(x) + 1
    vapply(ranking, function(thr){
        remain = which(ranking > thr)
        if(length(remain) > 2){
            if(method == "grubbs"){
                grubbs.test(x[remain])$p
            }else if(method == "chisq"){
                chisq.out.test(x[remain])$p
            }
        }else{
            1
        }
    }, 0.0)
}

.shrink <- function(x){
    block <- strsplit(x , " & ")[[1]]
    l <- length(block)
    nc <- vapply(block, nchar, 0L)
    if(l >= 2){
        out <- block[1]
        for(i in 2:l){
            end <- length(out)
            tmp <- block[i]
            if(nchar(out[end])+nchar(tmp) <= 45){
                out[end] <- paste(c(out[end], tmp), collapse=" & ")
            }else{
                out[end+1] <- tmp
            }
        }
    paste(out, collapse="\n")
    }else{
        x
    }
}

.eachVecLR <- function(x, e){
    p <- e$p
    index <- e$index
    sce <- e$sce
    ah <- e$ah
    .HCLUST <- e$.HCLUST
    .OUTLIERS <- e$.OUTLIERS
    top <- e$top
    GeneInfo <- e$GeneInfo
    out.dir <- e$out.dir
    .smallTwoDplot <- e$.smallTwoDplot
    input <- e$input
    twoD <- e$twoD
    .hyperLinks <- e$.hyperLinks
    LR <- e$LR
    taxid <- e$taxid
    .eachRender <- e$.eachRender
    .XYZ_HEADER1 <- e$.XYZ_HEADER1
    .XYZ_HEADER2 <- e$.XYZ_HEADER2
    .XYZ_HEADER3 <- e$.XYZ_HEADER3
    .XYZ_ENRICH <- e$.XYZ_ENRICH
    out.vecLR <- e$out.vecLR
    algorithm <- e$algorithm
    goenrich <- e$goenrich
    meshenrich <- e$meshenrich
    reactomeenrich <- e$reactomeenrich
    doenrich <- e$doenrich
    ncgenrich <- e$ncgenrich
    dgnenrich <- e$dgnenrich

    # Each LR-Pattern Vector
    if(algorithm == "ntd2"){
        vecLR <- metadata(sce)$sctensor$lrpair@data[x[1], x[2],]
    }
    if(algorithm == "ntd"){
        vecLR <- metadata(sce)$sctensor$lrpair[x, ]
    }
    # Clustering
    ClusterLR <- .HCLUST(vecLR)
    # P-value of Grubbs test
    PvalueLR <- suppressWarnings(.OUTLIERS(vecLR))
    # FDR control by BH method
    QvalueLR <- p.adjust(PvalueLR, "BH")
    # TARGET elements by Clustering
    TARGET <- which(ClusterLR == "selected")
    TARGET <- TARGET[order(vecLR[TARGET], decreasing=TRUE)]
    # TOP elements
    if(top != "full"){
        TOP <- min(top, length(TARGET))
        TARGET <- TARGET[seq_len(TOP)]
    }else{
        TOP <- "full"
    }
    # L-R evidence
    targetL <- unlist(lapply(names(TARGET), function(x){strsplit(x, "_")[[1]][1]}))
    targetR <- unlist(lapply(names(TARGET), function(x){strsplit(x, "_")[[1]][2]}))
    targetL <- unlist(lapply(targetL, function(x){
        which(LR$GENEID_L == x)
    }))
    targetR <- unlist(lapply(targetR, function(x){
        which(LR$GENEID_R == x)
    }))
    target <- intersect(targetL, targetR)
    Evidence <- LR[target, "SOURCEDB"]

    # Enrichment (Too Heavy)
    all <- unique(unlist(strsplit(names(vecLR), "_")))
    sig <- unique(unlist(strsplit(names(TARGET), "_")))
    # GO-Check
    if("GO" %ni% AnnotationDbi::columns(ah)){
        goenrich <- FALSE
    }
    # MeSH-Check
    ah <- AnnotationHub()
    mcah <- mcols(ah)
    mesh_org_msg <- gsub("LRBaseDb", "MeSHDb",
        ah[metadata(sce)$ahid]$title)
    position <- regexpr("v[0-9][0-9][0-9]", mesh_org_msg)
    mesh_version <- substr(mesh_org_msg, position, position + 3)
    mesh_db_msg <- paste0("MeSHDb for MeSH.db (", mesh_version, ")")
    ahid1 <- mcah@rownames[which(mcah@listData$title == mesh_org_msg)]
    ahid2 <- mcah@rownames[which(mcah@listData$title == mesh_db_msg)]
    if(length(ahid1) != 0){
        message(paste0("Related MeSH IDs are retrieved from ",
            "AnnotationHub..."))
        e$meshannotation <- MeSHDbi::MeSHDb(ah[[ahid1]])
        e$meshdb <- MeSHDbi::MeSHDb(ah[[ahid2]])
    }else{
        meshenrich <- FALSE
        e$meshannotation <- NA
        e$meshdb <- NA
    }
    # Reactome-Check
    reactomespc <- .REACTOMESPC_taxid[[taxid]]
    if(is.null(reactomespc)){
        reactomeenrich <- FALSE
    }
    # DO/NCG/DGN-Check
    if(taxid != "9606"){
        doenrich <- FALSE
        ncgenrich <- FALSE
        dgnenrich <- FALSE
    }
    Enrich <- suppressWarnings(.ENRICHMENT(all, sig,
        e, reactomespc, goenrich, meshenrich,
        reactomeenrich, doenrich, ncgenrich, dgnenrich, p, ah))

    # Eigen Value
    # Each LR-Pattern Vector
    if(algorithm == "ntd2"){
        Value <- metadata(sce)$sctensor$lrpair@data[x[1], x[2], TARGET]
        Percentage <- Value / sum(metadata(sce)$sctensor$lrpair@data[x[1], x[2], ]) * 100
    }
    if(algorithm == "ntd"){
        Value <- metadata(sce)$sctensor$lrpair[x, TARGET]
        Percentage <- Value / sum(metadata(sce)$sctensor$lrpair[x, ]) * 100
    }
    # Hyper Link (Too Heavy)
    cat("Hyper-links are embedded...\n")
    if(!is.null(GeneInfo$GeneName)){
        GeneName <- GeneInfo$GeneName[, c("SYMBOL", "ENTREZID")]
    }else{
        GeneName <- rep(FALSE, length=length(TARGET))
    }
    LINKS <- vapply(seq_along(TARGET), function(xx){
        # IDs
        Ranking <- xx
        L_R <- strsplit(names(TARGET[xx]), "_")
        LigandGeneID <- L_R[[1]][1]
        ReceptorGeneID <- L_R[[1]][2]
        if(!is.null(GeneInfo$GeneName)){
            LigandGeneName <- GeneName[which(GeneName[,2] == LigandGeneID), 1]
            ReceptorGeneName <- GeneName[which(GeneName[,2] == ReceptorGeneID), 1]

            if(length(LigandGeneName) == 0 || LigandGeneName %in% c("", NA)){
                LigandGeneName <- LigandGeneID
            }
            if(length(ReceptorGeneName) == 0 || ReceptorGeneName %in% c("", NA)){
                ReceptorGeneName <- ReceptorGeneID
            }
        }else{
            LigandGeneName <- LigandGeneID
            ReceptorGeneName <- ReceptorGeneID
        }
        # Return Hyper links
        .hyperLinks(Ranking, LigandGeneID,
        ReceptorGeneID, LR, taxid, Value[xx],
        Percentage[xx],
        GeneInfo, PvalueLR[xx],
        QvalueLR[xx], Evidence[xx])
    }, "")
    LINKS <- paste(LINKS, collapse="")

    # Output object
    list(
        ClusterLR=ClusterLR,
        PvalueLR=PvalueLR,
        QvalueLR=QvalueLR,
        TARGET=TARGET,
        TOP=TOP,
        Enrich=Enrich,
        Value=Value,
        Percentage=Percentage,
        LINKS=LINKS
    )
}

.genePlot <- function(sce, assayNames, input, out.dir, GeneInfo, LR){
    GeneName <- GeneInfo$GeneName
    LigandGeneID <- unique(LR$GENEID_L)
    ReceptorGeneID <- unique(LR$GENEID_R)
    if(!is.null(GeneName)){
        LigandGeneName <- vapply(LigandGeneID, function(x){
            GeneName[which(GeneName$ENTREZID == x)[1], "SYMBOL"]}, "")
        ReceptorGeneName <- vapply(ReceptorGeneID, function(x){
            GeneName[which(GeneName$ENTREZID == x)[1], "SYMBOL"]}, "")
    }else{
        LigandGeneName <- LigandGeneID
        ReceptorGeneName <- ReceptorGeneID
    }

    # Plot (Ligand)
    lapply(seq_along(LigandGeneID), function(x){
        Ligandfile <- paste0(out.dir, "/figures/Ligand/",
            LigandGeneID[x], ".png")
        target <- which(rownames(input) == LigandGeneID[x])
        if(length(target) != 0){
            g <- .smallTwoDplot(sce, assayNames, LigandGeneID[x],
                LigandGeneName[x], "reds")
            ggsave(Ligandfile, plot=g, dpi=200, width=6, height=6)
        }
    })

    # Plot (Receptor)
    lapply(seq_along(ReceptorGeneID), function(x){
        Receptorfile <- paste0(out.dir, "/figures/Receptor/",
            ReceptorGeneID[x], ".png")
        target <- which(rownames(input) == ReceptorGeneID[x])
        if(length(target) != 0){
            g <- .smallTwoDplot(sce, assayNames, ReceptorGeneID[x],
                ReceptorGeneName[x], "blues")
            ggsave(Receptorfile, plot=g, dpi=200, width=6, height=6)
        }
    })
}

.ligandPatternPlot <- function(numLPattern, celltypes, sce, col.ligand, ClusterL, out.dir, twoD){
    vapply(seq_len(numLPattern), function(i){
        label.ligand <- unlist(vapply(names(celltypes), function(x){
                metadata(sce)$sctensor$ligand[paste0("Dim", i), x]}, 0.0))
        label.ligand[] <- smoothPalette(label.ligand,
            palfunc=colorRampPalette(col.ligand, alpha=TRUE))
        LPatternfile <- paste0(out.dir, "/figures/Pattern_", i, "__", ".png")
        png(filename=LPatternfile, width=1000, height=1000, bg="transparent")
        par(ps=20)
        plot(twoD, col=label.ligand, pch=16, cex=2, bty="n",
            xaxt="n", yaxt="n", xlab="", ylab="",
            main="")
        dev.off()
    }, 0L)
}

.receptorPatternPlot <- function(numRPattern, celltypes, sce, col.receptor, ClusterR, out.dir, twoD){
    vapply(seq_len(numRPattern), function(i){
        label.receptor <- unlist(vapply(names(celltypes), function(x){
                metadata(sce)$sctensor$receptor[paste0("Dim", i), x]}, 0.0))
        label.receptor[] <- smoothPalette(label.receptor,
            palfunc=colorRampPalette(col.receptor, alpha=TRUE))
        RPatternfile = paste0(out.dir, "/figures/Pattern__", i, "_", ".png")
        png(filename=RPatternfile, width=1000, height=1000, bg="transparent")
        par(ps=20)
        plot(twoD, col=label.receptor, pch=16, cex=2, bty="n",
            xaxt="n", yaxt="n", xlab="", ylab="",
            main="")
        dev.off()
    }, 0L)
}

.GOENRICHMENT <- function(all, sig, ah, category, p){
    goParams <- try(new("GOHyperGParams",
        geneIds=sig,
        universeGeneIds=all,
        annotation=ah,
        ontology=category,
        pvalueCutoff=p,
        conditional=FALSE,
        testDirection="over"), silent=TRUE)
    if(class(goParams) != "try-error"){
        # Hyper geometric p-value
        out <- try(summary(hyperGTest(goParams)), silent=TRUE)
        if(is(out)[1] == "try-error"){
            list(Term=NULL, Pvalue=NULL)
        }else{
            list(Term=out$Term, Pvalue=out$Pvalue)
        }        
    }else{
        list(Term=NULL, Pvalue=NULL)        
    }
}

.MeSHENRICHMENT <- function(all, sig, e, category, p){
    if(class(e$meshannotation) != "MeSHDb"){
        list(Term=NULL, Pvalue=NULL)
    }else{
        meshParams <- new("MeSHHyperGParams",
            geneIds = sig,
            universeGeneIds = all,
            annotation = "e$meshannotation",
            meshdb = "e$meshdb",
            category = category,
            database = "gene2pubmed",
            pvalueCutoff = p,
            pAdjust = "none")
        # Hyper geometric p-value
        out <- try(summary(meshHyperGTest(meshParams)), silent=TRUE)
        if(is(out)[1] == "try-error"){
            list(Term=NULL, Pvalue=NULL)
        }else{
            outTerm <- unique(out$MESHTERM)
            outPvalue <- sapply(outTerm, function(x){
                out$Pvalue[which(out$MESHTERM == x)[1]]
            })
            list(Term=outTerm, Pvalue=outPvalue)
        }
    }
}

.ReactomeENRICHMENT <- function(all, sig, reactomespc, p){
    if(is.na(reactomespc)){
        list(Term=NULL, Pvalue=NULL)
    }else{
        out <- try(enrichPathway(gene=sig,
          organism=reactomespc,
          pvalueCutoff=p, readable=TRUE), silent=TRUE)
        if(is(out)[1] == "try-error"){
            list(Term=NULL, Pvalue=NULL)
        }else if(is.null(out)){
            list(Term=NULL, Pvalue=NULL)
        }else{
            list(Term=out@result$Description,
                Pvalue=out@result$pvalue)
        }
    }
}

.DOENRICHMENT <- function(all, sig, p){
    out <- try(enrichDO(gene=sig,
      pvalueCutoff=p, readable=TRUE), silent=TRUE)
    if(is(out)[1] == "try-error"){
        list(Term=NULL, Pvalue=NULL)
    }else if(is.null(out)){
        list(Term=NULL, Pvalue=NULL)
    }else{
        list(Term=out@result$Description,
            Pvalue=out@result$pvalue)
    }
}

.NCGENRICHMENT <- function(all, sig, p){
    out <- try(enrichNCG(gene=sig,
      pvalueCutoff=p, readable=TRUE), silent=TRUE)
    if(is(out)[1] == "try-error"){
        list(Term=NULL, Pvalue=NULL)
    }else if(is.null(out)){
        list(Term=NULL, Pvalue=NULL)
    }else{
        list(Term=out@result$Description,
            Pvalue=out@result$pvalue)
    }
}

.DGNENRICHMENT <- function(all, sig, p){
    out <- try(enrichDGN(gene=sig,
      pvalueCutoff=p, readable=TRUE), silent=TRUE)
    if(is(out)[1] == "try-error"){
        list(Term=NULL, Pvalue=NULL)
    }else if(is.null(out)){
        list(Term=NULL, Pvalue=NULL)
    }else{
        list(Term=out@result$Description,
            Pvalue=out@result$pvalue)
    }
}

.NOSIG <- list(Term=NULL, PValue=NULL)

.ENRICHMENT <- function(all, sig, e,
    reactomespc, goenrich, meshenrich, reactomeenrich,
    doenrich, ncgenrich, dgnenrich, p, ah){
    # GO
    if(goenrich){
        cat("GO-Enrichment Analysis is running...(1/3)\n")
        BP <- .GOENRICHMENT(all, sig, ah, "BP", p)
        cat("GO-Enrichment Analysis is running...(2/3)\n")
        MF <- .GOENRICHMENT(all, sig, ah, "MF", p)
        cat("GO-Enrichment Analysis is running...(3/3)\n")
        CC <- .GOENRICHMENT(all, sig, ah, "CC", p)
    }else{
        BP <- .NOSIG
        MF <- .NOSIG
        CC <- .NOSIG
    }
    # MeSH
    if(meshenrich){
        cat("MeSH-Enrichment Analysis is running...(1/16)\n")
        A <- .MeSHENRICHMENT(all, sig, e, "A", p)
        cat("MeSH-Enrichment Analysis is running...(2/16)\n")
        B <- .MeSHENRICHMENT(all, sig, e, "B", p)
        cat("MeSH-Enrichment Analysis is running...(3/16)\n")
        C <- .MeSHENRICHMENT(all, sig, e, "C", p)
        cat("MeSH-Enrichment Analysis is running...(4/16)\n")
        D <- .MeSHENRICHMENT(all, sig, e, "D", p)
        cat("MeSH-Enrichment Analysis is running...(5/16)\n")
        E <- .MeSHENRICHMENT(all, sig, e, "E", p)
        cat("MeSH-Enrichment Analysis is running...(6/16)\n")
        F <- .MeSHENRICHMENT(all, sig, e, "F", p)
        cat("MeSH-Enrichment Analysis is running...(7/16)\n")
        G <- .MeSHENRICHMENT(all, sig, e, "G", p)
        cat("MeSH-Enrichment Analysis is running...(8/16)\n")
        H <- .MeSHENRICHMENT(all, sig, e, "H", p)
        cat("MeSH-Enrichment Analysis is running...(9/16)\n")
        I <- .MeSHENRICHMENT(all, sig, e, "I", p)
        cat("MeSH-Enrichment Analysis is running...(10/16)\n")
        J <- .MeSHENRICHMENT(all, sig, e, "J", p)
        cat("MeSH-Enrichment Analysis is running...(11/16)\n")
        K <- .MeSHENRICHMENT(all, sig, e, "K", p)
        cat("MeSH-Enrichment Analysis is running...(12/16)\n")
        L <- .MeSHENRICHMENT(all, sig, e, "L", p)
        cat("MeSH-Enrichment Analysis is running...(13/16)\n")
        M <- .MeSHENRICHMENT(all, sig, e, "M", p)
        cat("MeSH-Enrichment Analysis is running...(14/16)\n")
        N <- .MeSHENRICHMENT(all, sig, e, "N", p)
        cat("MeSH-Enrichment Analysis is running...(15/16)\n")
        V <- .MeSHENRICHMENT(all, sig, e, "V", p)
        cat("MeSH-Enrichment Analysis is running...(16/16)\n")
        Z <- .MeSHENRICHMENT(all, sig, e, "Z", p)
    }else{
        A <- .NOSIG
        B <- .NOSIG
        C <- .NOSIG
        D <- .NOSIG
        E <- .NOSIG
        F <- .NOSIG
        G <- .NOSIG
        H <- .NOSIG
        I <- .NOSIG
        J <- .NOSIG
        K <- .NOSIG
        L <- .NOSIG
        M <- .NOSIG
        N <- .NOSIG
        V <- .NOSIG
        Z <- .NOSIG
    }
    # Reactome
    if(reactomeenrich){
        cat("Reactome-Enrichment Analysis is running...(1/1)\n")
        Reactome <- .ReactomeENRICHMENT(all, sig, reactomespc, p)
    }else{
        Reactome <- .NOSIG
    }
    # DO
    if(doenrich){
        cat("DO-Enrichment Analysis is running...(1/1)\n")
        DO <- .DOENRICHMENT(all, sig, p)
    }else{
        DO <- .NOSIG
    }
    # NCG
    if(ncgenrich){
        cat("NCG-Enrichment Analysis is running...(1/1)\n")
        NCG <- .NCGENRICHMENT(all, sig, p)
    }else{
        NCG <- .NOSIG
    }
    # DGN
    if(dgnenrich){
        cat("DGN-Enrichment Analysis is running...(1/1)\n")
        DGN <- .DGNENRICHMENT(all, sig, p)
    }else{
        DGN <- .NOSIG
    }

    # Output
    out <- list(BP, MF, CC,
            A, B, C, D, E, F, G, H, I, J, K, L, M, N, V, Z,
            Reactome, DO, NCG, DGN)
    # Exception
    out <- lapply(out, function(x){
            if(length(x$Term) == 0 || length(x$Pvalue) == 0){
                .NOSIG
            }else{
                x
            }
        })
    names(out) <- c(
        "GO_BP", "GO_MF", "GO_CC",
        "MeSH_A", "MeSH_B", "MeSH_C", "MeSH_D",
        "MeSH_E", "MeSH_F", "MeSH_G", "MeSH_H",
        "MeSH_I", "MeSH_J", "MeSH_K", "MeSH_L",
        "MeSH_M", "MeSH_N", "MeSH_V", "MeSH_Z",
        "Reactome", "DO", "NCG", "DGN"
        )
    out
}

.GenerateFC <- function (x, thr){
    if (thr == "E1") {
        a <- 0.3213536
        b <- 0.1211649
    }
    else if (thr == "E2") {
        a <- 0.7019536
        b <- 0.3638012
    }
    else if (thr == "E5") {
        a <- 1.9079161
        b <- 0.6665197
    }
    else if (thr == "E10") {
        a <- 4.4291731
        b <- 0.8141489
    }
    else if (thr == "E50") {
        a <- 21.430831
        b <- 1.161132
    }
    else if (thr == "E100") {
        a <- 30.733509
        b <- 1.131347
    }
    else if (is.numeric(thr)) {
        stop("Wrong thr!!!")
    }
    10^(a * exp(-b * log10(x + 1)))
}


# Random Matrix
.matRnbinom <- function(m, disp, rn.index, num.Cell){
    t(vapply(rn.index, function(x) {
        rnbinom(n = num.Cell, mu = m[x], size = 1/disp[x])
    }, 1.0*seq_len(num.Cell)))
}

# Setting DEG
.matFC <- function(nDEG, num.Gene, num.Cell, CCI, m, row.index, rn.index){
    # Set FC
    fc.matrix <- matrix(1, nrow=num.Gene, ncol=length(num.Cell))
    for(x in seq_along(CCI)){
        lp <- which(CCI[[x]]$LPattern == 1)
        r <- row.index[[x]]
        fc.matrix[r, lp] <- .GenerateFC(m[rn.index][r], CCI[[x]]$fc)
    }
    for(x in seq_along(CCI)){
        lp <- which(CCI[[x]]$RPattern == 1)
        r <- row.index[[x]] + sum(nDEG)
        fc.matrix[r, lp] <- .GenerateFC(m[rn.index][r], CCI[[x]]$fc)
    }
    fc.matrix
}

# Assign DEG
.setDEG <- function(original.matrix, fc.matrix, num.Cell, rn.index, row.index, m, disp){
    for (i in seq_along(num.Cell)) {
        deg.index <- which(fc.matrix[, i] != 1)
        col.index <- sum(num.Cell[1:i - 1]) + 1:sum(num.Cell[i])
        if(length(deg.index) != 0){
            original.matrix[deg.index, col.index] <- t(sapply(deg.index,
                function(x){
                    rnbinom(n = num.Cell[i],
                        mu = fc.matrix[x, i] * m[rn.index[x]],
                        size = 1/disp[rn.index[x]])
            }))
        }
    }
    original.matrix
}

# Dropout Matrix
.matDrop <- function(original.matrix, lambda, nCell){
    mean.vector <- apply(original.matrix, 1, mean)
    var.vector <- apply(original.matrix, 1, var)
    droprate <- exp(-lambda * mean.vector^2)
    droprate.matrix <- vapply(seq_len(sum(nCell)), function(y){
        unlist(lapply(droprate, function(x){
            rbinom(1, 1, prob = (1 - x))
        }))
    }, seq_len(nrow(original.matrix)))
}

.simulateDropoutCounts <- function(nGene, nCell, cciInfo, lambda, seed){
    # Set Parameters
    CCI <- lapply(grep("CCI", names(cciInfo)), function(x){
        cciInfo[[x]]
    })
    LPattern <- lapply(grep("CCI", names(cciInfo)), function(x){
        cciInfo[[x]]$LPattern
    })
    RPattern <- lapply(grep("CCI", names(cciInfo)), function(x){
        cciInfo[[x]]$RPattern
    })
    nDEG <- unlist(lapply(grep("CCI", names(cciInfo)), function(x){
        cciInfo[[x]]$nGene
    }))
    fc <- unlist(lapply(grep("CCI", names(cciInfo)), function(x){
        cciInfo[[x]]$fc
    }))
    set.seed(seed)
    data("m")
    data("v")
    disp <- (v - m)/m^2
    rn.index <- sample(which(disp > 0), nGene, replace = TRUE)
    row.index <- list()
    start <- 1
    for(i in seq_along(nDEG)){
        if(i == 1){
            row.index[[i]] <- 1:nDEG[i]
        }else{
            row.index[[i]] <- start:(start+nDEG[i]-1)
        }
        start <- start + nDEG[i]
    }

    # Check
    if(sum(nDEG) > cciInfo$nPair){
        stop("Please specify larger cciInfo$nPair!")
    }
    if(length(nCell) != length(cciInfo$CCI1$LPattern)){
        stop(paste0("Please specify the length of nCell is",
            " same as cciInfo$CCI*$LPattern ",
            "and cciInfo$CCI*$RPattern!"))
    }

    # Original matrix
    original.matrix <- .matRnbinom(m, disp, rn.index, sum(nCell))

    # Setting DEG
    fc.matrix <- .matFC(nDEG, nGene, nCell, CCI, m, row.index, rn.index)
    original.matrix <- .setDEG(original.matrix, fc.matrix,
        nCell, rn.index, row.index, m, disp)

    # Setting Dropout
    droprate.matrix <- .matDrop(original.matrix, lambda, nCell)
    testdata.matrix <- original.matrix * droprate.matrix

    # Naming
    rownames(testdata.matrix) <- paste0("Gene", seq_len(nrow(testdata.matrix)))
    colnames(testdata.matrix) <- paste0("Cell", seq_len(ncol(testdata.matrix)))
    celltypes <- colnames(testdata.matrix)
    names(celltypes) <- unlist(lapply(seq_along(nCell), function(x){
        paste0("Celltype", rep(x, length=nCell[x]))
    }))

    # L-R list
    rn <- rownames(testdata.matrix)
    start1 <- sum(nDEG)+1
    end1 <- 2*sum(nDEG)
    start2 <- end1 + 1
    end2 <- end1 + cciInfo$nPair - end1/2
    start3 <- end2 + 1
    end3 <- end2 + cciInfo$nPair - end1/2
    LR <- rbind(
              cbind(rn[seq_len(sum(nDEG))], rn[start1:end1]),
              cbind(rn[start2:end2], rn[start3:end3]))
    LR <- as.data.frame(LR)
    colnames(LR) <- c("GENEID_L", "GENEID_R")

    # L-R <-> CCI relationship
    LR_CCI <- seq_len(cciInfo$nPair)
    names(LR_CCI) <- c(
        unlist(lapply(seq_along(nDEG), function(x){
            rep(paste0("CCI", x), nDEG[x])
        })),
        rep("nonDEG", cciInfo$nPair - sum(nDEG)))
    rownames(LR) <- LR_CCI

    # Set random seed as default mode
    set.seed(NULL)

    # Output
    return(
        list(simcount = testdata.matrix,
        LR=LR,
        celltypes=celltypes,
        LR_CCI=LR_CCI))
}

newCCSParams <- function() {
    new("CCSParams")
}

setMethod("getParam", "CCSParams", function(object, name){
    slot(object, name)
})
setMethod("setParam<-", "CCSParams", function(object, name, value){
    slot(object, name) <- value
    validObject(object)
    return(object)
})
setMethod("show", "CCSParams", function(object){
    cat("A", crayon::bold("CCSParams"), "object of class",
        crayon::bold(class(object)), "\n")
})

context("CCSParamsFunctions")

# newCCSParams
params <- newCCSParams()
expect_true(class(params) =="CCSParams")

# getParam
expect_true(is.numeric(getParam(params, "nGene")))
expect_true(is.numeric(getParam(params, "nCell")))
expect_true(is.list(getParam(params, "cciInfo")))
expect_true(is.numeric(getParam(params, "lambda")))
expect_true(is.numeric(getParam(params, "seed")))

# setParam
setParam(params, "nGene") <- 20000
expect_true(identical(getParam(params, "nGene"), 20000))

setParam(params, "nCell") <- c(12, 43, 323)
expect_true(identical(getParam(params, "nCell"), c(12, 43, 323)))

setParam(params, "cciInfo") <- list(nPair=2000,
                                   CCI1=list(
                                       LPattern=c(1,0,0),
                                       RPattern=c(0,1,1),
                                       nGene=200,
                                       fc="E10"),
                                   CCI2=list(
                                       LPattern=c(0,0,1),
                                       RPattern=c(1,1,1),
                                       nGene=200,
                                       fc="E10"),
                                   CCI3=list(
                                       LPattern=c(1,1,1),
                                       RPattern=c(1,0,1),
                                       nGene=200,
                                       fc="E10")
                                   )
expect_true(identical(getParam(params, "cciInfo"),
    list(nPair=2000,
       CCI1=list(
           LPattern=c(1,0,0),
           RPattern=c(0,1,1),
           nGene=200,
           fc="E10"),
       CCI2=list(
           LPattern=c(0,0,1),
           RPattern=c(1,1,1),
           nGene=200,
           fc="E10"),
       CCI3=list(
           LPattern=c(1,1,1),
           RPattern=c(1,0,1),
           nGene=200,
           fc="E10")
       )))

setParam(params, "lambda") <- 0.1
expect_true(identical(getParam(params, "lambda"), 0.1))

setParam(params, "seed") <- 123
expect_true(identical(getParam(params, "seed"), 123))

library("testthat")
library("scTensor")

options(testthat.use_colours = FALSE)

test_file("testthat/test_GermMale.R")
test_file("testthat/test_tsneGermMale.R")
test_file("testthat/test_labelGermMale.R")
test_file("testthat/test_cellCellFunctions.R")
test_file("testthat/test_CCSParamsFunctions.R")

# Header Row
.XYZ_HEADER1 <- function(index, i){
    TITLE <- paste0("Details of (",
    paste(index[i, seq_len(3)], collapse=","),
    ") Pattern")
    HEADER <- paste0("---\n",
    "title: <font color='#87b13f'>XXXXX</font>\n",
    "output:\n",
    "    html_document:\n",
    "        toc: true\n",
    "        toc_float: true\n",
    "        toc_depth: 2\n",
    "---\n",
    "# <font color='#1881c2'>(", paste(index[i, seq_len(2)], collapse=","),
    ",:) Pattern</font>\n\n",
    "![](figures/CCIHypergraph_",
    paste(index[i, seq_len(2)], collapse="_"), ".png)\n\n")
    sub("XXXXX", TITLE, HEADER)
}

.XYZ_HEADER1_2 <- function(index, i){
    TITLE <- paste0("Details of (",
    paste(c(index[i, seq_len(2)], ""), collapse=","),
    ") -related L-R pairs")
    HEADER <- paste0("---\n",
    "title: <font color='#87b13f'>XXXXX</font>\n",
    "output:\n",
    "    html_document:\n",
    "        toc: true\n",
    "        toc_float: true\n",
    "        toc_depth: 2\n",
    "---\n",
    "# <font color='#1881c2'>(", paste(index[i, seq_len(2)], collapse=","),
    ",:) Pattern</font>\n\n",
    "![](figures/CCIHypergraph_",
    paste(index[i, seq_len(2)], collapse="_"), ".png)\n\n")
    sub("XXXXX", TITLE, HEADER)
}

.XYZ_HEADER2 <- function(i, x, top){
    paste0("# <font color='#1881c2'>(\\*,\\*,", i, ") Pattern",
    " (Top", top, " LR-pairs)</font>\n\n",
    "```{r}\n", # Top
    "library(\"scTensor\")\n",
    "load(\"reanalysis.RData\")\n",
    "col <- .setColor(\"many\")[", x, "]\n",
    "scTensor:::.myvisNetwork(g, col)\n",
    "```\n\n", # Bottom
    "|Rank|Ligand Gene|Receptor Gene|",
    "Ligand Expression (Log10(exp + 1))|",
    "Receptor Expression (Log10(exp + 1))|",
    "LR-pair factor value (Percentage)|",
    "P-value (Grubbs test)|",
    "Q-value (BH method)|",
    "L-R Evidence cf. [rikenbit/lrbase-workflow](https://github.com/rikenbit/lrbase-workflow)|",
    "PubMed|\n",
    "|----|----|----|----|----|----|----|----|----|----|\n")
}

.XYZ_HEADER2_2 <- function(index, i, top){
    paste0("# <font color='#1881c2'>(",
    paste(c(index[i, seq_len(2)], ""), collapse=","),
    ") -related L-R pairs (Top", top, " pairs)</font>\n\n",
    "```{r}\n", # Top
    "library(\"scTensor\")\n",
    "load(\"reanalysis.RData\")\n",
    "col <- .setColor(\"many\")[", i, "]\n",
    "scTensor:::.myvisNetwork(g, col)\n",
    "```\n\n", # Bottom
    "|Rank|Ligand Gene|Receptor Gene|",
    "Ligand Expression (Log10(exp + 1))|",
    "Receptor Expression (Log10(exp + 1))|",
    "LR-pair factor value (Percentage)|",
    "P-value (Grubbs test)|",
    "Q-value (BH method)|",
    "L-R Evidence cf. [rikenbit/lrbase-workflow](https://github.com/rikenbit/lrbase-workflow)|",
    "PubMed|\n",
    "|----|----|----|----|----|----|----|----|----|----|\n")
}

.MAINHEADER <- function(author, title){
    HEADER <- paste0("---\ntitle: XXXXX\n",
        "author: YYYYY\ndate:",
        " \"`r Sys.time()`\"\n",
        "output:\n",
        " BiocStyle::html_document:\n",
        "  toc_float: true\n",
        "vignette: >\n ",
        "%\\VignetteIndexEntry{Vignette Title}\n ",
        "%\\VignetteEngine{knitr::rmarkdown}\n ",
        "%\\VignetteEncoding{UTF-8}\n---\n")
    sub("YYYYY", author, sub("XXXXX", title, HEADER))
}

# 1. About Dataset and scTensor Algorithm
.BODY1 <- paste0("\n\n# About scTensor Algorithm\n\n",
    "![](Workflow.png)\n",
    "[scTensor](https://bioconductor.org/packages/release/",
    "bioc/html/scTensor.html) is the R/Bioconductor package",
    " for visualization of cell-cell interaction within ",
    "single-cell RNA-Seq data. ",
    "The calculation consists of several steps.\n\n",
    "Firstly, [LRBase.XXX.eg.db](https://bioconductor.org/",
    "packages/release/bioc/html/LRBase.Hsa.eg.db.html)-type",
    " package is loaded for retriving ligand-receptor gene ",
    "relationship (XXX is the abbreviation of some organisms ",
    "like \"Hsa\" as Homo sapiens). ",
    "scTensor searches the corresponding pair of genes in the ",
    "rownames of input data matrix and extracted as vector. ",
    "In this step, the gene identifier is limited as [NCBI ",
    "Gene ID](https://www.ncbi.nlm.nih.gov/gene) for now.\n\n",
    "Next, the outer product of two vectors are calculated ",
    "and summarized as a matrix. ",
    "Here, the multiple matrices can be represented as a three-order",
    " \"tensor\" (Ligand-Cell * Receptor-Cell * LR-Pair). ",
    "scTensor decomposes the tensor into a small tensor (core tensor)",
    " and three factor matrices. ",
    "Tensor decomposition is very similar to the matrix decomposition",
    " like PCA (principal component analysis). ",
    "The core tensor is similar to eigenvalue of PCA and means how",
    " much the pattern is outstanding. ",
    "Likewise, three matrices is similar to the PC scores and ",
    "loadings of PCA and represents which ligand-cell/receptor-cell",
    "/LR-pair are informative. ",
    "When the matrices have negative values, distinguishing that ",
    "which direction (+/-) is important and which is not, is ",
    "difficult to interpret and laboring task. ",
    "That's why, scTensor performs non-negative Tucker ",
    "decomposition (NTD), which is non-negative version ",
    "of Tucker decomposition (c.f. [nnTensor]",
    "(https://cran.r-project.org/package=nnTensor)). \n\n",
    "Finaly, the result of NTD is summarized as this report. ",
    "Most of plots belows are visualized by ",
    "[plotly](https://plot.ly/r/) package and interactively ",
    "search the presise information of the plot. ",
    "The three factor matrices can be interactively viewed ",
    "and which celltypes are responds to the cell-cell interaction.",
    "The mode-3 (LR-pair) direction of sum of the core tensor ",
    "is calculated and visualized as Ligand-Receptor Patterns. ",
    "Detail of (Ligand-Cell, Receptor-Cell, LR-pair) Patterns ",
    "are also visualized as below.\n\n",
    "For more detail, visit the [vignette](https://bioconductor.org",
    "/packages/devel/bioc/vignettes/scTensor/inst/doc/scTensor.html)",
    " of [scTensor](https://bioconductor.org/",
    "packages/release/bioc/html/scTensor.html)"
    )

.BODY1_2 <- paste0("\n\n# About scTensor Algorithm\n\n",
    "![](Workflow_2.png)\n",
    "[scTensor](https://bioconductor.org/packages/release/",
    "bioc/html/scTensor.html) is the R/Bioconductor package",
    " for visualization of cell-cell interaction within ",
    "single-cell RNA-Seq data. ",
    "The calculation consists of several steps.\n\n",
    "Firstly, [LRBase.XXX.eg.db](https://bioconductor.org/",
    "packages/release/bioc/html/LRBase.Hsa.eg.db.html)-type",
    " package is loaded for retriving ligand-receptor gene ",
    "relationship (XXX is the abbreviation of some organisms ",
    "like \"Hsa\" as Homo sapiens). ",
    "scTensor searches the corresponding pair of genes in the ",
    "rownames of input data matrix and extracted as vector. ",
    "In this step, the gene identifier is limited as [NCBI ",
    "Gene ID](https://www.ncbi.nlm.nih.gov/gene) for now.\n\n",
    "Next, the all elements of extracted two vectors are summed ",
    "up with all possible combination (Kronecker sum) and summarized",
    " as a matrix. ",
    "Here, the multiple matrices can be represented as a three-order",
    " \"tensor\" (Ligand-Cell * Receptor-Cell * LR-Pair). ",
    "scTensor decomposes the tensor into a small tensor (core tensor)",
    " and three factor matrices. ",
    "Tensor decomposition is very similar to the matrix decomposition",
    " like PCA (principal component analysis). ",
    "The core tensor is similar to eigenvalue of PCA and means how",
    " much the pattern is outstanding. ",
    "Likewise, three matrices is similar to the PC scores and ",
    "loadings of PCA and represents which ligand-cell/receptor-cell",
    "/LR-pair are informative. ",
    "When the matrices have negative values, distinguishing that ",
    "which direction (+/-) is important and which is not, is ",
    "difficult to interpret and laboring task. ",
    "That's why, scTensor performs non-negative Tucker ",
    "decomposition (NTD), which is non-negative version ",
    "of Tucker decomposition (c.f. [nnTensor]",
    "(https://cran.r-project.org/package=nnTensor)). \n\n",
    "Finaly, the result of NTD is summarized as this report. ",
    "Most of plots belows are visualized by ",
    "[plotly](https://plot.ly/r/) package and interactively ",
    "search the presise information of the plot. ",
    "The three factor matrices can be interactively viewed ",
    "and which celltypes are responds to the cell-cell interaction.",
    "The mode-3 (LR-pair) direction of sum of the core tensor ",
    "is calculated and visualized as Ligand-Receptor Patterns. ",
    "Detail of (Ligand-Cell, Receptor-Cell, LR-pair) Patterns ",
    "are also visualized as below.\n\n",
    "For more detail, visit the [vignette](https://bioconductor.org",
    "/packages/devel/bioc/vignettes/scTensor/inst/doc/scTensor.html)",
    " of [scTensor](https://bioconductor.org/",
    "packages/release/bioc/html/scTensor.html)"
    )

# 2. Global statistics and plots
.BODY2 <- paste0("\n\n# Global statistics and plots\n\n",
    "The result of scTensor is saved as a R binary file",
    " (reanalysis.RData).\n",
    "```{r}\n", # Top
    "load(\"reanalysis.RData\")\n\n",
    "# SingleCellExperiment object\n",
    "sce\n",
    "# Data size\n",
    "metadata(sce)$datasize\n",
    "# Reduced data size\n",
    "metadata(sce)$ranks\n",
    "# Reconstruction Error of NTD\n",
    "head(metadata(sce)$recerror)\n",
    "tail(metadata(sce)$recerror)\n",
    "# Relative Change of NTD\n",
    "head(metadata(sce)$relchange)\n",
    "tail(metadata(sce)$relchange)\n",
    "# Gene expression matrix\n",
    "is(input)\n",
    "dim(input)\n",
    "input[seq_len(2), seq_len(2)]\n",
    "# The result of 2D dimensional reduction (e.g. t-SNE)\n",
    "is(twoD)\n",
    "dim(twoD)\n",
    "head(twoD)\n",
    "# Ligand-Receptor corresponding table",
    " extracted from LRBase.XXX.eg.db\n",
    "is(LR)\n",
    "dim(LR)\n",
    "head(LR, 2)\n",
    "# Celltype label and color scheme\n",
    "is(celltypes)\n",
    "length(celltypes)\n",
    "head(celltypes)\n",
    "# Core tensor values\n",
    "is(index)\n",
    "dim(index)\n",
    "head(index)\n",
    "is(corevalue)\n",
    "length(corevalue)\n",
    "head(corevalue)\n",
    "# Selected corevalue position with thr threshold \"thr\"\n",
    "is(selected)\n",
    "length(selected)\n",
    "head(selected)\n",
    "# The result of 2-class clustering\n",
    "# The result of analysis in each L vector\n",
    "is(ClusterL)\n",
    "dim(ClusterL)\n",
    "# The result of analysis in each R vector\n",
    "is(ClusterR)\n",
    "dim(ClusterR)\n",
    "# The result of analysis in each LR vector\n",
    "is(out.vecLR)\n",
    "dim(out.vecLR)\n",
    "head(out.vecLR)\n",
    "head(out.vecLR[1,1][[1]])\n",
    "```\n\n", # Bottom
    "\n\n## Number of cells in each celltype\n\n",
    "```{r}\n", # Top
    "color <- names(celltypes)\n",
    "colors <- celltypes[vapply(unique(color), ",
    "function(x){which(color == x)[1]}, 0L)]\n",
    "numcells <- vapply(names(colors), function(x){",
    "length(which(color == x))",
    "}, 0L)\n",
    "numcells <- data.frame(numcells=numcells, ",
    "celltypes=names(numcells))\n\n",
    "suppressPackageStartupMessages(library(plotly))\n",
    "plot_ly(numcells, x=~celltypes, y=~numcells, type=\"bar\", ",
    "marker = list(color = colors))\n",
    "```\n\n", # Bottom
    "\n\n## Number of expressed genes in ",
    "each celltype (Non-zero genes)\n\n",
    "```{r}\n", # Top
    "expgenes <- apply(input, 2, ",
    "function(x){length(which(x != 0))})\n",
    "expgenes <- data.frame(expgenes=expgenes, ",
    "celltypes=names(celltypes))\n",
    "plot_ly(expgenes, y=~expgenes, color=color, ",
    "colors=colors, type=\"box\")\n",
    "```\n\n", # Bottom
    "\n\n## Two dimensional plot of all cells\n\n",
    "```{r}\n", # Top
    "plot_ly(x=twoD[,1], y=twoD[,2], ",
    "color = color, ",
    "colors = colors, ",
    "type = \"scatter\", ",
    "text = rownames(twoD), ",
    "mode = \"markers\")\n",
    "```\n\n", # Bottom
    "\n\n## Distribution of core tensor values\n\n",
    "```{r}\n", # Top
    "corenames <- vapply(seq_len(nrow(index)), ",
    "function(x){paste(index[x,seq_len(3)], collapse=\",\")}, \"\")\n",
    "plot_ly(x=seq_along(corevalue), y=corevalue, ",
    "type=\"bar\", color=names(corevalue), text=corenames, ",
    "colors = c(\"#999999\", \"#E41A1C\"))\n",
    "```\n" # Bottom
    )

.BODY2_2 <- paste0("\n\n# Global statistics and plots\n\n",
    "The result of scTensor is saved as a R binary file",
    " (reanalysis.RData).\n",
    "```{r}\n", # Top
    "load(\"reanalysis.RData\")\n\n",
    "# SingleCellExperiment object\n",
    "sce\n",
    "# Data size\n",
    "metadata(sce)$datasize\n",
    "# Reduced data size\n",
    "metadata(sce)$ranks\n",
    "# Reconstruction Error of NTD\n",
    "head(metadata(sce)$recerror)\n",
    "tail(metadata(sce)$recerror)\n",
    "# Relative Change of NTD\n",
    "head(metadata(sce)$relchange)\n",
    "tail(metadata(sce)$relchange)\n",
    "# Gene expression matrix\n",
    "is(input)\n",
    "dim(input)\n",
    "input[seq_len(2), seq_len(2)]\n",
    "# The result of 2D dimensional reduction (e.g. t-SNE)\n",
    "is(twoD)\n",
    "dim(twoD)\n",
    "head(twoD)\n",
    "# Ligand-Receptor corresponding table",
    " extracted from LRBase.XXX.eg.db\n",
    "is(LR)\n",
    "dim(LR)\n",
    "head(LR, 2)\n",
    "# Celltype label and color scheme\n",
    "is(celltypes)\n",
    "length(celltypes)\n",
    "head(celltypes)\n",
    "# Core tensor values\n",
    "is(index)\n",
    "dim(index)\n",
    "head(index)\n",
    "is(corevalue)\n",
    "length(corevalue)\n",
    "head(corevalue)\n",
    "# Selected corevalue position with thr threshold \"thr\"\n",
    "is(selected)\n",
    "length(selected)\n",
    "head(selected)\n",
    "# The result of 2-class clustering\n",
    "# The result of analysis in each L vector\n",
    "is(ClusterL)\n",
    "dim(ClusterL)\n",
    "# The result of analysis in each R vector\n",
    "is(ClusterR)\n",
    "dim(ClusterR)\n",
    "# The result of analysis in each LR vector\n",
    "is(out.vecLR)\n",
    "```\n\n", # Bottom
    "\n\n## Number of cells in each celltype\n\n",
    "```{r}\n", # Top
    "color <- names(celltypes)\n",
    "colors <- celltypes[vapply(unique(color), ",
    "function(x){which(color == x)[1]}, 0L)]\n",
    "numcells <- vapply(names(colors), function(x){",
    "length(which(color == x))",
    "}, 0L)\n",
    "numcells <- data.frame(numcells=numcells, ",
    "celltypes=names(numcells))\n\n",
    "suppressPackageStartupMessages(library(plotly))\n",
    "plot_ly(numcells, x=~celltypes, y=~numcells, type=\"bar\", ",
    "marker = list(color = colors))\n",
    "```\n\n", # Bottom
    "\n\n## Number of expressed genes in ",
    "each celltype (Non-zero genes)\n\n",
    "```{r}\n", # Top
    "expgenes <- apply(input, 2, ",
    "function(x){length(which(x != 0))})\n",
    "expgenes <- data.frame(expgenes=expgenes, ",
    "celltypes=names(celltypes))\n",
    "plot_ly(expgenes, y=~expgenes, color=color, ",
    "colors=colors, type=\"box\")\n",
    "```\n\n", # Bottom
    "\n\n## Two dimensional plot of all cells\n\n",
    "```{r}\n", # Top
    "plot_ly(x=twoD[,1], y=twoD[,2], ",
    "color = color, ",
    "colors = colors, ",
    "type = \"scatter\", ",
    "text = rownames(twoD), ",
    "mode = \"markers\")\n",
    "```\n\n", # Bottom
    "\n\n## Distribution of core tensor values\n\n",
    "```{r}\n", # Top
    "corenames <- vapply(seq_len(nrow(index)), ",
    "function(x){paste(index[x,seq_len(3)], collapse=\",\")}, \"\")\n",
    "plot_ly(x=seq_along(corevalue), y=corevalue, ",
    "type=\"bar\", color=names(corevalue), text=corenames, ",
    "colors = c(\"#999999\", \"#E41A1C\"))\n",
    "```\n" # Bottom
    )

# 3. L-Pattern
.BODY3 <- function(numLPattern, ClusterL){
    BODY3 <- paste0("\n\n# Ligand-Cell Patterns\n\n",
    "```{r}\n", # Top
    "suppressPackageStartupMessages(library(\"heatmaply\"))\n",
    "l <- metadata(sce)$sctensor$ligand\n",
    "if(nrow(l) >= 2){\n",
    "rownames(l) <- paste0(\"(\",seq_len(nrow(l)), \",*,*)\")\n",
    "heatmaply(l,",
    "xlab=\"Celltype\",",
    "ylab=\"Pattern\",",
    "fontsize_col=20,",
    "fontsize_row=20,",
    "subplot_widths=c(0.7, 0.1),",
    "subplot_heights=c(0.2, 0.7),",
    "labRow = rownames(l),",
    "labCol = colnames(l)",
    ")\n",
    "}\n",
    "```\n" # Bottom
    )
    BODY3 <- paste0(BODY3,
        paste(vapply(seq_len(numLPattern), function(i){
            ClusterNameL <- paste(names(which(ClusterL[i,] == "selected")),
                collapse=" & ")
            titleL <- paste0("(", i, ",*,*)-Pattern", " = ", ClusterNameL)
            paste0("\n\n## ", titleL, "\n![](figures/Pattern_", i, "__.png)")
        }, ""), collapse=""))
    BODY3 <- paste0(BODY3, "\n")

}

# 4. R-Pattern
.BODY4 <- function(numRPattern, ClusterR){
    BODY4 <- paste0("\n\n# Receptor-Cell Patterns\n\n",
        "```{r}\n", # Top
        "r <- metadata(sce)$sctensor$receptor\n",
        "if(nrow(r) >= 2){\n",
        "rownames(r) <- paste0(\"(*,\",seq_len(nrow(r)), \",*)\")\n",
        "heatmaply(r,",
        "xlab=\"Celltype\",",
        "ylab=\"Pattern\",",
        "fontsize_col=20,",
        "fontsize_row=20,",
        "subplot_widths=c(0.7, 0.1),",
        "subplot_heights=c(0.2, 0.7),",
        "labRow = rownames(r),",
        "labCol = colnames(r)",
        ")\n",
        "}\n",
        "```\n" # Bottom
        )
    BODY4 <- paste0(BODY4,
        paste(vapply(seq_len(numRPattern), function(i){
            ClusterNameR <- paste(names(which(ClusterR[i,] == "selected")),
                collapse=" & ")
            titleR <- paste0("(*,", i, ",*)-Pattern", " = ", ClusterNameR)
            paste0("\n\n## ", titleR, "\n![](figures/Pattern__", i, "_.png)")
        }, ""), collapse=""))
    BODY4 <- paste0(BODY4, "\n")
}

# 5. LR-Pattern
.BODY5 <- paste0("\n\n# LR-pair Patterns\n\n",
"```{r}\n", # Top
"lr <- metadata(sce)$sctensor$lrpair\n",
"rownames(lr) <- paste0(\"(*,*,\",seq_len(nrow(lr)), \")\")\n",
"if(!is.null(GeneInfo$GeneName)){\n",
"GeneName <- GeneInfo$GeneName\n",
"Ligand <- vapply(colnames(lr), function(x){\n",
"    vapply(strsplit(x, \"_\")[[1]][1], function(xx){\n",
"    GeneName[which(GeneName[,2] == xx), 1][1]\n",
"    }, \"\")\n",
"}, \"\")\n",
"Receptor <- vapply(colnames(lr), function(x){\n",
"    vapply(strsplit(x, \"_\")[[1]][2], function(xx){\n",
"    GeneName[which(GeneName[,2] == xx), 1][1]\n",
"    }, \"\")\n",
"}, \"\")\n",
"}else{\n",
"Ligand <- vapply(colnames(lr), function(x){strsplit(x, \"_\")[[1]][1]}, \"\")\n",
"Receptor <- vapply(colnames(lr), function(x){strsplit(x, \"_\")[[1]][2]}, \"\")\n",
"}\n",
"colnames(lr) <- vapply(seq_along(Ligand), function(x){\n",
"    paste(c(Ligand[x], Receptor[x]), collapse=\" - \")\n",
"}, \"\")\n",
"target <- which(rank(colMaxs(lr)) <= 100)\n",
"heatmaply(lr[, target],",
"xlab=\"LR-Pair\",",
"ylab=\"Pattern\",",
"fontsize_col=20,",
"fontsize_row=20,",
"subplot_widths=c(0.7, 0.1),",
"subplot_heights=c(0.2, 0.7),",
"labRow = rownames(lr[, target]),",
"labCol = colnames(lr[, target])",
")\n",
"```\n" # Bottom
)

# 6. CCI-wise Hypergraph
.BODY6 <- paste0("\n\n# CCI-wise Hypergraph\n\n",
    "![](figures/CCIHypergraph.png){ width=100% }\n")

# 7. Gene-wise Hypergraph
.BODY7 <- paste0(
    "\n\n# Gene-wise Hypergraph\n\n",
    "```{r}\n", # Top
    "library(\"scTensor\")\n",
    "load(\"reanalysis.RData\")\n",
    "scTensor:::.myvisNetwork(g)\n",
    "```\n\n", # Bottom
    "[Details of Ligand Gene-centric Overview (selected)](ligand.html)\n\n",
    "[Details of Ligand Gene-centric Overview (all)](ligand_all.html)\n\n",
    "[Details of Receptor Gene-centric Overview (selected)](receptor.html)\n\n",
    "[Details of Receptor Gene-centric Overview (all)](receptor_all.html)\n\n"
    )

# 8. (Ligand, Receptor, LR-pair)-Pattern
.BODY8 <- function(selected, rmdfiles, index, corevalue){
    if(length(selected) != 0){
        htmlfiles <- gsub("Rmd", "html", rmdfiles)
        BODY8 <- vapply(selected, function(x){
            i <- selected[x]
            paste0("\n\n## (", paste(index[i, seq_len(3)],
                collapse=","),
                ") Pattern : (", round(corevalue[i], 2), "%)\n",
                "[Details of (", paste(index[i, seq_len(3)],
                collapse=","),
                ") Pattern", "](", htmlfiles[i], ")\n")
            }, "")
        BODY8 <- paste(BODY8, collapse = "\n")
        BODY8 <- paste0("# (Ligand-Cell, Receptor-Cell, LR-pair)",
            " Patterns\n\n", BODY8)
    }else{
        BODY8 <- "# (Ligand-Cell, Receptor-Cell, LR-pair) Patterns\n\n"
    }
}

.BODY8_2 <- function(selected, rmdfiles, index, corevalue){
    if(length(selected) != 0){
        htmlfiles <- gsub("Rmd", "html", rmdfiles)
        BODY8 <- vapply(selected, function(i){
            paste0("\n\n## (",
                paste(c(index[i, seq_len(2)], ""), collapse=","),
                ") -related L-R Pairs : (", round(corevalue[i], 2), "%)\n",
                "[Details of (",
                paste(c(index[i, seq_len(2)], ""), collapse=","),
                ") -related L-R Pairs", "](", htmlfiles[i], ")\n")
            }, "")
        BODY8 <- paste(BODY8, collapse = "\n")
        BODY8 <- paste0("# (Ligand-Cell, Receptor-Cell, )",
            " -related L-R Pairs\n\n", BODY8)
    }else{
        BODY8 <- "# (Ligand-Cell, Receptor-Cell, ) -related L-R Pairs\n\n"
    }
}

# 9. Session Information
.BODY9 <- paste0("\n\n# Session Information\n\n",
    "\n```{r}\n",
    "sessionInfo()\n",
    "```\n")

# 10. License
.BODY10 <- paste0("\n\n# License\n\n",
    "Copyright (c) 2018 Koki Tsuyuzaki and Laboratory for ",
    "Bioinformatics Research, RIKEN Center for Biosystems Dynamics",
    " Reseach Released under the ",
    "[Artistic License 2.0](",
    "http://www.perlfoundation.org/artistic_license_2_0)\n")

.LIGAND_HEADER <- paste0(
    "---\n",
    "title: <font color='#1881c2'>Details of Ligand Gene-centric Overview (selected)",
    "</font>\n",
    "---\n\n",
    "```{r}\n", # Top
    "library(\"scTensor\")\n",
    "load(\"reanalysis.RData\")\n",
    "scTensor:::.myvisNetwork(g)\n",
    "```\n\n", # Bottom
    "<style type='text/css'>\n",
    "table,th,td {\n",
    "width: 20%;\n",
    "border: 1px solid #f0f0f0;\n",
    "}\n",
    "</style>\n\n",
    "|Rank|No. of related pairs|Ligand Gene|Receptor Genes|",
    "Related CCIs|\n",
    "|---------------|---------------|---------------|---------------|---------------|"
)

.RECEPTOR_HEADER <- paste0(
    "---\n",
    "title: <font color='#1881c2'>Details of Receptor Gene-centric Overview (selected)",
    "</font>\n",
    "---\n\n",
    "```{r}\n", # Top
    "library(\"scTensor\")\n",
    "load(\"reanalysis.RData\")\n",
    "scTensor:::.myvisNetwork(g)\n",
    "```\n\n", # Bottom
    "<style type='text/css'>\n",
    "table,th,td {\n",
    "width: 20%;\n",
    "border: 1px solid #f0f0f0;\n",
    "}\n",
    "</style>\n\n",
    "|Rank|No. of related pairs|Receptor Gene|Ligand Genes|",
    "Related CCIs|\n",
    "|---------------|---------------|---------------|---------------|---------------|"
)

.LIGAND_BODY_2 <- function(out.vecLR, GeneInfo, index, selected){
    # Setting
    convertGeneName <- function(geneid, GeneInfo){
        if(!is.null(GeneInfo$GeneName)){
            genename <- GeneInfo$GeneName[
                which(GeneInfo$GeneName$ENTREZID == geneid), "SYMBOL"]
            if(length(genename) == 0 || genename %in% c("", NA)){
                genename = geneid
            }
            if(length(genename) != 1){
                # Black list
                genename = setdiff(genename, "cxcl11.6")[1]
            }
            genename
        }else{
            geneid
        }
    }

    # Node
    nodes <- lapply(seq_len(length(out.vecLR)), function(x){
        names(out.vecLR[[x]]$TARGET)
    })
    LnodesGeneID <- lapply(nodes, function(x){
        vapply(x, function(xx){
            strsplit(xx, "_")[[1]][1]
        }, "")
    })
    RnodesGeneID <- lapply(nodes, function(x){
        vapply(x, function(xx){
            strsplit(xx, "_")[[1]][2]
        }, "")
    })
    LnodesGeneName <- lapply(LnodesGeneID, function(x){
        vapply(x, function(xx){
            convertGeneName(xx, GeneInfo)
        }, "")
    })
    RnodesGeneName <- lapply(RnodesGeneID, function(x){
        vapply(x, function(xx){
            convertGeneName(xx, GeneInfo)
        }, "")
    })
    uniqueLnodesGeneName <- unique(unlist(LnodesGeneName))
    Freq <- vapply(uniqueLnodesGeneName, function(x){
            length(which(unlist(LnodesGeneName) == x))
        }, 0L)
    Freq <- sort(Freq, decreasing=TRUE)
    Rank <- rank(-Freq)
    Ligand <- names(Freq)
    Receptor <- lapply(Ligand, function(x){
        ReceptorGeneName <- unique(
            unlist(
                lapply(seq_len(length(LnodesGeneName)), function(xx){
                    RnodesGeneName[[xx]][which(x == LnodesGeneName[[xx]])]
                })
            )
        )
        ReceptorGeneID <- unlist(lapply(ReceptorGeneName, function(y){
            target <- which(GeneInfo$GeneName[, "SYMBOL"] == y)
            if(length(target) == 0){
                y
            }else{
                GeneInfo$GeneName[target[1], "ENTREZID"]
            }
        }))
        paste(
            paste0("[", ReceptorGeneName,
                "](https://www.ncbi.nlm.nih.gov/gene/",
            ReceptorGeneID, ") ![](figures/Receptor/",
            ReceptorGeneID, ".png)"), collapse=" ")
    })

    CCI <- vapply(Ligand, function(x){
        hit <- vapply(seq_len(length(out.vecLR)), function(xx){
            length(which(LnodesGeneName[[xx]] == x))
        }, 0L)
        vec <- seq_len(length(out.vecLR))[which(hit != 0)]
        # Out
        cciout <- vapply(vec, function(xx){
            paste0("[See the details of (",
                paste(c(index[xx, paste0("Mode", 1:2)], ""), collapse=","),
                ")-related L-R patterns](",
                "pattern_",
                paste(index[xx, paste0("Mode", 1:2)], collapse="_"),
                ".html)")
        }, "")
        paste(unique(cciout), collapse=" ")
    }, "")
    Ligand <- vapply(Ligand, function(x){
        target <- which(GeneInfo$GeneName$SYMBOL == x)
        if(length(target) == 0){
            LigandGeneID <- x
        }else{
            LigandGeneID <- GeneInfo$GeneName[target[1], "ENTREZID"]
        }
        paste0("[", x, "](https://www.ncbi.nlm.nih.gov/gene/",
            LigandGeneID, ") ![](figures/Ligand/",
            LigandGeneID, ".png)")
    }, "")    
    paste0(
    "|", Rank,
    "|", Freq,
    "|", Ligand,
    "|", Receptor,
    "|", CCI, "|\n", collapse="")
}

.LIGAND_BODY <- function(out.vecLR, GeneInfo, index, selected){
    # Setting
    convertGeneName <- function(geneid, GeneInfo){
        if(!is.null(GeneInfo$GeneName)){
            genename <- GeneInfo$GeneName[
                which(GeneInfo$GeneName$ENTREZID == geneid), "SYMBOL"]
            if(length(genename) == 0 || genename %in% c("", NA)){
                genename = geneid
            }
            if(length(genename) != 1){
                # Black list
                genename = setdiff(genename, "cxcl11.6")[1]
            }
            genename
        }else{
            geneid
        }
    }

    # Node
    nodes <- lapply(seq_len(ncol(out.vecLR)), function(x){
        names(out.vecLR["TARGET", x][[1]])
    })
    LnodesGeneID <- lapply(nodes, function(x){
        vapply(x, function(xx){
            strsplit(xx, "_")[[1]][1]
        }, "")
    })
    RnodesGeneID <- lapply(nodes, function(x){
        vapply(x, function(xx){
            strsplit(xx, "_")[[1]][2]
        }, "")
    })
    LnodesGeneName <- lapply(LnodesGeneID, function(x){
        vapply(x, function(xx){
            convertGeneName(xx, GeneInfo)
        }, "")
    })
    RnodesGeneName <- lapply(RnodesGeneID, function(x){
        vapply(x, function(xx){
            convertGeneName(xx, GeneInfo)
        }, "")
    })
    uniqueLnodesGeneName <- unique(unlist(LnodesGeneName))
    Freq <- vapply(uniqueLnodesGeneName, function(x){
            length(which(unlist(LnodesGeneName) == x))
        }, 0L)
    Freq <- sort(Freq, decreasing=TRUE)
    Rank <- rank(-Freq)
    Ligand <- names(Freq)
    Receptor <- lapply(Ligand, function(x){
        ReceptorGeneName <- unique(
            unlist(
                lapply(seq_len(length(LnodesGeneName)), function(xx){
                    RnodesGeneName[[xx]][which(x == LnodesGeneName[[xx]])]
                })
            )
        )
        ReceptorGeneID <- unlist(lapply(ReceptorGeneName, function(y){
            target <- which(GeneInfo$GeneName[, "SYMBOL"] == y)
            if(length(target) == 0){
                y
            }else{
                GeneInfo$GeneName[target[1], "ENTREZID"]
            }
        }))
        paste(
            paste0("[", ReceptorGeneName,
                "](https://www.ncbi.nlm.nih.gov/gene/",
            ReceptorGeneID, ") ![](figures/Receptor/",
            ReceptorGeneID, ".png)"), collapse=" ")
    })

    CCI <- vapply(Ligand, function(x){
        hit <- vapply(seq_len(ncol(out.vecLR)), function(xx){
            length(which(LnodesGeneName[[xx]] == x))
        }, 0L)
        vec <- seq_len(ncol(out.vecLR))[which(hit != 0)]
        target <- unlist(lapply(vec, function(xx){
                p <- gsub("pattern", "", colnames(out.vecLR)[xx])
                which(index[selected, "Mode3"] == p)
            }))
        out <- index[target, paste0("Mode", seq_len(3))]
        # Out
        if(is.vector(out)){
            paste0("[See the details of (",
                paste(out, collapse=","),
                ")-Pattern](",
                "pattern_",
                paste(out, collapse="_"),
                ".html)")
        }else{
            paste(vapply(seq_len(nrow(out)), function(xx){
                paste0("[See the details of (",
                    paste(out[xx,], collapse=","),
                    ")-Pattern](",
                    "pattern_",
                    paste(out[xx,], collapse="_"),
                    ".html)")
            }, ""), collapse=" ")
        }
    }, "")
    Ligand <- vapply(Ligand, function(x){
        target <- which(GeneInfo$GeneName$SYMBOL == x)
        if(length(target) == 0){
            LigandGeneID <- x
        }else{
            LigandGeneID <- GeneInfo$GeneName[target[1], "ENTREZID"]
        }
        paste0("[", x, "](https://www.ncbi.nlm.nih.gov/gene/",
            LigandGeneID, ") ![](figures/Ligand/",
            LigandGeneID, ".png)")
    }, "")
    paste0(
    "|", Rank,
    "|", Freq,
    "|", Ligand,
    "|", Receptor,
    "|", CCI, "|\n", collapse="")
}

.RECEPTOR_BODY_2 <- function(out.vecLR, GeneInfo, index, selected){
    # Setting
    convertGeneName <- function(geneid, GeneInfo){
        if(!is.null(GeneInfo$GeneName)){
            genename <- GeneInfo$GeneName[
                which(GeneInfo$GeneName$ENTREZID == geneid),
                "SYMBOL"]
            if(length(genename) == 0 || genename %in% c("", NA)){
                genename = geneid
            }
            if(length(genename) != 1){
                # Black list
                genename = setdiff(genename, "cxcl11.6")[1]
            }
            genename
        }else{
            geneid
        }
    }

    # Node
    nodes <- lapply(seq_len(length(out.vecLR)), function(x){
        names(out.vecLR[[x]]$TARGET)
    })
    LnodesGeneID <- lapply(nodes, function(x){
        vapply(x, function(xx){
            strsplit(xx, "_")[[1]][1]
        }, "")
    })
    RnodesGeneID <- lapply(nodes, function(x){
        vapply(x, function(xx){
            strsplit(xx, "_")[[1]][2]
        }, "")
    })
    LnodesGeneName <- lapply(LnodesGeneID, function(x){
        vapply(x, function(xx){
            convertGeneName(xx, GeneInfo)
        }, "")
    })
    RnodesGeneName <- lapply(RnodesGeneID, function(x){
        vapply(x, function(xx){
            convertGeneName(xx, GeneInfo)
        }, "")
    })
    uniqueRnodesGeneName <- unique(unlist(RnodesGeneName))
    Freq <- vapply(uniqueRnodesGeneName, function(x){
            length(which(unlist(RnodesGeneName) == x))
        }, 0L)
    Freq <- sort(Freq, decreasing=TRUE)
    Rank <- rank(-Freq)
    Receptor <- names(Freq)
    Ligand <- lapply(Receptor, function(x){
        LigandGeneName <- unique(
            unlist(
                lapply(seq_len(length(RnodesGeneName)), function(xx){
                    LnodesGeneName[[xx]][which(x == RnodesGeneName[[xx]])]
                })
            )
        )
        LigandGeneID <- unlist(lapply(LigandGeneName, function(y){
            target <- which(GeneInfo$GeneName$SYMBOL == y)
            if(length(target) == 0){
                y
            }else{
                GeneInfo$GeneName[target[1], "ENTREZID"]
            }
        }))
        paste(
            paste0("[", LigandGeneName,
                "](https://www.ncbi.nlm.nih.gov/gene/",
            LigandGeneID, ") ![](figures/Ligand/",
            LigandGeneID, ".png)"), collapse=" ")
    })

    CCI <- vapply(Receptor, function(x){
        hit <- vapply(seq_len(length(out.vecLR)), function(xx){
            length(which(RnodesGeneName[[xx]] == x))
        }, 0L)
        vec <- seq_len(length(out.vecLR))[which(hit != 0)]
        # Out
        cciout <- vapply(vec, function(xx){
            paste0("[See the details of (",
                paste(c(index[xx, paste0("Mode", 1:2)], ""), collapse=","),
                ")-related L-R patterns](",
                "pattern_",
                paste(index[xx, paste0("Mode", 1:2)], collapse="_"),
                ".html)")
        }, "")
        paste(unique(cciout), collapse=" ")
    }, "")
    Receptor <- vapply(Receptor, function(x){
        target <- which(GeneInfo$GeneName$SYMBOL == x)
        if(length(target) == 0){
            ReceptorGeneID <- x
        }else{
            ReceptorGeneID <- GeneInfo$GeneName[target[1], "ENTREZID"]
        }
        paste0("[", x, "](https://www.ncbi.nlm.nih.gov/gene/",
            ReceptorGeneID, ") ![](figures/Receptor/",
            ReceptorGeneID, ".png)")
    }, "")
    paste0(
    "|", Rank,
    "|", Freq,
    "|", Receptor,
    "|", Ligand,
    "|", CCI, "|\n", collapse="")
}

.RECEPTOR_BODY <- function(out.vecLR, GeneInfo, index, selected){
    # Setting
    convertGeneName <- function(geneid, GeneInfo){
        if(!is.null(GeneInfo$GeneName)){
            genename <- GeneInfo$GeneName[
                which(GeneInfo$GeneName$ENTREZID == geneid),
                "SYMBOL"]
            if(length(genename) == 0 || genename %in% c("", NA)){
                genename = geneid
            }
            if(length(genename) != 1){
                # Black list
                genename = setdiff(genename, "cxcl11.6")[1]
            }
            genename
        }else{
            geneid
        }
    }

    # Node
    nodes <- lapply(seq_len(ncol(out.vecLR)), function(x){
        names(out.vecLR["TARGET", x][[1]])
    })
    LnodesGeneID <- lapply(nodes, function(x){
        vapply(x, function(xx){
            strsplit(xx, "_")[[1]][1]
        }, "")
    })
    RnodesGeneID <- lapply(nodes, function(x){
        vapply(x, function(xx){
            strsplit(xx, "_")[[1]][2]
        }, "")
    })
    LnodesGeneName <- lapply(LnodesGeneID, function(x){
        vapply(x, function(xx){
            convertGeneName(xx, GeneInfo)
        }, "")
    })
    RnodesGeneName <- lapply(RnodesGeneID, function(x){
        vapply(x, function(xx){
            convertGeneName(xx, GeneInfo)
        }, "")
    })
    uniqueRnodesGeneName <- unique(unlist(RnodesGeneName))
    Freq <- vapply(uniqueRnodesGeneName, function(x){
            length(which(unlist(RnodesGeneName) == x))
        }, 0L)
    Freq <- sort(Freq, decreasing=TRUE)
    Rank <- rank(-Freq)
    Receptor <- names(Freq)
    Ligand <- lapply(Receptor, function(x){
        LigandGeneName <- unique(
            unlist(
                lapply(seq_len(length(RnodesGeneName)), function(xx){
                    LnodesGeneName[[xx]][which(x == RnodesGeneName[[xx]])]
                })
            )
        )
        LigandGeneID <- unlist(lapply(LigandGeneName, function(y){
            target <- which(GeneInfo$GeneName$SYMBOL == y)
            if(length(target) == 0){
                y
            }else{
                GeneInfo$GeneName[target[1], "ENTREZID"]
            }
        }))
        paste(
            paste0("[", LigandGeneName,
                "](https://www.ncbi.nlm.nih.gov/gene/",
            LigandGeneID, ") ![](figures/Ligand/",
            LigandGeneID, ".png)"), collapse=" ")
    })

    CCI <- vapply(Receptor, function(x){
        hit <- vapply(seq_len(ncol(out.vecLR)), function(xx){
            length(which(RnodesGeneName[[xx]] == x))
        }, 0L)
        vec <- seq_len(ncol(out.vecLR))[which(hit != 0)]
        target <- unlist(lapply(vec, function(xx){
                p <- gsub("pattern", "", colnames(out.vecLR)[xx])
                which(index[selected, "Mode3"] == p)
            }))
        out <- index[target, c("Mode1", "Mode2", "Mode3")]
        # Out
        if(is.vector(out)){
            paste0("[See the details of (",
                paste(out, collapse=","),
                ")-Pattern](",
                "pattern_",
                paste(out, collapse="_"),
                ".html)")
        }else{
            paste(vapply(seq_len(nrow(out)), function(xx){
                paste0("[See the details of (",
                    paste(out[xx,], collapse=","),
                    ")-Pattern](",
                    "pattern_",
                    paste(out[xx,], collapse="_"),
                    ".html)")
            }, ""), collapse=" ")
        }
    }, "")
    Receptor <- vapply(Receptor, function(x){
        target <- which(GeneInfo$GeneName$SYMBOL == x)
        if(length(target) == 0){
            ReceptorGeneID <- x
        }else{
            ReceptorGeneID <- GeneInfo$GeneName[target[1], "ENTREZID"]
        }
        paste0("[", x, "](https://www.ncbi.nlm.nih.gov/gene/",
            ReceptorGeneID, ") ![](figures/Receptor/",
            ReceptorGeneID, ".png)")
    }, "")
    paste0(
    "|", Rank,
    "|", Freq,
    "|", Receptor,
    "|", Ligand,
    "|", CCI, "|\n", collapse="")
}

.LIGANDALL_HEADER <- paste0(
    "---\n",
    "title: <font color='#1881c2'>Details of Ligand Gene-centric Overview (all)",
    "</font>\n",
    "---\n\n",
    "<style type='text/css'>\n",
    "table,th,td {\n",
    "width: 50%;\n",
    "border: 1px solid #f0f0f0;\n",
    "}\n",
    "</style>\n\n",
    "|Ligand Gene|Receptor Gene|\n",
    "|---------------------------------|---------------------------------|"
)

.RECEPTORALL_HEADER <- paste0(
    "---\n",
    "title: <font color='#1881c2'>Details of Receptor Gene-centric Overview (all)",
    "</font>\n",
    "---\n\n",
    "<style type='text/css'>\n",
    "table,th,td {\n",
    "width: 50%;\n",
    "border: 1px solid #f0f0f0;\n",
    "}\n",
    "</style>\n\n",
    "|Receptor Gene|Ligand Gene|\n",
    "|---------------------------------|---------------------------------|"
)

.LIGANDALL_BODY <- function(GeneInfo, LR, input){
    if(!is.null(GeneInfo$GeneName)){
        GeneName <- GeneInfo$GeneName
        LigandGeneID <- unique(LR$GENEID_L)
        LigandGeneName <- vapply(LigandGeneID, function(x){
            GeneName[which(GeneName$ENTREZID == x)[1], "SYMBOL"]}, "")
        naposition <- which(vapply(LigandGeneName, is.na, TRUE))
        LigandGeneName[naposition] <- LigandGeneID[naposition]
        # Sort by Alphabet of the ligand genes
        orderLigand <- order(LigandGeneName)
        LigandGeneID <- LigandGeneID[orderLigand]
        LigandGeneName <- LigandGeneName[orderLigand]
    }else{
        LigandGeneName <- LigandGeneID  
    }

    # Ligand Link
    Ligand <- vapply(seq_along(LigandGeneID), function(x){
        target <- which(rownames(input) == LigandGeneID[x])
        if(length(target) != 0){
            paste0("[", LigandGeneName[x],
                "](https://www.ncbi.nlm.nih.gov/gene/",
                LigandGeneID[x], ") ",
                "![](figures/Ligand/", LigandGeneID[x],
                ".png)")
        }else{
            paste0("[", LigandGeneName[x],
                "](https://www.ncbi.nlm.nih.gov/gene/",
                LigandGeneID[x], ")")
        }
    }, "")

    # Receptor Link
    Receptor <- vapply(seq_along(LigandGeneID), function(x){
        target <- which(LR$GENEID_L == LigandGeneID[x])
        ReceptorGeneID <- unique(LR$GENEID_R[target])
        if(!is.null(GeneInfo$GeneName)){
            ReceptorGeneName <- vapply(ReceptorGeneID, function(xx){
                GeneName[which(GeneName$ENTREZID == xx)[1], "SYMBOL"]
            }, "")
            naposition <- which(vapply(ReceptorGeneName, is.na, TRUE))
            ReceptorGeneName[naposition] <- ReceptorGeneID[naposition]
        }else{
            ReceptorGeneName <- ReceptorGeneID
        }
        paste(paste0("[", ReceptorGeneName,
            "](https://www.ncbi.nlm.nih.gov/gene/",
            ReceptorGeneID, ")"),
            collapse=" ")
    }, "")
    # Output
    paste0(
    "|", Ligand,
    "|", Receptor, "|\n", collapse="")
}

.RECEPTORALL_BODY <- function(GeneInfo, LR, input){
    if(!is.null(GeneInfo$GeneName)){
        GeneName <- GeneInfo$GeneName
        ReceptorGeneID <- unique(LR$GENEID_R)
        ReceptorGeneName <- vapply(ReceptorGeneID, function(x){
            GeneName[which(GeneName$ENTREZID == x)[1], "SYMBOL"]}, "")
        naposition <- which(vapply(ReceptorGeneName, is.na, TRUE))
        ReceptorGeneName[naposition] <- ReceptorGeneID[naposition]
        # Sort by Alphabet of the Receptor genes
        orderReceptor <- order(ReceptorGeneName)
        ReceptorGeneID <- ReceptorGeneID[orderReceptor]
        ReceptorGeneName <- ReceptorGeneName[orderReceptor]
    }else{
        ReceptorGeneName <- ReceptorGeneID
    }

    # Receptor Link
    Receptor <- vapply(seq_along(ReceptorGeneID), function(x){
        target <- which(rownames(input) == ReceptorGeneID[x])
        if(length(target) != 0){
            paste0("[", ReceptorGeneName[x],
                "](https://www.ncbi.nlm.nih.gov/gene/",
                ReceptorGeneID[x], ") ",
                "![](figures/Receptor/", ReceptorGeneID[x],
                ".png)")
        }else{
            paste0("[", ReceptorGeneName[x],
                "](https://www.ncbi.nlm.nih.gov/gene/",
                ReceptorGeneID[x], ")")
        }
    }, "")

    # Ligand Link
    Ligand <- vapply(seq_along(ReceptorGeneID), function(x){
        target <- which(LR$GENEID_R == ReceptorGeneID[x])
        LigandGeneID <- unique(LR$GENEID_L[target])
        if(!is.null(GeneInfo$GeneName)){
            LigandGeneName <- vapply(LigandGeneID, function(xx){
                GeneName[which(GeneName$ENTREZID == xx)[1], "SYMBOL"]
            }, "")
            naposition <- which(vapply(LigandGeneName, is.na, TRUE))
            LigandGeneName[naposition] <- LigandGeneID[naposition]
        }else{
            LigandGeneName <- LigandGeneID
        }
        paste(paste0("[", LigandGeneName,
            "](https://www.ncbi.nlm.nih.gov/gene/",
            LigandGeneID, ")"),
            collapse=" ")
    }, "")

    # Output
    paste0(
    "|", Receptor,
    "|", Ligand, "|\n", collapse="")
}

.XYZ_ENRICH_2 <- function(out.vecLR, i){
    go_bp <- .check_NULL_GO_BP_2(out.vecLR, i)
    go_mf <- .check_NULL_GO_MF_2(out.vecLR, i)
    go_cc <- .check_NULL_GO_CC_2(out.vecLR, i)
    mesh_a <- .check_NULL_MeSH_A_2(out.vecLR, i)
    mesh_b <- .check_NULL_MeSH_B_2(out.vecLR, i)
    mesh_c <- .check_NULL_MeSH_C_2(out.vecLR, i)
    mesh_d <- .check_NULL_MeSH_D_2(out.vecLR, i)
    mesh_e <- .check_NULL_MeSH_E_2(out.vecLR, i)
    mesh_f <- .check_NULL_MeSH_F_2(out.vecLR, i)
    mesh_g <- .check_NULL_MeSH_G_2(out.vecLR, i)
    mesh_h <- .check_NULL_MeSH_H_2(out.vecLR, i)
    mesh_i <- .check_NULL_MeSH_I_2(out.vecLR, i)
    mesh_j <- .check_NULL_MeSH_J_2(out.vecLR, i)
    mesh_k <- .check_NULL_MeSH_K_2(out.vecLR, i)
    mesh_l <- .check_NULL_MeSH_L_2(out.vecLR, i)
    mesh_m <- .check_NULL_MeSH_M_2(out.vecLR, i)
    mesh_n <- .check_NULL_MeSH_N_2(out.vecLR, i)
    mesh_v <- .check_NULL_MeSH_V_2(out.vecLR, i)
    mesh_z <- .check_NULL_MeSH_Z_2(out.vecLR, i)
    reactome <- .check_NULL_Reactome_2(out.vecLR, i)
    do <- .check_NULL_DO_2(out.vecLR, i)
    ncg <- .check_NULL_NCG_2(out.vecLR, i)
    dgn <- .check_NULL_DGN_2(out.vecLR, i)
    # Output
    paste(c(go_bp, go_mf, go_cc,
    mesh_a, mesh_b, mesh_c, mesh_d, mesh_e, mesh_f,
    mesh_g, mesh_h, mesh_i, mesh_j, mesh_k, mesh_l,
    mesh_m, mesh_n, mesh_v, mesh_z,
    reactome,
    do, ncg, dgn), collapse="")
}

################ GO_BP ################
.check_NULL_GO_BP_2 <- function(out.vecLR, i){
    if(!is.null(out.vecLR[[i]]$Enrich$GO_BP$Pvalue)){
        paste0("## <font color='#1881c2'>GO-Enrichment Analysis (BP : Biological Process)</font>\n\n",
            "```{r}\n", # Top
            "negLogPval <- -log10(out.vecLR[['",
            i,
            "']]$Enrich$GO_BP$Pvalue + 1E-10)\n",
            "term <- out.vecLR[['",
            i,
            "']]$Enrich$GO_BP$Term\n",
            "target <- seq_len(min(100, length(negLogPval)))\n",
            "negLogPval <- negLogPval[target]\n",
            "term <- term[target]\n",
            "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
            "type=\"bar\", color=~negLogPval, text=term,\n",
            "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
            "layout(p, title=\"GO-Enrichment Analysis (BP)\",\n",
            "   xaxis=list(title=\"Term\"),\n",
            "   yaxis=list(title=\"-Log(P-value)\"),\n",
            "   legend=list(name=\"\"))\n",
            "```\n\n", # Bottom
            "![](figures/Tagcloud/GO_BP_",
            i,
            ".png)\n\n")
    }else{
        ""
    }
}

################ GO_MF ################
.check_NULL_GO_MF_2 <- function(out.vecLR, i){
    if(!is.null(out.vecLR[[i]]$Enrich$GO_MF$Pvalue)){
        paste0("## <font color='#1881c2'>GO-Enrichment Analysis (MF : Molecular Function)</font>\n\n",
            "```{r}\n", # Top
            "negLogPval <- -log10(out.vecLR[['",
            i,
            "']]$Enrich$GO_MF$Pvalue + 1E-10)\n",
            "term <- out.vecLR[['",
            i,
            "']]$Enrich$GO_MF$Term\n",
            "target <- seq_len(min(100, length(negLogPval)))\n",
            "negLogPval <- negLogPval[target]\n",
            "term <- term[target]\n",
            "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
            "type=\"bar\", color=~negLogPval, text=term,\n",
            "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
            "layout(p, title=\"GO-Enrichment Analysis (MF)\",\n",
            "   xaxis=list(title=\"Term\"),\n",
            "   yaxis=list(title=\"-Log(P-value)\"),\n",
            "   legend=list(name=\"\"))\n",
            "```\n\n", # Bottom
            "![](figures/Tagcloud/GO_MF_",
            i,
            ".png)\n\n")
    }else{
        ""
    }
}

################ GO_CC ################
.check_NULL_GO_CC_2 <- function(out.vecLR, i){
    if(!is.null(out.vecLR[[i]]$Enrich$GO_CC$Pvalue)){
        paste0("## <font color='#1881c2'>GO-Enrichment Analysis (CC : Cellular Component)</font>\n\n",
            "```{r}\n", # Top
            "negLogPval <- -log10(out.vecLR[['",
            i,
            "']]$Enrich$GO_CC$Pvalue + 1E-10)\n",
            "term <- out.vecLR[['",
            i,
            "']]$Enrich$GO_CC$Term\n",
            "target <- seq_len(min(100, length(negLogPval)))\n",
            "negLogPval <- negLogPval[target]\n",
            "term <- term[target]\n",
            "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
            "type=\"bar\", color=~negLogPval, text=term,\n",
            "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
            "layout(p, title=\"GO-Enrichment Analysis (CC)\",\n",
            "   xaxis=list(title=\"Term\"),\n",
            "   yaxis=list(title=\"-Log(P-value)\"),\n",
            "   legend=list(name=\"\"))\n",
            "```\n\n", # Bottom
            "![](figures/Tagcloud/GO_CC_",
            i,
            ".png)\n\n")
    }else{
        ""
    }
}

################ MeSH_A ################
.check_NULL_MeSH_A_2 <- function(out.vecLR, i){
    if(!is.null(out.vecLR[[i]]$Enrich$MeSH_A$Pvalue)){
        paste0("## <font color='#1881c2'>MeSH-Enrichment Analysis (A : Anatomy)</font>\n\n",
            "```{r}\n", # Top
            "negLogPval <- -log10(out.vecLR[['",
            i,
            "']]$Enrich$MeSH_A$Pvalue + 1E-10)\n",
            "term <- out.vecLR[['",
            i,
            "']]$Enrich$MeSH_A$Term\n",
            "target <- seq_len(min(100, length(negLogPval)))\n",
            "negLogPval <- negLogPval[target]\n",
            "term <- term[target]\n",
            "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
            "type=\"bar\", color=~negLogPval, text=term,\n",
            "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
            "layout(p, title=\"MeSH-Enrichment Analysis (A)\",\n",
            "   xaxis=list(title=\"Term\"),\n",
            "   yaxis=list(title=\"-Log(P-value)\"),\n",
            "   legend=list(name=\"\"))\n",
            "```\n\n", # Bottom
            "![](figures/Tagcloud/MeSH_A_",
            i,
            ".png)\n\n")
    }else{
        ""
    }
}

################ MeSH_B ################
.check_NULL_MeSH_B_2 <- function(out.vecLR, i){
    if(!is.null(out.vecLR[[i]]$Enrich$MeSH_B$Pvalue)){
        paste0("## <font color='#1881c2'>MeSH-Enrichment Analysis (B : Organisms)</font>\n\n",
            "```{r}\n", # Top
            "negLogPval <- -log10(out.vecLR[['",
            i,
            "']]$Enrich$MeSH_B$Pvalue + 1E-10)\n",
            "term <- out.vecLR[['",
            i,
            "']]$Enrich$MeSH_B$Term\n",
            "target <- seq_len(min(100, length(negLogPval)))\n",
            "negLogPval <- negLogPval[target]\n",
            "term <- term[target]\n",
            "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
            "type=\"bar\", color=~negLogPval, text=term,\n",
            "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
            "layout(p, title=\"MeSH-Enrichment Analysis (B)\",\n",
            "   xaxis=list(title=\"Term\"),\n",
            "   yaxis=list(title=\"-Log(P-value)\"),\n",
            "   legend=list(name=\"\"))\n",
            "```\n\n", # Bottom
            "![](figures/Tagcloud/MeSH_B_",
            i,
            ".png)\n\n")
    }else{
        ""
    }
}

################ MeSH_C ################
.check_NULL_MeSH_C_2 <- function(out.vecLR, i){
    if(!is.null(out.vecLR[[i]]$Enrich$MeSH_C$Pvalue)){
        paste0("## <font color='#1881c2'>MeSH-Enrichment Analysis (C : Diseases)</font>\n\n",
            "```{r}\n", # Top
            "negLogPval <- -log10(out.vecLR[['",
            i,
            "']]$Enrich$MeSH_C$Pvalue + 1E-10)\n",
            "term <- out.vecLR[['",
            i,
            "']]$Enrich$MeSH_C$Term\n",
            "target <- seq_len(min(100, length(negLogPval)))\n",
            "negLogPval <- negLogPval[target]\n",
            "term <- term[target]\n",
            "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
            "type=\"bar\", color=~negLogPval, text=term,\n",
            "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
            "layout(p, title=\"MeSH-Enrichment Analysis (C)\",\n",
            "   xaxis=list(title=\"Term\"),\n",
            "   yaxis=list(title=\"-Log(P-value)\"),\n",
            "   legend=list(name=\"\"))\n",
            "```\n\n", # Bottom
            "![](figures/Tagcloud/MeSH_C_",
            i,
            ".png)\n\n")
    }else{
        ""
    }
}

################ MeSH_D ################
.check_NULL_MeSH_D_2 <- function(out.vecLR, i){
    if(!is.null(out.vecLR[[i]]$Enrich$MeSH_D$Pvalue)){
        paste0("## <font color='#1881c2'>MeSH-Enrichment Analysis (D : Drugs)</font>\n\n",
            "```{r}\n", # Top
            "negLogPval <- -log10(out.vecLR[['",
            i,
            "']]$Enrich$MeSH_D$Pvalue + 1E-10)\n",
            "term <- out.vecLR[['",
            i,
            "']]$Enrich$MeSH_D$Term\n",
            "target <- seq_len(min(100, length(negLogPval)))\n",
            "negLogPval <- negLogPval[target]\n",
            "term <- term[target]\n",
            "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
            "type=\"bar\", color=~negLogPval, text=term,\n",
            "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
            "layout(p, title=\"MeSH-Enrichment Analysis (D)\",\n",
            "   xaxis=list(title=\"Term\"),\n",
            "   yaxis=list(title=\"-Log(P-value)\"),\n",
            "   legend=list(name=\"\"))\n",
            "```\n\n", # Bottom
            "![](figures/Tagcloud/MeSH_D_",
            i,
            ".png)\n\n")
    }else{
        ""
    }
}

################ MeSH_E ################
.check_NULL_MeSH_E_2 <- function(out.vecLR, i){
    if(!is.null(out.vecLR[[i]]$Enrich$MeSH_E$Pvalue)){
        paste0("## <font color='#1881c2'>MeSH-Enrichment Analysis (E : Analytical, Diagnostic and Therapeutic Techniques and Equipment)</font>\n\n",
            "```{r}\n", # Top
            "negLogPval <- -log10(out.vecLR[['",
            i,
            "']]$Enrich$MeSH_E$Pvalue + 1E-10)\n",
            "term <- out.vecLR[['",
            i,
            "']]$Enrich$MeSH_E$Term\n",
            "target <- seq_len(min(100, length(negLogPval)))\n",
            "negLogPval <- negLogPval[target]\n",
            "term <- term[target]\n",
            "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
            "type=\"bar\", color=~negLogPval, text=term,\n",
            "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
            "layout(p, title=\"MeSH-Enrichment Analysis (E)\",\n",
            "   xaxis=list(title=\"Term\"),\n",
            "   yaxis=list(title=\"-Log(P-value)\"),\n",
            "   legend=list(name=\"\"))\n",
            "```\n\n", # Bottom
            "![](figures/Tagcloud/MeSH_E_",
            i,
            ".png)\n\n")
    }else{
        ""
    }
}

################ MeSH_F ################
.check_NULL_MeSH_F_2 <- function(out.vecLR, i){
    if(!is.null(out.vecLR[[i]]$Enrich$MeSH_F$Pvalue)){
        paste0("## <font color='#1881c2'>MeSH-Enrichment Analysis (F : Psychiatry and Psychology)</font>\n\n",
            "```{r}\n", # Top
            "negLogPval <- -log10(out.vecLR[['",
            i,
            "']]$Enrich$MeSH_F$Pvalue + 1E-10)\n",
            "term <- out.vecLR[['",
            i,
            "']]$Enrich$MeSH_F$Term\n",
            "target <- seq_len(min(100, length(negLogPval)))\n",
            "negLogPval <- negLogPval[target]\n",
            "term <- term[target]\n",
            "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
            "type=\"bar\", color=~negLogPval, text=term,\n",
            "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
            "layout(p, title=\"MeSH-Enrichment Analysis (F)\",\n",
            "   xaxis=list(title=\"Term\"),\n",
            "   yaxis=list(title=\"-Log(P-value)\"),\n",
            "   legend=list(name=\"\"))\n",
            "```\n\n", # Bottom
            "![](figures/Tagcloud/MeSH_F_",
            i,
            ".png)\n\n")
    }else{
        ""
    }
}

################ MeSH_G ################
.check_NULL_MeSH_G_2 <- function(out.vecLR, i){
    if(!is.null(out.vecLR[[i]]$Enrich$MeSH_G$Pvalue)){
        paste0("## <font color='#1881c2'>MeSH-Enrichment Analysis (G : Phenomena and Processes)</font>\n\n",
            "```{r}\n", # Top
            "negLogPval <- -log10(out.vecLR[['",
            i,
            "']]$Enrich$MeSH_G$Pvalue + 1E-10)\n",
            "term <- out.vecLR[['",
            i,
            "']]$Enrich$MeSH_G$Term\n",
            "target <- seq_len(min(100, length(negLogPval)))\n",
            "negLogPval <- negLogPval[target]\n",
            "term <- term[target]\n",
            "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
            "type=\"bar\", color=~negLogPval, text=term,\n",
            "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
            "layout(p, title=\"MeSH-Enrichment Analysis (G)\",\n",
            "   xaxis=list(title=\"Term\"),\n",
            "   yaxis=list(title=\"-Log(P-value)\"),\n",
            "   legend=list(name=\"\"))\n",
            "```\n\n", # Bottom
            "![](figures/Tagcloud/MeSH_G_",
            i,
            ".png)\n\n")
    }else{
        ""
    }
}

################ MeSH_H ################
.check_NULL_MeSH_H_2 <- function(out.vecLR, i){
    if(!is.null(out.vecLR[[i]]$Enrich$MeSH_H$Pvalue)){
        paste0("## <font color='#1881c2'>MeSH-Enrichment Analysis (H : Disciplines and Occupations)</font>\n\n",
            "```{r}\n", # Top
            "negLogPval <- -log10(out.vecLR[['",
            i,
            "']]$Enrich$MeSH_H$Pvalue + 1E-10)\n",
            "term <- out.vecLR[['",
            i,
            "']]$Enrich$MeSH_H$Term\n",
            "target <- seq_len(min(100, length(negLogPval)))\n",
            "negLogPval <- negLogPval[target]\n",
            "term <- term[target]\n",
            "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
            "type=\"bar\", color=~negLogPval, text=term,\n",
            "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
            "layout(p, title=\"MeSH-Enrichment Analysis (H)\",\n",
            "   xaxis=list(title=\"Term\"),\n",
            "   yaxis=list(title=\"-Log(P-value)\"),\n",
            "   legend=list(name=\"\"))\n",
            "```\n\n", # Bottom
            "![](figures/Tagcloud/MeSH_H_",
            i,
            ".png)\n\n")
    }else{
        ""
    }
}

################ MeSH_I ################
.check_NULL_MeSH_I_2 <- function(out.vecLR, i){
    if(!is.null(out.vecLR[[i]]$Enrich$MeSH_I$Pvalue)){
        paste0("## <font color='#1881c2'>MeSH-Enrichment Analysis (I : Anthropology, Education, Sociology and Social Phenomena)</font>\n\n",
            "```{r}\n", # Top
            "negLogPval <- -log10(out.vecLR[['",
            i,
            "']]$Enrich$MeSH_I$Pvalue + 1E-10)\n",
            "term <- out.vecLR[['",
            i,
            "']]$Enrich$MeSH_I$Term\n",
            "target <- seq_len(min(100, length(negLogPval)))\n",
            "negLogPval <- negLogPval[target]\n",
            "term <- term[target]\n",
            "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
            "type=\"bar\", color=~negLogPval, text=term,\n",
            "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
            "layout(p, title=\"MeSH-Enrichment Analysis (I)\",\n",
            "   xaxis=list(title=\"Term\"),\n",
            "   yaxis=list(title=\"-Log(P-value)\"),\n",
            "   legend=list(name=\"\"))\n",
            "```\n\n", # Bottom
            "![](figures/Tagcloud/MeSH_I_",
            i,
            ".png)\n\n")
    }else{
        ""
    }
}

################ MeSH_J ################
.check_NULL_MeSH_J_2 <- function(out.vecLR, i){
    if(!is.null(out.vecLR[[i]]$Enrich$MeSH_J$Pvalue)){
        paste0("## <font color='#1881c2'>MeSH-Enrichment Analysis (J : Technology and Food and Beverages)</font>\n\n",
            "```{r}\n", # Top
            "negLogPval <- -log10(out.vecLR[['",
            i,
            "']]$Enrich$MeSH_J$Pvalue + 1E-10)\n",
            "term <- out.vecLR[['",
            i,
            "']]$Enrich$MeSH_J$Term\n",
            "target <- seq_len(min(100, length(negLogPval)))\n",
            "negLogPval <- negLogPval[target]\n",
            "term <- term[target]\n",
            "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
            "type=\"bar\", color=~negLogPval, text=term,\n",
            "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
            "layout(p, title=\"MeSH-Enrichment Analysis (J)\",\n",
            "   xaxis=list(title=\"Term\"),\n",
            "   yaxis=list(title=\"-Log(P-value)\"),\n",
            "   legend=list(name=\"\"))\n",
            "```\n\n", # Bottom
            "![](figures/Tagcloud/MeSH_J_",
            i,
            ".png)\n\n")
    }else{
        ""
    }
}

################ MeSH_K ################
.check_NULL_MeSH_K_2 <- function(out.vecLR, i){
    if(!is.null(out.vecLR[[i]]$Enrich$MeSH_K$Pvalue)){
        paste0("## <font color='#1881c2'>MeSH-Enrichment Analysis (K : Humanities)</font>\n\n",
            "```{r}\n", # Top
            "negLogPval <- -log10(out.vecLR[['",
            i,
            "']]$Enrich$MeSH_K$Pvalue + 1E-10)\n",
            "term <- out.vecLR[['",
            i,
            "']]$Enrich$MeSH_K$Term\n",
            "target <- seq_len(min(100, length(negLogPval)))\n",
            "negLogPval <- negLogPval[target]\n",
            "term <- term[target]\n",
            "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
            "type=\"bar\", color=~negLogPval, text=term,\n",
            "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
            "layout(p, title=\"MeSH-Enrichment Analysis (K)\",\n",
            "   xaxis=list(title=\"Term\"),\n",
            "   yaxis=list(title=\"-Log(P-value)\"),\n",
            "   legend=list(name=\"\"))\n",
            "```\n\n", # Bottom
            "![](figures/Tagcloud/MeSH_K_",
            i,
            ".png)\n\n")
    }else{
        ""
    }
}

################ MeSH_L ################
.check_NULL_MeSH_L_2 <- function(out.vecLR, i){
    if(!is.null(out.vecLR[[i]]$Enrich$MeSH_L$Pvalue)){
        paste0("## <font color='#1881c2'>MeSH-Enrichment Analysis (L : Information Science)</font>\n\n",
            "```{r}\n", # Top
            "negLogPval <- -log10(out.vecLR[['",
            i,
            "']]$Enrich$MeSH_L$Pvalue + 1E-10)\n",
            "term <- out.vecLR[['",
            i,
            "']]$Enrich$MeSH_L$Term\n",
            "target <- seq_len(min(100, length(negLogPval)))\n",
            "negLogPval <- negLogPval[target]\n",
            "term <- term[target]\n",
            "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
            "type=\"bar\", color=~negLogPval, text=term,\n",
            "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
            "layout(p, title=\"MeSH-Enrichment Analysis (L)\",\n",
            "   xaxis=list(title=\"Term\"),\n",
            "   yaxis=list(title=\"-Log(P-value)\"),\n",
            "   legend=list(name=\"\"))\n",
            "```\n\n", # Bottom
            "![](figures/Tagcloud/MeSH_L_",
            i,
            ".png)\n\n")
    }else{
        ""
    }
}

################ MeSH_M ################
.check_NULL_MeSH_M_2 <- function(out.vecLR, i){
    if(!is.null(out.vecLR[[i]]$Enrich$MeSH_M$Pvalue)){
        paste0("## <font color='#1881c2'>MeSH-Enrichment Analysis (M : Persons)</font>\n\n",
            "```{r}\n", # Top
            "negLogPval <- -log10(out.vecLR[['",
            i,
            "']]$Enrich$MeSH_M$Pvalue + 1E-10)\n",
            "term <- out.vecLR[['",
            i,
            "']]$Enrich$MeSH_M$Term\n",
            "target <- seq_len(min(100, length(negLogPval)))\n",
            "negLogPval <- negLogPval[target]\n",
            "term <- term[target]\n",
            "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
            "type=\"bar\", color=~negLogPval, text=term,\n",
            "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
            "layout(p, title=\"MeSH-Enrichment Analysis (M)\",\n",
            "   xaxis=list(title=\"Term\"),\n",
            "   yaxis=list(title=\"-Log(P-value)\"),\n",
            "   legend=list(name=\"\"))\n",
            "```\n\n", # Bottom
            "![](figures/Tagcloud/MeSH_M_",
            i,
            ".png)\n\n")
    }else{
        ""
    }
}

################ MeSH_N ################
.check_NULL_MeSH_N_2 <- function(out.vecLR, i){
    if(!is.null(out.vecLR[[i]]$Enrich$MeSH_N$Pvalue)){
        paste0("## <font color='#1881c2'>MeSH-Enrichment Analysis (N : Health Care)</font>\n\n",
            "```{r}\n", # Top
            "negLogPval <- -log10(out.vecLR[['",
            i,
            "']]$Enrich$MeSH_N$Pvalue + 1E-10)\n",
            "term <- out.vecLR[['",
            i,
            "']]$Enrich$MeSH_N$Term\n",
            "target <- seq_len(min(100, length(negLogPval)))\n",
            "negLogPval <- negLogPval[target]\n",
            "term <- term[target]\n",
            "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
            "type=\"bar\", color=~negLogPval, text=term,\n",
            "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
            "layout(p, title=\"MeSH-Enrichment Analysis (N)\",\n",
            "   xaxis=list(title=\"Term\"),\n",
            "   yaxis=list(title=\"-Log(P-value)\"),\n",
            "   legend=list(name=\"\"))\n",
            "```\n\n", # Bottom
            "![](figures/Tagcloud/MeSH_N_",
            i,
            ".png)\n\n")
    }else{
        ""
    }
}

################ MeSH_V ################
.check_NULL_MeSH_V_2 <- function(out.vecLR, i){
    if(!is.null(out.vecLR[[i]]$Enrich$MeSH_V$Pvalue)){
        paste0("## <font color='#1881c2'>MeSH-Enrichment Analysis (V : Publication Type)</font>\n\n",
            "```{r}\n", # Top
            "negLogPval <- -log10(out.vecLR[['",
            i,
            "']]$Enrich$MeSH_V$Pvalue + 1E-10)\n",
            "term <- out.vecLR[['",
            i,
            "']]$Enrich$MeSH_V$Term\n",
            "target <- seq_len(min(100, length(negLogPval)))\n",
            "negLogPval <- negLogPval[target]\n",
            "term <- term[target]\n",
            "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
            "type=\"bar\", color=~negLogPval, text=term,\n",
            "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
            "layout(p, title=\"MeSH-Enrichment Analysis (V)\",\n",
            "   xaxis=list(title=\"Term\"),\n",
            "   yaxis=list(title=\"-Log(P-value)\"),\n",
            "   legend=list(name=\"\"))\n",
            "```\n\n", # Bottom
            "![](figures/Tagcloud/MeSH_V_",
            i,
            ".png)\n\n")
    }else{
        ""
    }
}

################ MeSH_Z ################
.check_NULL_MeSH_Z_2 <- function(out.vecLR, i){
    if(!is.null(out.vecLR[[i]]$Enrich$MeSH_Z$Pvalue)){
        paste0("## <font color='#1881c2'>MeSH-Enrichment Analysis (Z : Geographical Locations)</font>\n\n",
            "```{r}\n", # Top
            "negLogPval <- -log10(out.vecLR[['",
            i,
            "']]$Enrich$MeSH_Z$Pvalue + 1E-10)\n",
            "term <- out.vecLR[['",
            i,
            "']]$Enrich$MeSH_Z$Term\n",
            "target <- seq_len(min(100, length(negLogPval)))\n",
            "negLogPval <- negLogPval[target]\n",
            "term <- term[target]\n",
            "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
            "type=\"bar\", color=~negLogPval, text=term,\n",
            "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
            "layout(p, title=\"MeSH-Enrichment Analysis (Z)\",\n",
            "   xaxis=list(title=\"Term\"),\n",
            "   yaxis=list(title=\"-Log(P-value)\"),\n",
            "   legend=list(name=\"\"))\n",
            "```\n\n", # Bottom
            "![](figures/Tagcloud/MeSH_Z_",
            i,
            ".png)\n\n")
    }else{
        ""
    }
}

################ Reactome ################
.check_NULL_Reactome_2 <- function(out.vecLR, i){
    if(!is.null(out.vecLR[[i]]$Enrich$Reactome$Pvalue)){
        paste0("## <font color='#1881c2'>Reactome Pathway-Enrichment Analysis</font>\n\n",
            "```{r}\n", # Top
            "negLogPval <- -log10(out.vecLR[['",
            i,
            "']]$Enrich$Reactome$Pvalue + 1E-10)\n",
            "term <- out.vecLR[['",
            i,
            "']]$Enrich$Reactome$Term\n",
            "target <- seq_len(min(100, length(negLogPval)))\n",
            "negLogPval <- negLogPval[target]\n",
            "term <- term[target]\n",
            "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
            "type=\"bar\", color=~negLogPval, text=term,\n",
            "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
            "layout(p, title=\"Reactome-Enrichment Analysis\",\n",
            "   xaxis=list(title=\"Term\"),\n",
            "   yaxis=list(title=\"-Log(P-value)\"),\n",
            "   legend=list(name=\"\"))\n",
            "```\n\n", # Bottom
            "![](figures/Tagcloud/Reactome_",
            i,
            ".png)\n\n")
    }else{
        ""
    }
}

################ DO ################
.check_NULL_DO_2 <- function(out.vecLR, i){
    if(!is.null(out.vecLR[[i]]$Enrich$DO$Pvalue)){
        paste0("## <font color='#1881c2'>DO-Enrichment Analysis</font>\n\n",
            "```{r}\n", # Top
            "negLogPval <- -log10(out.vecLR[['",
            i,
            "']]$Enrich$DO$Pvalue + 1E-10)\n",
            "term <- out.vecLR[['",
            i,
            "']]$Enrich$DO$Term\n",
            "target <- seq_len(min(100, length(negLogPval)))\n",
            "negLogPval <- negLogPval[target]\n",
            "term <- term[target]\n",
            "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
            "type=\"bar\", color=~negLogPval, text=term,\n",
            "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
            "layout(p, title=\"DO-Enrichment Analysis\",\n",
            "   xaxis=list(title=\"Term\"),\n",
            "   yaxis=list(title=\"-Log(P-value)\"),\n",
            "   legend=list(name=\"\"))\n",
            "```\n\n", # Bottom
            "![](figures/Tagcloud/DO_",
            i,
            ".png)\n\n")
    }else{
        ""
    }
}

################ NCG ################
.check_NULL_NCG_2 <- function(out.vecLR, i){
    if(!is.null(out.vecLR[[i]]$Enrich$NCG$Pvalue)){
        paste0("## <font color='#1881c2'>NCG-Enrichment Analysis</font>\n\n",
            "```{r}\n", # Top
            "negLogPval <- -log10(out.vecLR[['",
            i,
            "']]$Enrich$NCG$Pvalue + 1E-10)\n",
            "term <- out.vecLR[['",
            i,
            "']]$Enrich$NCG$Term\n",
            "target <- seq_len(min(100, length(negLogPval)))\n",
            "negLogPval <- negLogPval[target]\n",
            "term <- term[target]\n",
            "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
            "type=\"bar\", color=~negLogPval, text=term,\n",
            "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
            "layout(p, title=\"NCG-Enrichment Analysis\",\n",
            "   xaxis=list(title=\"Term\"),\n",
            "   yaxis=list(title=\"-Log(P-value)\"),\n",
            "   legend=list(name=\"\"))\n",
            "```\n\n", # Bottom
            "![](figures/Tagcloud/NCG_",
            i,
            ".png)\n\n")
    }else{
        ""
    }
}

################ DGN ################
.check_NULL_DGN_2 <- function(out.vecLR, i){
    if(!is.null(out.vecLR[[i]]$Enrich$DGN$Pvalue)){
        paste0("## <font color='#1881c2'>DGN-Enrichment Analysis</font>\n\n",
            "```{r}\n", # Top
            "negLogPval <- -log10(out.vecLR[['",
            i,
            "']]$Enrich$DGN$Pvalue + 1E-10)\n",
            "term <- out.vecLR[['",
            i,
            "']]$Enrich$DGN$Term\n",
            "target <- seq_len(min(100, length(negLogPval)))\n",
            "negLogPval <- negLogPval[target]\n",
            "term <- term[target]\n",
            "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
            "type=\"bar\", color=~negLogPval, text=term,\n",
            "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
            "layout(p, title=\"DGN-Enrichment Analysis\",\n",
            "   xaxis=list(title=\"Term\"),\n",
            "   yaxis=list(title=\"-Log(P-value)\"),\n",
            "   legend=list(name=\"\"))\n",
            "```\n\n", # Bottom
            "![](figures/Tagcloud/DGN_",
            i,
            ".png)\n\n")
    }else{
        ""
    }
}

.XYZ_ENRICH <- function(out.vecLR, i){
    patternName <- paste0("pattern", i)
    go_bp <- .check_NULL_GO_BP(out.vecLR, patternName)
    go_mf <- .check_NULL_GO_MF(out.vecLR, patternName)
    go_cc <- .check_NULL_GO_CC(out.vecLR, patternName)
    mesh_a <- .check_NULL_MeSH_A(out.vecLR, patternName)
    mesh_b <- .check_NULL_MeSH_B(out.vecLR, patternName)
    mesh_c <- .check_NULL_MeSH_C(out.vecLR, patternName)
    mesh_d <- .check_NULL_MeSH_D(out.vecLR, patternName)
    mesh_e <- .check_NULL_MeSH_E(out.vecLR, patternName)
    mesh_f <- .check_NULL_MeSH_F(out.vecLR, patternName)
    mesh_g <- .check_NULL_MeSH_G(out.vecLR, patternName)
    mesh_h <- .check_NULL_MeSH_H(out.vecLR, patternName)
    mesh_i <- .check_NULL_MeSH_I(out.vecLR, patternName)
    mesh_j <- .check_NULL_MeSH_J(out.vecLR, patternName)
    mesh_k <- .check_NULL_MeSH_K(out.vecLR, patternName)
    mesh_l <- .check_NULL_MeSH_L(out.vecLR, patternName)
    mesh_m <- .check_NULL_MeSH_M(out.vecLR, patternName)
    mesh_n <- .check_NULL_MeSH_N(out.vecLR, patternName)
    mesh_v <- .check_NULL_MeSH_V(out.vecLR, patternName)
    mesh_z <- .check_NULL_MeSH_Z(out.vecLR, patternName)
    reactome <- .check_NULL_Reactome(out.vecLR, patternName)
    do <- .check_NULL_DO(out.vecLR, patternName)
    ncg <- .check_NULL_NCG(out.vecLR, patternName)
    dgn <- .check_NULL_DGN(out.vecLR, patternName)
    # Output
    paste(c(go_bp, go_mf, go_cc,
    mesh_a, mesh_b, mesh_c, mesh_d, mesh_e, mesh_f,
    mesh_g, mesh_h, mesh_i, mesh_j, mesh_k, mesh_l,
    mesh_m, mesh_n, mesh_v, mesh_z,
    reactome,
    do, ncg, dgn), collapse="")
}


################ GO_BP ################
.check_NULL_GO_BP <- function(out.vecLR, patternName){
    pval <- eval(parse(text=paste0(
        "out.vecLR[\"Enrich\", \"",
        patternName, "\"][[1]]$GO_BP$Pvalue")))
    if(!is.null(pval)){
        paste0("## <font color='#1881c2'>GO-Enrichment Analysis (BP : Biological Process)</font>\n\n",
        "```{r}\n", # Top
        "negLogPval <- -log10(out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$GO_BP$Pvalue + 1E-10)\n",
        "term <- out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$GO_BP$Term\n",
        "target <- seq_len(min(100, length(negLogPval)))\n",
        "negLogPval <- negLogPval[target]\n",
        "term <- term[target]\n",
        "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
        "type=\"bar\", color=~negLogPval, text=term,\n",
        "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
        "layout(p, title=\"GO-Enrichment Analysis (BP)\",\n",
        "   xaxis=list(title=\"Term\"),\n",
        "   yaxis=list(title=\"-Log(P-value)\"),\n",
        "   legend=list(name=\"\"))\n",
        "```\n\n", # Bottom
        "![](figures/Tagcloud/GO_BP_",
        patternName,
        ".png)\n\n")
    }else{
        ""
    }
}

################ GO_MF ################
.check_NULL_GO_MF <- function(out.vecLR, patternName){
    pval <- eval(parse(text=paste0(
        "out.vecLR[\"Enrich\", \"",
        patternName, "\"][[1]]$GO_MF$Pvalue")))
    if(!is.null(pval)){
        paste0("## <font color='#1881c2'>GO-Enrichment Analysis (MF : Molecular Function)</font>\n\n",
        "```{r}\n", # Top
        "negLogPval <- -log10(out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$GO_MF$Pvalue + 1E-10)\n",
        "term <- out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$GO_MF$Term\n",
        "target <- seq_len(min(100, length(negLogPval)))\n",
        "negLogPval <- negLogPval[target]\n",
        "term <- term[target]\n",
        "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
        "type=\"bar\", color=~negLogPval, text=term,\n",
        "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
        "layout(p, title=\"GO-Enrichment Analysis (MF)\",\n",
        "   xaxis=list(title=\"Term\"),\n",
        "   yaxis=list(title=\"-Log(P-value)\"),\n",
        "   legend=list(name=\"\"))\n",
        "```\n\n", # Bottom
        "![](figures/Tagcloud/GO_MF_",
        patternName,
        ".png)\n\n")
    }else{
        ""
    }
}

################ GO_CC ################
.check_NULL_GO_CC <- function(out.vecLR, patternName){
    pval <- eval(parse(text=paste0(
        "out.vecLR[\"Enrich\", \"",
        patternName, "\"][[1]]$GO_CC$Pvalue")))
    if(!is.null(pval)){
        paste0("## <font color='#1881c2'>GO-Enrichment Analysis (CC : Cellular Component)</font>\n\n",
        "```{r}\n", # Top
        "negLogPval <- -log10(out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$GO_CC$Pvalue + 1E-10)\n",
        "term <- out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$GO_CC$Term\n",
        "target <- seq_len(min(100, length(negLogPval)))\n",
        "negLogPval <- negLogPval[target]\n",
        "term <- term[target]\n",
        "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
        "type=\"bar\", color=~negLogPval, text=term,\n",
        "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
        "layout(p, title=\"GO-Enrichment Analysis (CC)\",\n",
        "   xaxis=list(title=\"Term\"),\n",
        "   yaxis=list(title=\"-Log(P-value)\"),\n",
        "   legend=list(name=\"\"))\n",
        "```\n\n", # Bottom
        "![](figures/Tagcloud/GO_CC_",
        patternName,
        ".png)\n\n")
    }else{
        ""
    }
}

################ MeSH_A ################
.check_NULL_MeSH_A <- function(out.vecLR, patternName){
    pval <- eval(parse(text=paste0(
        "out.vecLR[\"Enrich\", \"",
        patternName, "\"][[1]]$MeSH_A$Pvalue")))
    if(!is.null(pval)){
        paste0("## <font color='#1881c2'>MeSH-Enrichment Analysis (A : Anatomy)</font>\n\n",
        "```{r}\n", # Top
        "negLogPval <- -log10(out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$MeSH_A$Pvalue + 1E-10)\n",
        "term <- out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$MeSH_A$Term\n",
        "target <- seq_len(min(100, length(negLogPval)))\n",
        "negLogPval <- negLogPval[target]\n",
        "term <- term[target]\n",
        "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
        "type=\"bar\", color=~negLogPval, text=term,\n",
        "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
        "layout(p, title=\"MeSH-Enrichment Analysis (A)\",\n",
        "   xaxis=list(title=\"Term\"),\n",
        "   yaxis=list(title=\"-Log(P-value)\"),\n",
        "   legend=list(name=\"\"))\n",
        "```\n\n", # Bottom
        "![](figures/Tagcloud/MeSH_A_",
        patternName,
        ".png)\n\n")
    }else{
        ""
    }
}

################ MeSH_B ################
.check_NULL_MeSH_B <- function(out.vecLR, patternName){
    pval <- eval(parse(text=paste0(
        "out.vecLR[\"Enrich\", \"",
        patternName, "\"][[1]]$MeSH_B$Pvalue")))
    if(!is.null(pval)){
        paste0("## <font color='#1881c2'>MeSH-Enrichment Analysis (B : Organisms)</font>\n\n",
        "```{r}\n", # Top
        "negLogPval <- -log10(out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$MeSH_B$Pvalue + 1E-10)\n",
        "term <- out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$MeSH_B$Term\n",
        "target <- seq_len(min(100, length(negLogPval)))\n",
        "negLogPval <- negLogPval[target]\n",
        "term <- term[target]\n",
        "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
        "type=\"bar\", color=~negLogPval, text=term,\n",
        "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
        "layout(p, title=\"MeSH-Enrichment Analysis (B)\",\n",
        "   xaxis=list(title=\"Term\"),\n",
        "   yaxis=list(title=\"-Log(P-value)\"),\n",
        "   legend=list(name=\"\"))\n",
        "```\n\n", # Bottom
        "![](figures/Tagcloud/MeSH_B_",
        patternName,
        ".png)\n\n")
    }else{
        ""
    }
}

################ MeSH_C ################
.check_NULL_MeSH_C <- function(out.vecLR, patternName){
    pval <- eval(parse(text=paste0(
        "out.vecLR[\"Enrich\", \"",
        patternName, "\"][[1]]$MeSH_C$Pvalue")))
    if(!is.null(pval)){
        paste0("## <font color='#1881c2'>MeSH-Enrichment Analysis (C : Diseases)</font>\n\n",
        "```{r}\n", # Top
        "negLogPval <- -log10(out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$MeSH_C$Pvalue + 1E-10)\n",
        "term <- out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$MeSH_C$Term\n",
        "target <- seq_len(min(100, length(negLogPval)))\n",
        "negLogPval <- negLogPval[target]\n",
        "term <- term[target]\n",
        "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
        "type=\"bar\", color=~negLogPval, text=term,\n",
        "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
        "layout(p, title=\"MeSH-Enrichment Analysis (C)\",\n",
        "   xaxis=list(title=\"Term\"),\n",
        "   yaxis=list(title=\"-Log(P-value)\"),\n",
        "   legend=list(name=\"\"))\n",
        "```\n\n", # Bottom
        "![](figures/Tagcloud/MeSH_C_",
        patternName,
        ".png)\n\n")
    }else{
        ""
    }
}

################ MeSH_D ################
.check_NULL_MeSH_D <- function(out.vecLR, patternName){
    pval <- eval(parse(text=paste0(
        "out.vecLR[\"Enrich\", \"",
        patternName, "\"][[1]]$MeSH_D$Pvalue")))
    if(!is.null(pval)){
        paste0("## <font color='#1881c2'>MeSH-Enrichment Analysis (D : Drugs)</font>\n\n",
        "```{r}\n", # Top
        "negLogPval <- -log10(out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$MeSH_D$Pvalue + 1E-10)\n",
        "term <- out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$MeSH_D$Term\n",
        "target <- seq_len(min(100, length(negLogPval)))\n",
        "negLogPval <- negLogPval[target]\n",
        "term <- term[target]\n",
        "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
        "type=\"bar\", color=~negLogPval, text=term,\n",
        "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
        "layout(p, title=\"MeSH-Enrichment Analysis (D)\",\n",
        "   xaxis=list(title=\"Term\"),\n",
        "   yaxis=list(title=\"-Log(P-value)\"),\n",
        "   legend=list(name=\"\"))\n",
        "```\n\n", # Bottom
        "![](figures/Tagcloud/MeSH_D_",
        patternName,
        ".png)\n\n")
    }else{
        ""
    }
}

################ MeSH_E ################
.check_NULL_MeSH_E <- function(out.vecLR, patternName){
    pval <- eval(parse(text=paste0(
        "out.vecLR[\"Enrich\", \"",
        patternName, "\"][[1]]$MeSH_E$Pvalue")))
    if(!is.null(pval)){
        paste0("## <font color='#1881c2'>MeSH-Enrichment Analysis (E : Analytical, Diagnostic and Therapeutic Techniques and Equipment)</font>\n\n",
        "```{r}\n", # Top
        "negLogPval <- -log10(out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$MeSH_E$Pvalue + 1E-10)\n",
        "term <- out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$MeSH_E$Term\n",
        "target <- seq_len(min(100, length(negLogPval)))\n",
        "negLogPval <- negLogPval[target]\n",
        "term <- term[target]\n",
        "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
        "type=\"bar\", color=~negLogPval, text=term,\n",
        "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
        "layout(p, title=\"MeSH-Enrichment Analysis (E)\",\n",
        "   xaxis=list(title=\"Term\"),\n",
        "   yaxis=list(title=\"-Log(P-value)\"),\n",
        "   legend=list(name=\"\"))\n",
        "```\n\n", # Bottom
        "![](figures/Tagcloud/MeSH_E_",
        patternName,
        ".png)\n\n")
    }else{
        ""
    }
}

################ MeSH_F ################
.check_NULL_MeSH_F <- function(out.vecLR, patternName){
    pval <- eval(parse(text=paste0(
        "out.vecLR[\"Enrich\", \"",
        patternName, "\"][[1]]$MeSH_F$Pvalue")))
    if(!is.null(pval)){
        paste0("## <font color='#1881c2'>MeSH-Enrichment Analysis (F : Psychiatry and Psychology)</font>\n\n",
        "```{r}\n", # Top
        "negLogPval <- -log10(out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$MeSH_F$Pvalue + 1E-10)\n",
        "term <- out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$MeSH_F$Term\n",
        "target <- seq_len(min(100, length(negLogPval)))\n",
        "negLogPval <- negLogPval[target]\n",
        "term <- term[target]\n",
        "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
        "type=\"bar\", color=~negLogPval, text=term,\n",
        "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
        "layout(p, title=\"MeSH-Enrichment Analysis (F)\",\n",
        "   xaxis=list(title=\"Term\"),\n",
        "   yaxis=list(title=\"-Log(P-value)\"),\n",
        "   legend=list(name=\"\"))\n",
        "```\n\n", # Bottom
        "![](figures/Tagcloud/MeSH_F_",
        patternName,
        ".png)\n\n")
    }else{
        ""
    }
}

################ MeSH_G ################
.check_NULL_MeSH_G <- function(out.vecLR, patternName){
    pval <- eval(parse(text=paste0(
        "out.vecLR[\"Enrich\", \"",
        patternName, "\"][[1]]$MeSH_G$Pvalue")))
    if(!is.null(pval)){
        paste0("## <font color='#1881c2'>MeSH-Enrichment Analysis (G : Phenomena and Processes)</font>\n\n",
        "```{r}\n", # Top
        "negLogPval <- -log10(out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$MeSH_G$Pvalue + 1E-10)\n",
        "term <- out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$MeSH_G$Term\n",
        "target <- seq_len(min(100, length(negLogPval)))\n",
        "negLogPval <- negLogPval[target]\n",
        "term <- term[target]\n",
        "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
        "type=\"bar\", color=~negLogPval, text=term,\n",
        "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
        "layout(p, title=\"MeSH-Enrichment Analysis (G)\",\n",
        "   xaxis=list(title=\"Term\"),\n",
        "   yaxis=list(title=\"-Log(P-value)\"),\n",
        "   legend=list(name=\"\"))\n",
        "```\n\n", # Bottom
        "![](figures/Tagcloud/MeSH_G_",
        patternName,
        ".png)\n\n")
    }else{
        ""
    }
}

################ MeSH_H ################
.check_NULL_MeSH_H <- function(out.vecLR, patternName){
    pval <- eval(parse(text=paste0(
        "out.vecLR[\"Enrich\", \"",
        patternName, "\"][[1]]$MeSH_H$Pvalue")))
    if(!is.null(pval)){
        paste0("## <font color='#1881c2'>MeSH-Enrichment Analysis (H : Disciplines and Occupations)</font>\n\n",
        "```{r}\n", # Top
        "negLogPval <- -log10(out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$MeSH_H$Pvalue + 1E-10)\n",
        "term <- out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$MeSH_H$Term\n",
        "target <- seq_len(min(100, length(negLogPval)))\n",
        "negLogPval <- negLogPval[target]\n",
        "term <- term[target]\n",
        "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
        "type=\"bar\", color=~negLogPval, text=term,\n",
        "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
        "layout(p, title=\"MeSH-Enrichment Analysis (H)\",\n",
        "   xaxis=list(title=\"Term\"),\n",
        "   yaxis=list(title=\"-Log(P-value)\"),\n",
        "   legend=list(name=\"\"))\n",
        "```\n\n", # Bottom
        "![](figures/Tagcloud/MeSH_H_",
        patternName,
        ".png)\n\n")
    }else{
        ""
    }
}

################ MeSH_I ################
.check_NULL_MeSH_I <- function(out.vecLR, patternName){
    pval <- eval(parse(text=paste0(
        "out.vecLR[\"Enrich\", \"",
        patternName, "\"][[1]]$MeSH_I$Pvalue")))
    if(!is.null(pval)){
        paste0("## <font color='#1881c2'>MeSH-Enrichment Analysis (I : Anthropology, Education, Sociology and Social Phenomena)</font>\n\n",
        "```{r}\n", # Top
        "negLogPval <- -log10(out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$MeSH_I$Pvalue + 1E-10)\n",
        "term <- out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$MeSH_I$Term\n",
        "target <- seq_len(min(100, length(negLogPval)))\n",
        "negLogPval <- negLogPval[target]\n",
        "term <- term[target]\n",
        "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
        "type=\"bar\", color=~negLogPval, text=term,\n",
        "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
        "layout(p, title=\"MeSH-Enrichment Analysis (I)\",\n",
        "   xaxis=list(title=\"Term\"),\n",
        "   yaxis=list(title=\"-Log(P-value)\"),\n",
        "   legend=list(name=\"\"))\n",
        "```\n\n", # Bottom
        "![](figures/Tagcloud/MeSH_I_",
        patternName,
        ".png)\n\n")
    }else{
        ""
    }
}

################ MeSH_J ################
.check_NULL_MeSH_J <- function(out.vecLR, patternName){
    pval <- eval(parse(text=paste0(
        "out.vecLR[\"Enrich\", \"",
        patternName, "\"][[1]]$MeSH_J$Pvalue")))
    if(!is.null(pval)){
        paste0("## <font color='#1881c2'>MeSH-Enrichment Analysis (J : Technology and Food and Beverages)</font>\n\n",
        "```{r}\n", # Top
        "negLogPval <- -log10(out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$MeSH_J$Pvalue + 1E-10)\n",
        "term <- out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$MeSH_J$Term\n",
        "target <- seq_len(min(100, length(negLogPval)))\n",
        "negLogPval <- negLogPval[target]\n",
        "term <- term[target]\n",
        "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
        "type=\"bar\", color=~negLogPval, text=term,\n",
        "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
        "layout(p, title=\"MeSH-Enrichment Analysis (J)\",\n",
        "   xaxis=list(title=\"Term\"),\n",
        "   yaxis=list(title=\"-Log(P-value)\"),\n",
        "   legend=list(name=\"\"))\n",
        "```\n\n", # Bottom
        "![](figures/Tagcloud/MeSH_J_",
        patternName,
        ".png)\n\n")
    }else{
        ""
    }
}

################ MeSH_K ################
.check_NULL_MeSH_K <- function(out.vecLR, patternName){
    pval <- eval(parse(text=paste0(
        "out.vecLR[\"Enrich\", \"",
        patternName, "\"][[1]]$MeSH_K$Pvalue")))
    if(!is.null(pval)){
        paste0("## <font color='#1881c2'>MeSH-Enrichment Analysis (K : Humanities)</font>\n\n",
        "```{r}\n", # Top
        "negLogPval <- -log10(out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$MeSH_K$Pvalue + 1E-10)\n",
        "term <- out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$MeSH_K$Term\n",
        "target <- seq_len(min(100, length(negLogPval)))\n",
        "negLogPval <- negLogPval[target]\n",
        "term <- term[target]\n",
        "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
        "type=\"bar\", color=~negLogPval, text=term,\n",
        "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
        "layout(p, title=\"MeSH-Enrichment Analysis (K)\",\n",
        "   xaxis=list(title=\"Term\"),\n",
        "   yaxis=list(title=\"-Log(P-value)\"),\n",
        "   legend=list(name=\"\"))\n",
        "```\n\n", # Bottom
        "![](figures/Tagcloud/MeSH_K_",
        patternName,
        ".png)\n\n")
    }else{
        ""
    }
}

################ MeSH_L ################
.check_NULL_MeSH_L <- function(out.vecLR, patternName){
    pval <- eval(parse(text=paste0(
        "out.vecLR[\"Enrich\", \"",
        patternName, "\"][[1]]$MeSH_L$Pvalue")))
    if(!is.null(pval)){
        paste0("## <font color='#1881c2'>MeSH-Enrichment Analysis (L : Information Science)</font>\n\n",
        "```{r}\n", # Top
        "negLogPval <- -log10(out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$MeSH_L$Pvalue + 1E-10)\n",
        "term <- out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$MeSH_L$Term\n",
        "target <- seq_len(min(100, length(negLogPval)))\n",
        "negLogPval <- negLogPval[target]\n",
        "term <- term[target]\n",
        "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
        "type=\"bar\", color=~negLogPval, text=term,\n",
        "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
        "layout(p, title=\"MeSH-Enrichment Analysis (L)\",\n",
        "   xaxis=list(title=\"Term\"),\n",
        "   yaxis=list(title=\"-Log(P-value)\"),\n",
        "   legend=list(name=\"\"))\n",
        "```\n\n", # Bottom
        "![](figures/Tagcloud/MeSH_L_",
        patternName,
        ".png)\n\n")
    }else{
        ""
    }
}

################ MeSH_M ################
.check_NULL_MeSH_M <- function(out.vecLR, patternName){
    pval <- eval(parse(text=paste0(
        "out.vecLR[\"Enrich\", \"",
        patternName, "\"][[1]]$MeSH_M$Pvalue")))
    if(!is.null(pval)){
        paste0("## <font color='#1881c2'>MeSH-Enrichment Analysis (M : Persons)</font>\n\n",
        "```{r}\n", # Top
        "negLogPval <- -log10(out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$MeSH_M$Pvalue + 1E-10)\n",
        "term <- out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$MeSH_M$Term\n",
        "target <- seq_len(min(100, length(negLogPval)))\n",
        "negLogPval <- negLogPval[target]\n",
        "term <- term[target]\n",
        "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
        "type=\"bar\", color=~negLogPval, text=term,\n",
        "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
        "layout(p, title=\"MeSH-Enrichment Analysis (M)\",\n",
        "   xaxis=list(title=\"Term\"),\n",
        "   yaxis=list(title=\"-Log(P-value)\"),\n",
        "   legend=list(name=\"\"))\n",
        "```\n\n", # Bottom
        "![](figures/Tagcloud/MeSH_M_",
        patternName,
        ".png)\n\n")
    }else{
        ""
    }
}

################ MeSH_N ################
.check_NULL_MeSH_N <- function(out.vecLR, patternName){
    pval <- eval(parse(text=paste0(
        "out.vecLR[\"Enrich\", \"",
        patternName, "\"][[1]]$MeSH_N$Pvalue")))
    if(!is.null(pval)){
        paste0("## <font color='#1881c2'>MeSH-Enrichment Analysis (N : Health Care)</font>\n\n",
        "```{r}\n", # Top
        "negLogPval <- -log10(out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$MeSH_N$Pvalue + 1E-10)\n",
        "term <- out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$MeSH_N$Term\n",
        "target <- seq_len(min(100, length(negLogPval)))\n",
        "negLogPval <- negLogPval[target]\n",
        "term <- term[target]\n",
        "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
        "type=\"bar\", color=~negLogPval, text=term,\n",
        "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
        "layout(p, title=\"MeSH-Enrichment Analysis (N)\",\n",
        "   xaxis=list(title=\"Term\"),\n",
        "   yaxis=list(title=\"-Log(P-value)\"),\n",
        "   legend=list(name=\"\"))\n",
        "```\n\n", # Bottom
        "![](figures/Tagcloud/MeSH_N_",
        patternName,
        ".png)\n\n")
    }else{
        ""
    }
}

################ MeSH_V ################
.check_NULL_MeSH_V <- function(out.vecLR, patternName){
    pval <- eval(parse(text=paste0(
        "out.vecLR[\"Enrich\", \"",
        patternName, "\"][[1]]$MeSH_V$Pvalue")))
    if(!is.null(pval)){
        paste0("## <font color='#1881c2'>MeSH-Enrichment Analysis (V : Publication Type)</font>\n\n",
        "```{r}\n", # Top
        "negLogPval <- -log10(out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$MeSH_V$Pvalue + 1E-10)\n",
        "term <- out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$MeSH_V$Term\n",
        "target <- seq_len(min(100, length(negLogPval)))\n",
        "negLogPval <- negLogPval[target]\n",
        "term <- term[target]\n",
        "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
        "type=\"bar\", color=~negLogPval, text=term,\n",
        "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
        "layout(p, title=\"MeSH-Enrichment Analysis (V)\",\n",
        "   xaxis=list(title=\"Term\"),\n",
        "   yaxis=list(title=\"-Log(P-value)\"),\n",
        "   legend=list(name=\"\"))\n",
        "```\n\n", # Bottom
        "![](figures/Tagcloud/MeSH_V_",
        patternName,
        ".png)\n\n")
    }else{
        ""
    }
}

################ MeSH_Z ################
.check_NULL_MeSH_Z <- function(out.vecLR, patternName){
    pval <- eval(parse(text=paste0(
        "out.vecLR[\"Enrich\", \"",
        patternName, "\"][[1]]$MeSH_Z$Pvalue")))
    if(!is.null(pval)){
        paste0("## <font color='#1881c2'>MeSH-Enrichment Analysis (Z : Geographical Locations)</font>\n\n",
        "```{r}\n", # Top
        "negLogPval <- -log10(out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$MeSH_Z$Pvalue + 1E-10)\n",
        "term <- out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$MeSH_Z$Term\n",
        "target <- seq_len(min(100, length(negLogPval)))\n",
        "negLogPval <- negLogPval[target]\n",
        "term <- term[target]\n",
        "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
        "type=\"bar\", color=~negLogPval, text=term,\n",
        "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
        "layout(p, title=\"MeSH-Enrichment Analysis (Z)\",\n",
        "   xaxis=list(title=\"Term\"),\n",
        "   yaxis=list(title=\"-Log(P-value)\"),\n",
        "   legend=list(name=\"\"))\n",
        "```\n\n", # Bottom
        "![](figures/Tagcloud/MeSH_Z_",
        patternName,
        ".png)\n\n")
    }else{
        ""
    }
}

################ Reactome ################
.check_NULL_Reactome <- function(out.vecLR, patternName){
    pval <- eval(parse(text=paste0(
        "out.vecLR[\"Enrich\", \"",
        patternName, "\"][[1]]$Reactome$Pvalue")))
    if(!is.null(pval)){
        paste0("## <font color='#1881c2'>Reactome Pathway-Enrichment Analysis</font>\n\n",
        "```{r}\n", # Top
        "negLogPval <- -log10(out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$Reactome$Pvalue + 1E-10)\n",
        "term <- out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$Reactome$Term\n",
        "target <- seq_len(min(100, length(negLogPval)))\n",
        "negLogPval <- negLogPval[target]\n",
        "term <- term[target]\n",
        "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
        "type=\"bar\", color=~negLogPval, text=term,\n",
        "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
        "layout(p, title=\"Reactome-Enrichment Analysis\",\n",
        "   xaxis=list(title=\"Term\"),\n",
        "   yaxis=list(title=\"-Log(P-value)\"),\n",
        "   legend=list(name=\"\"))\n",
        "```\n\n", # Bottom
        "![](figures/Tagcloud/Reactome_",
        patternName,
        ".png)\n\n")
    }else{
        ""
    }
}

################ DO ################
.check_NULL_DO <- function(out.vecLR, patternName){
    pval <- eval(parse(text=paste0(
        "out.vecLR[\"Enrich\", \"",
        patternName, "\"][[1]]$DO$Pvalue")))
    if(!is.null(pval)){
        paste0("## <font color='#1881c2'>DO-Enrichment Analysis</font>\n\n",
        "```{r}\n", # Top
        "negLogPval <- -log10(out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$DO$Pvalue + 1E-10)\n",
        "term <- out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$DO$Term\n",
        "target <- seq_len(min(100, length(negLogPval)))\n",
        "negLogPval <- negLogPval[target]\n",
        "term <- term[target]\n",
        "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
        "type=\"bar\", color=~negLogPval, text=term,\n",
        "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
        "layout(p, title=\"DO-Enrichment Analysis\",\n",
        "   xaxis=list(title=\"Term\"),\n",
        "   yaxis=list(title=\"-Log(P-value)\"),\n",
        "   legend=list(name=\"\"))\n",
        "```\n\n", # Bottom
        "![](figures/Tagcloud/DO_",
        patternName,
        ".png)\n\n")
    }else{
        ""
    }
}

################ NCG ################
.check_NULL_NCG <- function(out.vecLR, patternName){
    pval <- eval(parse(text=paste0(
        "out.vecLR[\"Enrich\", \"",
        patternName, "\"][[1]]$NCG$Pvalue")))
    if(!is.null(pval)){
        paste0("## <font color='#1881c2'>NCG-Enrichment Analysis</font>\n\n",
        "```{r}\n", # Top
        "negLogPval <- -log10(out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$NCG$Pvalue + 1E-10)\n",
        "term <- out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$NCG$Term\n",
        "target <- seq_len(min(100, length(negLogPval)))\n",
        "negLogPval <- negLogPval[target]\n",
        "term <- term[target]\n",
        "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
        "type=\"bar\", color=~negLogPval, text=term,\n",
        "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
        "layout(p, title=\"NCG-Enrichment Analysis\",\n",
        "   xaxis=list(title=\"Term\"),\n",
        "   yaxis=list(title=\"-Log(P-value)\"),\n",
        "   legend=list(name=\"\"))\n",
        "```\n\n", # Bottom
        "![](figures/Tagcloud/NCG_",
        patternName,
        ".png)\n\n")
    }else{
        ""
    }
}

################ DGN ################
.check_NULL_DGN <- function(out.vecLR, patternName){
    pval <- eval(parse(text=paste0(
        "out.vecLR[\"Enrich\", \"",
        patternName, "\"][[1]]$DGN$Pvalue")))
    if(!is.null(pval)){
        paste0("## <font color='#1881c2'>DGN-Enrichment Analysis</font>\n\n",
        "```{r}\n", # Top
        "negLogPval <- -log10(out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$DGN$Pvalue + 1E-10)\n",
        "term <- out.vecLR[\"Enrich\", \"",
        patternName,
        "\"][[1]]$DGN$Term\n",
        "target <- seq_len(min(100, length(negLogPval)))\n",
        "negLogPval <- negLogPval[target]\n",
        "term <- term[target]\n",
        "p <- plot_ly(x=seq_along(negLogPval), y=~negLogPval,\n",
        "type=\"bar\", color=~negLogPval, text=term,\n",
        "colors=c(\"#4b61ba\", \"gray\", \"#a87963\", \"red\"))\n",
        "layout(p, title=\"DGN-Enrichment Analysis\",\n",
        "   xaxis=list(title=\"Term\"),\n",
        "   yaxis=list(title=\"-Log(P-value)\"),\n",
        "   legend=list(name=\"\"))\n",
        "```\n\n", # Bottom
        "![](figures/Tagcloud/DGN_",
        patternName,
        ".png)\n\n")
    }else{
        ""
    }
}

.XYZ_HEADER3 <- function(i){
    paste0("# <font color='#1881c2'>(\\*,\\*,", i,
        ") Pattern-related Enrichment Analysis</font>\n\n",
        "```{r}\n", # Top
        "load(\"reanalysis.RData\")\n",
        "library(\"plotly\")\n",
        "```\n\n" # Bottom
        )
}

.XYZ_HEADER3_2 <- function(index, i){
        paste0("# <font color='#1881c2'>(",
        paste(c(index[i, seq_len(2)], ""), collapse=","),
        ") -related Enrichment Analysis</font>\n\n",
        "```{r}\n", # Top
        "load(\"reanalysis.RData\")\n",
        "library(\"plotly\")\n",
        "```\n\n" # Bottom
        )
}

## ----cellCellSimulate_Default, echo=TRUE--------------------------------------
suppressPackageStartupMessages(library("scTensor"))
sim <- cellCellSimulate()

## ----cellCellSimulate_Setting, echo=TRUE--------------------------------------
# Default parameters
params <- newCCSParams()
str(params)

# Setting different parameters
# No. of genes : 1000
setParam(params, "nGene") <- 1000
# 3 cell types, 20 cells in each cell type
setParam(params, "nCell") <- c(20, 20, 20)
# Setting for Ligand-Receptor pair list
setParam(params, "cciInfo") <- list(
    nPair=500, # Total number of L-R pairs
    # 1st CCI
    CCI1=list(
        LPattern=c(1,0,0), # Only 1st cell type has this pattern
        RPattern=c(0,1,0), # Only 2nd cell type has this pattern
        nGene=50, # 50 pairs are generated as CCI1
        fc="E10"), # Degree of differential expression (Fold Change)
    # 2nd CCI
    CCI2=list(
        LPattern=c(0,1,0),
        RPattern=c(0,0,1),
        nGene=30,
        fc="E100")
    )
# Degree of Dropout
setParam(params, "lambda") <- 10
# Random number seed
setParam(params, "seed") <- 123

# Simulation data
sim <- cellCellSimulate(params)

## ----input, echo=TRUE---------------------------------------------------------
dim(sim$input)
sim$input[1:2,1:3]

## ----LR, echo=TRUE------------------------------------------------------------
dim(sim$LR)
sim$LR[1:10,]
sim$LR[46:55,]
sim$LR[491:500,]

## ----celltypes, echo=TRUE-----------------------------------------------------
length(sim$celltypes)
head(sim$celltypes)
table(names(sim$celltypes))

## ----sessionInfo, echo=FALSE--------------------------------------------------
sessionInfo()

## ----AHub1, echo=TRUE---------------------------------------------------------
library("AnnotationHub")
ah <- AnnotationHub()
mcols(ah)

## ----AHub2, echo=TRUE---------------------------------------------------------
dbfile <- query(ah, c("LRBaseDb", "Homo sapiens"))[[1]]

## ----AHub3, echo=TRUE---------------------------------------------------------
library("LRBaseDbi")
LRBase.Hsa.eg.db <- LRBaseDbi::LRBaseDb(dbfile)

## ----Access, echo=TRUE--------------------------------------------------------
columns(LRBase.Hsa.eg.db)
keytypes(LRBase.Hsa.eg.db)
key_HSA <- keys(LRBase.Hsa.eg.db, keytype="GENEID_L")
head(select(LRBase.Hsa.eg.db, keys=key_HSA[1:2],
            columns=c("GENEID_L", "GENEID_R"), keytype="GENEID_L"))

## ----Other1, echo=TRUE--------------------------------------------------------
lrNomenclature(LRBase.Hsa.eg.db)
species(LRBase.Hsa.eg.db)
lrListDatabases(LRBase.Hsa.eg.db)
lrVersion(LRBase.Hsa.eg.db)

dbInfo(LRBase.Hsa.eg.db)
dbfile(LRBase.Hsa.eg.db)
dbschema(LRBase.Hsa.eg.db)
dbconn(LRBase.Hsa.eg.db)

## ----Other2, echo=TRUE--------------------------------------------------------
suppressPackageStartupMessages(library("RSQLite"))
dbGetQuery(dbconn(LRBase.Hsa.eg.db),
  "SELECT * FROM DATA WHERE GENEID_L = '9068' AND GENEID_R = '14' LIMIT 10")

## ----SCE1, echo=TRUE----------------------------------------------------------
suppressPackageStartupMessages(library("scTensor"))
suppressPackageStartupMessages(library("SingleCellExperiment"))

## ----SCE2, fig.cap="Germline, Male, GSE86146", echo=TRUE, fig.width=10, fig.height=10----
data(GermMale)
data(labelGermMale)
data(tsneGermMale)

sce <- SingleCellExperiment(assays=list(counts = GermMale))
reducedDims(sce) <- SimpleList(TSNE=tsneGermMale$Y)
plot(reducedDims(sce)[[1]], col=labelGermMale, pch=16, cex=2,
  xlab="Dim1", ylab="Dim2", main="Germline, Male, GSE86146")
legend("topleft", legend=c(paste0("FGC_", 1:3), paste0("Soma_", 1:4)),
  col=c("#9E0142", "#D53E4F", "#F46D43", "#ABDDA4", "#66C2A5", "#3288BD", "#5E4FA2"),
  pch=16)

## ----cellCellSetting, echo=TRUE-----------------------------------------------
cellCellSetting(sce, LRBase.Hsa.eg.db, names(labelGermMale))

## ----cellCellDecomp, echo=TRUE------------------------------------------------
set.seed(1234)
cellCellDecomp(sce, ranks=c(2,3))

## ----cellCellRank, echo=TRUE--------------------------------------------------
(rks <- cellCellRanks(sce))
rks$selected

## ----sessionInfo, echo=FALSE--------------------------------------------------
sessionInfo()

## ----reanalysis.RData, eval=FALSE---------------------------------------------
# library("scTensor")
# load("reanalysis.RData")

## ----Reanalysis, eval=FALSE---------------------------------------------------
# library("AnnotationHub")
# library("LRBaseDbi")
# 
# # Create LRBase object
# ah <- AnnotationHub()
# dbfile <- query(ah, c("LRBaseDb", "Homo sapiens"))[[1]]
# LRBase.Hsa.eg.db <- LRBaseDbi::LRBaseDb(dbfile)
# 
# # Register the file pass of user's LRBase
# metadata(sce)$lrbase <- dbfile(LRBase.Hsa.eg.db)
# 
# # CCI Tensor Decomposition
# cellCellDecomp(sce, ranks=c(6,5), assayNames="normcounts")
# 
# # HTML Report
# cellCellReport(sce, reducedDimNames="TSNE", assayNames="normcounts",
#     title="Cell-cell interaction within Germline_Male, GSE86146",
#     author="Koki Tsuyuzaki", html.open=TRUE,
#     goenrich=TRUE, meshenrich=FALSE, reactomeenrich=FALSE,
#     doenrich=FALSE, ncgenrich=FALSE, dgnenrich=FALSE)

## ----sessionInfo, echo=FALSE--------------------------------------------------
sessionInfo()

setClass("CCSParams",
    slots = c(
        nGene = "numeric",
        nCell = "numeric",
        cciInfo = "list",
        lambda = "numeric",
        seed = "numeric"
        ),
    prototype = prototype(
        nGene=1000,
        nCell=c(50, 50, 50),
        cciInfo=list(
            nPair=500,
            CCI1=list(
                LPattern=c(1,0,0),
                RPattern=c(0,1,0),
                nGene=50,
                fc="E10"),
            CCI2=list(
                LPattern=c(0,1,0),
                RPattern=c(0,0,1),
                nGene=50,
                fc="E10"),
            CCI3=list(
                LPattern=c(0,0,1),
                RPattern=c(1,0,0),
                nGene=50,
                fc="E10")
            ),
        lambda = 1,
        seed = 1234
    )
)

## ----scTensor with NCBI Gene ID, eval=FALSE-----------------------------------
# library("scTensor")
# library("AnnotationHub")
# library("LRBaseDbi")
# 
# # Input matrix
# input <- ...
# sce <- SingleCellExperiment(assays=list(counts = input))
# # Celltype vector
# label <- ...
# # LRBase.XXX.eg.db
# ah <- AnnotationHub()
# dbfile <- query(ah, c("LRBaseDb", "Homo sapiens"))[[1]]
# LRBase.Hsa.eg.db <- LRBaseDbi::LRBaseDb(dbfile)
# # Setting
# cellCellSetting(sce, LRBase.Hsa.eg.db, label)

## ----Seurat, eval=FALSE-------------------------------------------------------
# if(!require(Seurat)){
#     BiocManager::install("Seurat")
#     library(Seurat)
# }
# 
# # Load the PBMC dataset
# pbmc.data <- Read10X(data.dir = "filtered_gene_bc_matrices/hg19/")
# 
# # Initialize the Seurat object with the raw (non-normalized data).
# pbmc <- CreateSeuratObject(counts = pbmc.data,
#   project = "pbmc3k", min.cells = 3, min.features = 200)

## ----Ensembl with Organism DB, echo=TRUE--------------------------------------
suppressPackageStartupMessages(library("scTensor"))
if(!require(Homo.sapiens)){
    BiocManager::install("Homo.sapiens")
    suppressPackageStartupMessages(library(Homo.sapiens))
}
if(!require(scTGIF)){
    BiocManager::install("scTGIF")
    suppressPackageStartupMessages(library(scTGIF))
}

# 1. Input matrix
input <- matrix(1:20, nrow=4, ncol=5)
# 2. Gene identifier in each row
rowID <- c("ENSG00000204531", "ENSG00000181449",
  "ENSG00000136997", "ENSG00000136826")
# 3. Corresponding table
LefttoRight <- select(Homo.sapiens,
  column=c("ENSEMBL", "ENTREZID"),
  keytype="ENSEMBL", keys=rowID)
# ID conversion
(input <- convertRowID(input, rowID, LefttoRight))

## ----Ensembl with AnnotationHub, echo=TRUE------------------------------------
suppressPackageStartupMessages(library("AnnotationHub"))

# 1. Input matrix
input <- matrix(1:20, nrow=4, ncol=5)
# 3. Corresponding table
ah <- AnnotationHub()
# Database of Human
hs <- query(ah, c("OrgDb", "Homo sapiens"))[[1]]
LefttoRight <- select(hs,
  column=c("ENSEMBL", "ENTREZID"),
  keytype="ENSEMBL", keys=rowID)
(input <- convertRowID(input, rowID, LefttoRight))

## ----Gene Symbol with Organism DB, echo=TRUE----------------------------------
# 1. Input matrix
input <- matrix(1:20, nrow=4, ncol=5)
# 2. Gene identifier in each row
rowID <- c("POU5F1", "SOX2", "MYC", "KLF4")
# 3. Corresponding table
LefttoRight <- select(Homo.sapiens,
  column=c("SYMBOL", "ENTREZID"),
  keytype="SYMBOL", keys=rowID)
# ID conversion
(input <- convertRowID(input, rowID, LefttoRight))

## ----Gene Symbol with AnnotationHub, echo=TRUE--------------------------------
# 1. Input matrix
input <- matrix(1:20, nrow=4, ncol=5)
# 3. Corresponding table
ah <- AnnotationHub()
# Database of Human
hs <- query(ah, c("OrgDb", "Homo sapiens"))[[1]]
LefttoRight <- select(hs,
  column=c("SYMBOL", "ENTREZID"),
  keytype="SYMBOL", keys=rowID)
(input <- convertRowID(input, rowID, LefttoRight))

## ----Seurat normalization, eval=FALSE-----------------------------------------
# pbmc2 <- NormalizeData(pbmc, normalization.method = "LogNormalize",
#     scale.factor = 10000)
# sce <- as.SingleCellExperiment(pbmc2)
# assayNames(sce) # counts, logcounts

## ----Scater normalization, eval=FALSE-----------------------------------------
# if(!require(scater)){
#     BiocManager::install("scater")
#     library(scater)
# }
# sce <- SingleCellExperiment(assays=list(counts = input))
# cpm(sce) <- calculateCPM(sce)
# sce <- normalize(sce)
# assayNames(sce) # counts, normcounts, logcounts, cpm

## ----Original normalization, eval=FALSE---------------------------------------
# # User's Original Normalization Function
# CPMED <- function(input){
#     libsize <- colSums(input)
#     median(libsize) * t(t(input) / libsize)
# }
# # Normalization
# normcounts(sce) <- log10(CPMED(counts(sce)) + 1)

## ----sessionInfo, echo=FALSE--------------------------------------------------
sessionInfo()

.cellCellReport.Third_2 <- function(sce, thr, upper, assayNames, reducedDimNames, out.dir, author, title, p, top,
    goenrich, meshenrich, reactomeenrich,
    doenrich, ncgenrich, dgnenrich, nbins){
    # Core Tensor
    index <- metadata(sce)$sctensor$index
    corevalue <- index[, "Value"]
    corevalue <- corevalue / sum(corevalue) * 100
    # Thresholding of the elements of core tensor
    selected <- which(cumsum(corevalue) <= thr)
    if(length(selected) > upper){
        selected <- seq_len(upper)
    }
    if(length(selected) == 0){
        stop(paste0("None of core tensor element is selected.\n",
        "Please specify the larger thr or perform cellCellDecomp\n",
        "with larger ranks such as c(# celltypes,# celltypes,1)."))
    }else{
        names(corevalue) <- c(rep("selected", length=length(selected)),
            rep("not selected",
            length=length(corevalue) - length(selected)))
        # Import expression matrix
        input <- .importAssays(sce, assayNames)
        # Low dimensional data
        twoD <- eval(parse(text=paste0("reducedDims(sce)$", reducedDimNames)))
        # Ligand-Receptor, PMID
        lr.evidence <- metadata(sce)$lr.evidence
        LR <- .extractLR(sce, lr.evidence,
            c("GENEID_L", "GENEID_R", "SOURCEID", "SOURCEDB"))
        # SQLite connection
        con = dbConnect(SQLite(), metadata(sce)$lrbase)
        taxid <- dbGetQuery(con, "SELECT * FROM METADATA")
        taxid <- taxid[which(taxid$NAME == "TAXID"), "VALUE"]
        dbDisconnect(con)
        ###########################################
        # Taxonomy ID based information retrieval
        ###########################################
        # biomaRt Setting
        ah <- .annotationhub_taxid(taxid)
        # GeneName, Description, GO, Reactome, MeSH
        GeneInfo <- .geneInformation_taxid(sce, ah, taxid, LR)
        # Cell Label
        celltypes <- metadata(sce)$color
        names(celltypes) <- metadata(sce)$label
        # Setting of schex
        sce <- make_hexbin(sce, nbins=nbins,
            dimension_reduction=reducedDimNames)
        # Plot Ligand/Receptor Genes
        suppressMessages(
            invisible(.genePlot(sce, assayNames, input, out.dir, GeneInfo, LR)))
        # Plot (Each <L,R,*>)
        out <- vapply(seq_along(selected), function(i){
            filenames <- paste0(out.dir,
                "/figures/CCIHypergraph_", index[i, 1],
                "_", index[i, 2], ".png")
            png(filename=filenames, width=2000, height=950)
            invisible(.CCIhyperGraphPlot_2(metadata(sce)$sctensor,
                twoDplot=twoD,
                label=celltypes,
                emph=index[i, seq_len(2)]))
            dev.off()
        }, 0L)
        # <L,R,>
        SelectedLR <- index[selected, seq_len(2)]
        # Setting for Parallel Computing
        message(paste0(length(selected),
            " LR vectors will be calculated :"))
        e <<- new.env()
        e$p <- p
        e$index <- index
        e$sce <- sce
        e$ah <- ah
        e$.HCLUST <- .HCLUST
        e$.OUTLIERS <- .OUTLIERS
        e$top <- top
        e$GeneInfo <- GeneInfo
        e$out.dir <- out.dir
        e$.smallTwoDplot <- .smallTwoDplot
        e$input <- input
        e$twoD <- twoD
        e$.hyperLinks <- .hyperLinks
        e$LR <- LR
        e$taxid <- taxid
        e$.eachVecLR <- .eachVecLR
        e$.eachRender_2 <- .eachRender_2
        e$.XYZ_HEADER1_2 <- .XYZ_HEADER1_2
        e$.XYZ_HEADER2_2 <- .XYZ_HEADER2_2
        e$.XYZ_HEADER3_2 <- .XYZ_HEADER3_2
        e$.XYZ_ENRICH_2 <- .XYZ_ENRICH_2
        e$algorithm <- metadata(sce)$algorithm
        e$goenrich <- goenrich
        e$meshenrich <- meshenrich
        e$reactomeenrich <- reactomeenrich
        e$doenrich <- doenrich
        e$ncgenrich <- ncgenrich
        e$dgnenrich <- dgnenrich

        # EachVec（Heavy...）
        if(is.vector(SelectedLR)){
            SelectedLR <- t(SelectedLR)
        }
        out.vecLR <- apply(SelectedLR, 1,
            function(x, e){.eachVecLR(x, e)}, e=e)
        names(out.vecLR) <- apply(SelectedLR, 1, function(x){
            paste(c("pattern", x[1], x[2]), collapse="_")
        })
        e$out.vecLR <- out.vecLR

        # Tagcloud
        invisible(.tagCloud_2(out.vecLR, out.dir))
        # Plot（CCI Hypergraph）
        png(filename=paste0(out.dir, "/figures/CCIHypergraph.png"),
            width=2000, height=950)
        invisible(.CCIhyperGraphPlot_2(metadata(sce)$sctensor,
            twoDplot=twoD, label=celltypes))
        dev.off()
        # Plot（Gene-wise Hypergraph）
        invisible(g <- .geneHyperGraphPlot_2(out.vecLR, GeneInfo, out.dir))

        # Rmd（ligand, selected）
        message("ligand.Rmd is created...")
        outLg <- file(paste0(out.dir, "/ligand.Rmd"), "w")
        writeLines(.LIGAND_HEADER, outLg, sep="\n")
        writeLines(.LIGAND_BODY_2(out.vecLR, GeneInfo, index, selected), outLg, sep="\n")
        close(outLg)
        # Rmd（receptor, selected）
        message("receptor.Rmd is created...")
        outRp <- file(paste0(out.dir, "/receptor.Rmd"), "w")
        writeLines(.RECEPTOR_HEADER, outRp, sep="\n")
        writeLines(.RECEPTOR_BODY_2(out.vecLR, GeneInfo, index, selected), outRp, sep="\n")
        close(outRp)
        # Rmd（ligand, all）
        message("ligand_all.Rmd is created...")
        outLg_all <- file(paste0(out.dir, "/ligand_all.Rmd"), "w")
        writeLines(.LIGANDALL_HEADER, outLg_all, sep="\n")
        writeLines(.LIGANDALL_BODY(GeneInfo, LR, input),
            outLg_all, sep="\n")
        close(outLg_all)
        # Rmd（receptor, all）
        message("receptor_all.Rmd is created...")
        outRp_all <- file(paste0(out.dir, "/receptor_all.Rmd"), "w")
        writeLines(.RECEPTORALL_HEADER, outRp_all, sep="\n")
        writeLines(.RECEPTORALL_BODY(GeneInfo, LR, input),
            outRp_all, sep="\n")
        close(outRp_all)

        # Number of Patterns
        vecL <- metadata(sce)$sctensor$ligand
        vecR <- metadata(sce)$sctensor$receptor
        numLPattern <- nrow(vecL)
        numRPattern <- nrow(vecR)
        col.ligand <- .setColor("reds")
        col.receptor <- .setColor("blues")
        # Clustering
        ClusterL <- t(apply(vecL, 1, .HCLUST))
        ClusterR <- t(apply(vecR, 1, .HCLUST))
        # Ligand Pattern
        invisible(.ligandPatternPlot(numLPattern, celltypes, sce, col.ligand, ClusterL, out.dir, twoD))
        # Receptor Pattern
        invisible(.receptorPatternPlot(numRPattern, celltypes, sce,
            col.receptor, ClusterR, out.dir, twoD))
        # Save the result of scTensor
        save(sce, input, twoD, LR, celltypes, index, corevalue,
            selected, ClusterL, ClusterR, out.vecLR, g,
            file=paste0(out.dir, "/reanalysis.RData"))

        # Rendering
        message("ligand.Rmd is compiled to ligand.html...")
        render(paste0(out.dir, "/ligand.Rmd"), quiet=TRUE)
        message("ligand_all.Rmd is compiled to ligand_all.html...")
        render(paste0(out.dir, "/ligand_all.Rmd"), quiet=TRUE)
        message("receptor.Rmd is compiled to receptor.html...")
        render(paste0(out.dir, "/receptor.Rmd"), quiet=TRUE)
        message("receptor_all.Rmd is compiled to receptor_all.html...")
        render(paste0(out.dir, "/receptor_all.Rmd"), quiet=TRUE)
        message(paste0(length(selected),
            " pattern_X_Y.Rmd files are compiled to pattern_X_Y.html :"))
        out <- vapply(selected,
            function(x, e, SelectedLR){
                .eachRender_2(x, e, SelectedLR)}, "", e=e, SelectedLR=SelectedLR)

        # Output index.html
        RMDFILES <- vapply(selected, function(x){
            paste0(paste(c("pattern", index[x, seq_len(2)]),
                collapse="_"), ".Rmd")
        }, "")
        message("\nindex.Rmd is created...")
        outIdx <- file(paste0(out.dir, "/index.Rmd"), "w")
        writeLines(.MAINHEADER(author, title), outIdx, sep="\n")
        writeLines(.BODY1_2, outIdx, sep="\n")
        writeLines(.BODY2_2, outIdx, sep="\n")
        writeLines(.BODY3(numLPattern, ClusterL), outIdx, sep="\n")
        writeLines(.BODY4(numRPattern, ClusterR), outIdx, sep="\n")
        writeLines(.BODY6, outIdx, sep="\n")
        writeLines(.BODY7, outIdx, sep="\n")
        if(length(selected) != 0){
            writeLines(.BODY8_2(selected, RMDFILES, index, corevalue),
                outIdx, sep="\n")
        }
        writeLines(.BODY9, outIdx, sep="\n")
        writeLines(.BODY10, outIdx, sep="\n")
        close(outIdx)

        # Rendering
        message("index.Rmd is compiled to index.html...\n")
        render(paste0(out.dir, "/index.Rmd"), quiet=TRUE)
    }
}

.CCIhyperGraphPlot_2 <- function(outobj, twoDplot=NULL, vertex.size=18,
    xleft=1.75, ybottom=-0.5, xright=1.85, ytop=0.5, label="", emph=NULL, algorithm=""){
    # Number of Patterns
    numLPattern <- nrow(outobj$ligand)
    numRPattern <- nrow(outobj$receptor)

    #
    # Step.1 : Background Network
    #
    edgewd_L <- as.vector(vapply(seq_len(numLPattern), function(x){
        rep(x, numRPattern)
        }, rep(0L, numRPattern)))
    edgewd_R <- rep(seq_len(numRPattern), numLPattern)
    edgewd_Strength <- vapply(
        seq_len(numLPattern*numRPattern), function(x){
            targetL <- which(
                outobj$index[, "Mode1"] == edgewd_L[x])
            targetR <- which(
                outobj$index[, "Mode2"] == edgewd_R[x])
            sum(outobj$index[intersect(targetL, targetR), 3])
        }, 0.0)
    edgewd <- cbind(edgewd_L, edgewd_R, edgewd_Strength)
    colnames(edgewd) <- c("L", "R", "Strength")

    # Node name (Top<Ligand> and Bottom<Receptor>)
    nodesSetTop <- paste0("L", seq_len(numLPattern))
    nodesSetBottom <- paste0("R", seq_len(numRPattern))

    # Empty Graph
    g <- graph.empty()

    # Add nodes
    g <- add.vertices(g, nv=length(nodesSetTop),
        attr=list(name=nodesSetTop,
            type=rep(TRUE, length(nodesSetTop))))
    g <- add.vertices(g, nv=length(nodesSetBottom),
        attr=list(name=nodesSetBottom,
            type=rep(TRUE, length(nodesSetBottom))))

    # Add edges
    edgeListVec <- as.vector(t(as.matrix(
        data.frame(
            S1=paste0('L', edgewd[,1]),
            S2=paste0('R', edgewd[,2])
    ))))
    g <- add.edges(g, edgeListVec)

    # Edge weghts
    E(g)$weight <- edgewd[,3]

    # Edge color
    weight <- E(g)$weight
    E(g)$weight <- weight / max(weight) * 20
    mycolor <- smoothPalette(E(g)$weight,
        palfunc=colorRampPalette(.setColor("greens"), alpha=TRUE))
    if(!is.null(emph)){
        target <- intersect(
            which(get.edgelist(g)[, 1] == paste0("L", emph[1])),
            which(get.edgelist(g)[, 2] == paste0("R", emph[2])))
        mycolor[target] <- rgb(1,0,0,0.5)
    }

    # Layout
    x <- c(seq_along(nodesSetTop), seq_along(nodesSetBottom))
    y <- c(rep(1, length=length(nodesSetTop)),
        rep(0, length=length(nodesSetBottom)))
    mylayout <- cbind(x, y)

    # Network Plot
    par(oma=c(2,2,2,2))
    plot.igraph(g,
        layout=mylayout,
        vertex.size=18,
        vertex.label="",
        vertex.color="white",
        vertex.shape="square",
        edge.color=mycolor,
        vertex.frame.color="gray",
        edge.width=E(g)$weight)

    # Gradient
    gradient.rect(xleft, ybottom, xright, ytop,
        col=smoothPalette(sort(weight),
        palfunc=colorRampPalette(.setColor("greens"), alpha=TRUE)),
        gradient="y")
    text(2.2, ybottom+(ytop-ybottom)*0/4, round(quantile(weight)[1]))
    text(2.2, ybottom+(ytop-ybottom)*1/4, round(quantile(weight)[2]))
    text(2.2, ybottom+(ytop-ybottom)*2/4, round(quantile(weight)[3]))
    text(2.2, ybottom+(ytop-ybottom)*3/4, round(quantile(weight)[4]))
    text(2.2, ybottom+(ytop-ybottom)*4/4, round(quantile(weight)[5]))
    text(1.8, ybottom+(ytop-ybottom)*4.5/4, "CCI-Strength", cex=2)
    text(-1.5, -1, "Receptor Patterns", cex=2)
    text(-1.5, 1, "Ligand Patterns", cex=2)

    if(!is.null(twoDplot)){
        # Setting
        maLR <- max(numLPattern, numRPattern)
        if(1 <= maLR && maLR <= 16){
            omasize <- .omasize(numLPattern, numRPattern)
            oma4 <- .oma4(numLPattern, numRPattern)
            #
            # Step.2 : Ligand Plot
            #
            # Color
            col.ligand <- .setColor("reds")
            # Constant
            LOMA_1 = 48.85
            LOMA_2 = 42
            LOMA_3 = 4
            out <- vapply(seq_len(numLPattern), function(i){
                label.ligand <- unlist(vapply(names(label),
                    function(x){
                        outobj$ligand[paste0("Dim", i), x]
                    }, 0.0))
                label.ligand[] <- smoothPalette(label.ligand,
                    palfunc=colorRampPalette(col.ligand, alpha=TRUE))
                par(new=TRUE)
                par(oma = c(LOMA_1, LOMA_2+(i-1)*omasize,
                    LOMA_3, oma4-omasize*i))
                plot(twoDplot, col=label.ligand, pch=16, cex=0.5, bty="n",
                    xaxt="n", yaxt="n", xlab="", ylab="",
                    main=paste0("(", i, ",*,*)"))
                0L
                }, 0L)

            #
            # Step.3 : Receptor Plot
            #
            # Color
            col.receptor <- .setColor("blues")
            # Constant
            ROMA_1 = 4
            ROMA_2 = 42
            ROMA_3 = 48.85
            out <- vapply(seq_len(numRPattern), function(i){
                label.receptor <- unlist(vapply(names(label),
                    function(x){
                        outobj$receptor[paste0("Dim", i), x]
                    }, 0.0))
                label.receptor[] <- smoothPalette(label.receptor,
                    palfunc=colorRampPalette(col.receptor, alpha=TRUE))
                par(new=TRUE)
                par(oma = c(ROMA_1, ROMA_2+(i-1)*omasize,
                    ROMA_3, oma4-omasize*i))
                plot(twoDplot, col=label.receptor, pch=16, cex=0.5,
                    bty="n", xaxt="n", yaxt="n", xlab="", ylab="",
                    main=paste0("(*,", i, ",*)"))
                0L
                }, 0L)
        }else{
            warning(paste0("LR plot can be performed when \n",
                "the maximum number of Ligand/Receptor patterns are \n",
                "higher than 1 and smaller than 12 for now."))
        }
    }
}

.geneHyperGraphPlot_2 <- function(out.vecLR, GeneInfo, out.dir){
    # Setting
    convertGeneName <- function(geneid, GeneInfo){
        if(!is.null(GeneInfo$GeneName)){
            genename <- GeneInfo$GeneName[
                which(GeneInfo$GeneName$ENTREZID == geneid),
                "SYMBOL"][1]
            if(is.na(genename[1])){
                genename = geneid
            }
            genename            
        }else{
            geneid            
        }
    }

    # Node
    nodes <- lapply(seq_len(length(out.vecLR)), function(x){
        names(out.vecLR[[x]]$TARGET)})
    Lnodes <- lapply(nodes, function(x){
        vapply(x, function(xx){
            strsplit(xx, "_")[[1]][1]
        }, "")
    })
    Rnodes <-lapply(nodes, function(x){
        vapply(x, function(xx){
            strsplit(xx, "_")[[1]][2]
        }, "")
    })
    Lnodes <- lapply(Lnodes, function(x){
        vapply(x, function(xx){
            convertGeneName(xx, GeneInfo)
        }, "")
    })
    Rnodes <- lapply(Rnodes, function(x){
        vapply(x, function(xx){
            convertGeneName(xx, GeneInfo)
        }, "")
    })
    uniqueLnodes <- unique(unlist(Lnodes))
    uniqueRnodes <- unique(unlist(Rnodes))

    # Empty Graph
    g <- graph.empty(directed=FALSE)
    # Add nodes
    g <- add.vertices(g, nv=length(uniqueLnodes),
        attr=list(name=uniqueLnodes,
            type=rep(TRUE, length(uniqueLnodes)),
            color=rgb(1,0,0,0.5)))
    g <- add.vertices(g, nv=length(uniqueRnodes),
        attr=list(name=uniqueRnodes,
            type=rep(TRUE, length(uniqueRnodes)),
            color=rgb(0,0,1,0.5)))

    # Nodes Weight
    freqLnodes <- vapply(uniqueLnodes, function(x){
        length(which(unlist(Lnodes) == x))
        }, 0L)
    freqRnodes <- vapply(uniqueRnodes, function(x){
        length(which(unlist(Rnodes) == x))
        }, 0L)
    freq <- c(freqLnodes, freqRnodes)
    freq <- freq / max(freq) * 10

    # Add edges
    edgeListVec <- as.vector(t(as.matrix(
        data.frame(
            L=unlist(Lnodes),
            R=unlist(Rnodes)
    ))))
    g <- add.edges(g, edgeListVec)

    # Plot
    cols <- .setColor("many")
    edge.cols <- unlist(lapply(seq_len(length(out.vecLR)), function(x){
            rep(cols[x], length(out.vecLR[[x]]$TARGET))
        }))

    # Setting
    V(g)$size <- freq
    E(g)$color <- edge.cols
    E(g)$width <- 0.7
    l <- layout_with_dh(g)

    # All Pattern
    png(filename=paste0(out.dir, "/figures/GeneHypergraph.png"),
        width=2500, height=2500)
    plot.igraph(g, layout=l)
    legend("topleft",
        legend=c("ligand", "receptor",
            names(out.vecLR)),
        col=c(rgb(1,0,0,0.5), rgb(0,0,1,0.5),
            cols[seq_len(length(out.vecLR))]),
        pch=16, cex=2.2)
    dev.off()

    # Each Pattern
    out <- vapply(seq_len(length(out.vecLR)), function(x){
        tmp_edgecolor <- edge.cols
        tmp_edgecolor[which(tmp_edgecolor != cols[x])] <- rgb(0,0,0,0.1)
        tmp_nodecolor <- V(g)$color
        grayout <- setdiff(
            setdiff(
                names(V(g)),
                Lnodes[[x]]
                ), Rnodes[[x]]
            )
        target <- unlist(lapply(grayout, function(xx){
            which(names(V(g)) == xx)
        }))
        tmp_nodecolor[target] <- rgb(0,0,0,0.1)

        # Plot
        png(filename=paste0(
            out.dir, "/figures/GeneHypergraph",
            gsub("pattern", "", names(out.vecLR)[x]),
            ".png"),
            width=2500, height=2500)
        plot.igraph(g,
            vertex.color=tmp_nodecolor,
            edge.color=tmp_edgecolor, layout=l)
        legend("topleft",
            legend=c("ligand", "receptor",
                names(out.vecLR)[x]),
            col=c(rgb(1,0,0,0.5), rgb(0,0,1,0.5),
                cols[x]),
            pch=16, cex=2.2)
        dev.off()
    }, 0L)
    return(g)
}

.tagCloud_2 <- function(out.vecLR, out.dir){
    sapply(seq_len(length(out.vecLR)), function(x){
        # Pvalue
        Pvalues <- list(
            GO_BP=out.vecLR[[x]]$Enrich$GO_BP$Pvalue,
            GO_MF=out.vecLR[[x]]$Enrich$GO_MF$Pvalue,
            GO_CC=out.vecLR[[x]]$Enrich$GO_CC$Pvalue,
            MeSH_A=out.vecLR[[x]]$Enrich$MeSH_A$Pvalue,
            MeSH_B=out.vecLR[[x]]$Enrich$MeSH_B$Pvalue,
            MeSH_C=out.vecLR[[x]]$Enrich$MeSH_C$Pvalue,
            MeSH_D=out.vecLR[[x]]$Enrich$MeSH_D$Pvalue,
            MeSH_E=out.vecLR[[x]]$Enrich$MeSH_E$Pvalue,
            MeSH_F=out.vecLR[[x]]$Enrich$MeSH_F$Pvalue,
            MeSH_G=out.vecLR[[x]]$Enrich$MeSH_G$Pvalue,
            MeSH_H=out.vecLR[[x]]$Enrich$MeSH_H$Pvalue,
            MeSH_I=out.vecLR[[x]]$Enrich$MeSH_I$Pvalue,
            MeSH_J=out.vecLR[[x]]$Enrich$MeSH_J$Pvalue,
            MeSH_K=out.vecLR[[x]]$Enrich$MeSH_K$Pvalue,
            MeSH_L=out.vecLR[[x]]$Enrich$MeSH_L$Pvalue,
            MeSH_M=out.vecLR[[x]]$Enrich$MeSH_M$Pvalue,
            MeSH_N=out.vecLR[[x]]$Enrich$MeSH_N$Pvalue,
            MeSH_V=out.vecLR[[x]]$Enrich$MeSH_V$Pvalue,
            MeSH_Z=out.vecLR[[x]]$Enrich$MeSH_Z$Pvalue,
            Reactome=out.vecLR[[x]]$Enrich$Reactome$Pvalue,
            DO=out.vecLR[[x]]$Enrich$DO$Pvalue,
            NCG=out.vecLR[[x]]$Enrich$NCG$Pvalue,
            DGN=out.vecLR[[x]]$Enrich$DGN$Pvalue
            )
        # Term
        Terms <- list(
            GO_BP=out.vecLR[[x]]$Enrich$GO_BP$Term,
            GO_MF=out.vecLR[[x]]$Enrich$GO_MF$Term,
            GO_CC=out.vecLR[[x]]$Enrich$GO_CC$Term,
            MeSH_A=out.vecLR[[x]]$Enrich$MeSH_A$Term,
            MeSH_B=out.vecLR[[x]]$Enrich$MeSH_B$Term,
            MeSH_C=out.vecLR[[x]]$Enrich$MeSH_C$Term,
            MeSH_D=out.vecLR[[x]]$Enrich$MeSH_D$Term,
            MeSH_E=out.vecLR[[x]]$Enrich$MeSH_E$Term,
            MeSH_F=out.vecLR[[x]]$Enrich$MeSH_F$Term,
            MeSH_G=out.vecLR[[x]]$Enrich$MeSH_G$Term,
            MeSH_H=out.vecLR[[x]]$Enrich$MeSH_H$Term,
            MeSH_I=out.vecLR[[x]]$Enrich$MeSH_I$Term,
            MeSH_J=out.vecLR[[x]]$Enrich$MeSH_J$Term,
            MeSH_K=out.vecLR[[x]]$Enrich$MeSH_K$Term,
            MeSH_L=out.vecLR[[x]]$Enrich$MeSH_L$Term,
            MeSH_M=out.vecLR[[x]]$Enrich$MeSH_M$Term,
            MeSH_N=out.vecLR[[x]]$Enrich$MeSH_N$Term,
            MeSH_V=out.vecLR[[x]]$Enrich$MeSH_V$Term,
            MeSH_Z=out.vecLR[[x]]$Enrich$MeSH_Z$Term,
            Reactome=out.vecLR[[x]]$Enrich$Reactome$Term,
            DO=out.vecLR[[x]]$Enrich$DO$Term,
            NCG=out.vecLR[[x]]$Enrich$NCG$Term,
            DGN=out.vecLR[[x]]$Enrich$DGN$Term
            )
        lapply(names(Pvalues), function(xx){
            # Pvalue
            pval <- eval(parse(text=paste0("Pvalues$", xx)))
            # Term
            t <- as.character(eval(parse(text=paste0("Terms$", xx))))
            # Plot
            if(!is.null(pval)){                
                png(filename=paste0(out.dir, "/figures/Tagcloud/", xx,
                    "_", names(out.vecLR)[x],
                    ".png"), width=1000, height=1000)
                if(length(pval) == 1){
                    t <- sapply(t, function(x){.shrink2(x, thr=7)})
                    .SinglePlot(t)
                }else{
                    negLogPval <- -log10(pval+1E-10)
                    target <- seq_len(min(100, length(pval)))
                    if(length(pval) <= 5){
                        t <- sapply(t, function(x){.shrink2(x, thr=10)})
                    }else{
                        t <- sapply(t, function(x){.shrink2(x, thr=25)})
                    }
                    tagcloud(t[target], weights = negLogPval[target],
                    col = smoothPalette(negLogPval[target], palfunc = .palf),
                    order = "size", algorithm = "fill",
                    scale.multiplier=0.8)
                }
                dev.off()
            }
        })
    })
}

.eachRender_2 <- function(x, e, SelectedLR){
    index <- e$index
    out.dir <- e$out.dir
    out.vecLR <- e$out.vecLR
    .XYZ_HEADER1_2 <- e$.XYZ_HEADER1_2
    .XYZ_HEADER2_2 <- e$.XYZ_HEADER2_2
    .XYZ_HEADER3_2 <- e$.XYZ_HEADER3_2
    .XYZ_ENRICH_2 <- e$.XYZ_ENRICH_2

    indexLR <- paste0(c("pattern", SelectedLR[x, ]), collapse="_")
    TARGET <- out.vecLR[[indexLR]]$TARGET
    LINKS <- out.vecLR[[indexLR]]$LINKS

    # Bottom part of Rmarkdown
    XYZ_BOTTOM <- paste(
        c(.XYZ_HEADER2_2(index, x, length(TARGET)),
        LINKS,
        .XYZ_HEADER3_2(index, x),
        .XYZ_ENRICH_2(out.vecLR, indexLR)),
        collapse="")

    # Each (x,y,z)-rmdfile
    RMDFILE <- paste0(c("pattern", index[x, seq_len(2)]), collapse="_")
    RMDFILE <- paste0(RMDFILE, ".Rmd")
    cat(paste0("\n", RMDFILE, " is created...\n"))
    sink(file = paste0(out.dir, "/", RMDFILE))
    cat(paste(
        c(.XYZ_HEADER1_2(index, x), XYZ_BOTTOM),
        collapse=""))
    sink()

    # Rendering
    message(paste0(RMDFILE, " is compiled to ",
        gsub(".Rmd", ".html", RMDFILE)))
    render(paste0(out.dir, "/", RMDFILE), quiet=TRUE)
}

.CPMED <- function(input){
    libsize <- colSums(input)
    median(libsize) * t(t(input) / libsize)
}

.CPM <- function(input){
    libsize <- colSums(input)
    1e6 * t(t(input) / libsize)
}

.CPT <- function(input){
    libsize <- colSums(input)
    1e4 * t(t(input) / libsize)
}

.FTT <- function(input){
    sqrt(input) + sqrt(input + 1)
}

.importAssays <- function(sce, assayNames){
    if(assayNames %in% names(assays(sce))){
        eval(parse(text=paste0("assays(sce)$", assayNames)))
    }else{
        stop("Please specify the valid assayNames (cf. names(assays(sce)))")
    }
}

'%ni%' <- Negate('%in%')

.cellType <- function(input, celltypes, mergeas="mean"){
    all.celltypes = unique(names(celltypes))
    if(mergeas == "mean"){
        ct <- vapply(all.celltypes, function(x){
            rowMeans(input[, which(names(celltypes) == x), drop=FALSE])
        }, rep(0.0, nrow(input)))
    }else if(mergeas == "sum"){
        ct <- vapply(all.celltypes, function(x){
            rowSums(input[, which(names(celltypes) == x), drop=FALSE])
        }, rep(0.0, nrow(input)))
    }else{
        stop("Please specify mergeas as mean or sum")
    }
    ct[which(is.nan(ct))] <- 0
    return(ct)
}

.divLR <- function(input, LR){
    input_L <- as.matrix(input[as.character(
        intersect(rownames(input), LR$GENEID_L)), ])
    input_R <- as.matrix(input[as.character(
        intersect(rownames(input), LR$GENEID_R)), ])
    return(list(input_L, input_R))
}

.cellCellReconst <- function(A1, A2, A3, index,
    type="matrix", names, collapse=FALSE){
    if(type == "matrix"){
        out <- lapply(seq_len(ncol(A1)), function(x){
            outer(A1[, x], A2[, x])})
    }else if(type == "tensor"){
        out <- list()
        counter <- 1
        for(i in seq_len(nrow(index))){
            element <- as.tensor(array(1, dim=c(1,1,1)))
            element@data[] <- index[i, "Value"]
            A <- as.matrix(A1[index[i, "Mode1"], ])
            B <- as.matrix(A2[index[i, "Mode2"], ])
            C <- as.matrix(A3[index[i, "Mode3"], ])
            rec_out <- recTensor(element, list(t(A), t(B), t(C)))
            if(collapse){
                out[[counter]] <- modeSum(rec_out, m=3, drop=TRUE)@data
                dimnames(out[[counter]]) <- list(names[[1]], names[[2]])
            }else{
                out[[counter]] <- rec_out
                dimnames(out[[counter]]@data) <- names
            }
            counter <- counter + 1
        }
    }
    out
}

.core2table <- function(core){
    d <- dim(core)
    out <- matrix(NA, nrow=prod(d), ncol=5)
    out[seq_len(nrow(out)), seq_len(3)] <- as.matrix(
        expand.grid(seq_len(d[1]), seq_len(d[2]), seq_len(d[3])))
    colnames(out) <- c("Mode1", "Mode2", "Mode3", "Value", "Rank")
    veccore <- c()
    counter <- 1
    vapply(seq_len(nrow(out)), function(x){
        i <- out[x, 1]
        j <- out[x, 2]
        k <- out[x, 3]
        out[x, 4] <<- core[i,j,k]
    }, 0.0)
    out[, 5] <- nrow(out) - base::rank(out[, 4]) + 1
    out[order(out[,4], decreasing=TRUE), ]
}

.core2table_2 <- function(core){
    d <- dim(core)
    out <- matrix(NA, nrow=d[1]*d[2], ncol=4)
    out[seq_len(nrow(out)), seq_len(2)] <- as.matrix(
        expand.grid(seq_len(d[1]), seq_len(d[2])))
    colnames(out) <- c("Mode1", "Mode2", "Value", "Rank")
    veccore <- c()
    counter <- 1
    vapply(seq_len(nrow(out)), function(x){
        i <- out[x, 1]
        j <- out[x, 2]
        out[x, 3] <<- core[i,j]
    }, 0.0)
    out[, 4] <- nrow(out) - base::rank(out[, 3]) + 1
    out[order(out[,3], decreasing=TRUE), ]
}

.fastPossibleCombination <- function(input, LR, celltypes, num.sampling, outerfunc){
    lr <- .divLR(input, LR)
    L <- as.matrix(lr[[1]])
    R <- as.matrix(lr[[2]])
    tnsr <- array(0, dim=c(length(unique(names(celltypes))),
        length(unique(names(celltypes))), 0))
    Pair.name <- c()
    comb <- as.matrix(expand.grid(unique(names(celltypes)),
        unique(names(celltypes))))
    for(i in seq_len(nrow(LR))){
        l <- which(LR$GENEID_L[i] == rownames(L))
        r <- which(LR$GENEID_R[i] == rownames(R))
        if(length(l)==1 && length(r) == 1){
            pre_tnsr <- array(0, dim=c(length(unique(names(celltypes))),
                length(unique(names(celltypes))), 1))
            for(j in seq_len(num.sampling)){
                target <- vapply(unique(names(celltypes)), function(x){
                    sample(which(names(celltypes) == x), 1)}, 0L)
                pre_tnsr[,,1] <- pre_tnsr[,,1] + outer(as.vector(L[l, target]),
                as.vector(R[r, target]), outerfunc)
            }
            tnsr <- abind(tnsr, pre_tnsr, along=3)
            Pair.name <- c(Pair.name, paste(LR$GENEID_L[i],
                LR$GENEID_R[i], sep="_"))
        }
    }
    dimnames(tnsr) <- list(unique(names(celltypes)),
        unique(names(celltypes)), Pair.name)
    list(tnsr=tnsr, pairname=Pair.name)
}

.slowPossibleCombination <- function(input, LR, celltypes, outerfunc){
    lr <- .divLR(input, LR)
    L <- as.matrix(lr[[1]])
    R <- as.matrix(lr[[2]])
    tnsr <- array(0, dim=c(length(unique(names(celltypes))),
        length(unique(names(celltypes))), 0))
    Pair.name <- c()
    comb <- as.matrix(expand.grid(unique(names(celltypes)),
        unique(names(celltypes))))
    for(i in seq_len(nrow(LR))){
        l <- which(LR$GENEID_L[i] == rownames(L))
        r <- which(LR$GENEID_R[i] == rownames(R))
        if(length(l)==1 && length(r) == 1){
            pre_tnsr <- outer(as.vector(L[l,]), as.vector(R[r,]))
            pre_tnsr <- apply(comb, 1, function(x){
                sum(pre_tnsr[which(names(celltypes) == x[1]),
                which(names(celltypes) == x[2])])})
            dim(pre_tnsr) <- c(length(unique(celltypes)),
            length(unique(celltypes)))
            tnsr <- abind(tnsr, pre_tnsr, along=3)
            Pair.name <- c(Pair.name, paste(LR$GENEID_L[i],
                LR$GENEID_R[i], sep="_"))
        }
    }
    dimnames(tnsr) <- list(unique(names(celltypes)),
        unique(names(celltypes)), Pair.name)
    list(tnsr=tnsr, pairname=Pair.name)
}

.cellCellDecomp.Third_3 <- function(input, LR, celltypes, ranks, rank,
    centering, mergeas, outerfunc, comb, num.sampling, num.perm, decomp,
    thr1, thr2, thr3, L1_A, L2_A, verbose){
    if(centering){
        fout <- .celltypemergedtensor(input, LR, celltypes, mergeas, outerfunc)
    }else{
        if(comb == "random"){
            fout <- .fastPossibleCombination(input, LR, celltypes,
                num.sampling, outerfunc)
        }else if(comb == "all"){
            fout <- .slowPossibleCombination(input, LR, celltypes, outerfunc)
        }
    }
    tnsr <- as.tensor(fout$tnsr)
    Pair.name <- fout$pairname
    if(decomp){
        message(paste0(paste(dim(tnsr), collapse=" * "), " Tensor is created"))
        out <- try(MultiCX(Y=tnsr, modes=1:2, thr=thr3, c.method="random"))
        if(is(out)[1] == "try-error"){
            out <- MultiCX(Y=tnsr, modes=1:2, thr=thr3, c.method="random")
        }
        A1 <- t(out$C[[1]]$C)
        A2 <- t(out$C[[2]]$C)
        rownames(A1) <- paste0("Dim", seq_len(nrow(A1)))
        colnames(A1) <- unique(names(celltypes))
        rownames(A2) <- paste0("Dim", seq_len(nrow(A2)))
        colnames(A2) <- unique(names(celltypes))
        dimnames(out$U@data) <- list(
            Lpattern=paste0("Dim", seq_len(nrow(A1))),
            Rpattern=paste0("Dim", seq_len(nrow(A2))),
            LR=Pair.name
        )
        if (nrow(A1) * nrow(A2) == 1){
            core <- t(sum(out$U@data))
        }else{
            core <- modeSum(out$U, m=3, drop=TRUE)@data
            if(nrow(A1) == 1 || nrow(A2) == 1){
                dim(core) <- c(nrow(A1), nrow(A2))
            }
        }
        index <- .core2table_2(core)
        if(is.vector(index)){
            index <- t(index)
        }
        return(list(index=index, ligand=A1,
            receptor=A2, lrpair=out$U,
            recerror=out$RecError, relchange=out$RelChange))
    }else{
        tnsr_cc <- modeSum(tnsr, m=3, drop=TRUE)@data
        dimnames(tnsr_cc) <- list(unique(names(celltypes)),
            unique(names(celltypes)))
        return(list(cellcellpattern=tnsr_cc, cellcelllrpairpattern=tnsr))
    }
}

.cellCellDecomp.Third_2 <- function(input, LR, celltypes, ranks, rank,
    centering, mergeas, outerfunc, comb, num.sampling, num.perm, decomp,
    thr1, thr2, thr3, L1_A, L2_A, verbose){
    # ranks-check
    max.rank <- length(unique(celltypes))
    if(ranks[1] > max.rank || ranks[2] > max.rank){
        stop("ranks must be defined less than number of celltypes")
    }
    if(centering){
        fout <- .celltypemergedtensor(input, LR, celltypes, mergeas, outerfunc)
    }else{
        if(comb == "random"){
            fout <- .fastPossibleCombination(input, LR, celltypes,
                num.sampling, outerfunc)
        }else if(comb == "all"){
            fout <- .slowPossibleCombination(input, LR, celltypes, outerfunc)
        }
    }
    # rank check
    check1 <- dim(fout$tnsr)[2] * dim(fout$tnsr)[3] < ranks[1]
    check2 <- dim(fout$tnsr)[3] * dim(fout$tnsr)[1] < ranks[2]
    if(check1){
        stop("Please specify the ranks[1] as an smaller value")
    }
    if(check2){
        stop("Please specify the ranks[2] as an smaller value")
    }
    ranks <- ranks[1:2]
    if(dim(fout$tnsr)[1] >= 100 || dim(fout$tnsr)[2] >= 100){
        stop(paste0("Extreamly Large tensor will be generated!\n",
            "This problem will be solved by the next version of scTensor."))
    }
    tnsr <- as.tensor(fout$tnsr)
    Pair.name <- fout$pairname
    if(decomp){
        message(paste0(paste(dim(tnsr), collapse=" * "), " Tensor is created"))
        out <- try(NTD(X=tnsr, L1_A=L1_A, L2_A=L2_A,
            rank=ranks, modes=1:2, verbose=verbose,
            algorithm="Frobenius", num.iter=30))
        if(is(out)[1] == "try-error"){
            out <- NTD(X=tnsr, L1_A=L1_A, L2_A=L2_A,
                rank=ranks, modes=1:2, verbose=verbose,
                algorithm="Frobenius", num.iter=30)
        }
        A1 <- out$A[[1]]
        A2 <- out$A[[2]]
        rownames(A1) <- paste0("Dim", seq_len(ranks[1]))
        colnames(A1) <- unique(names(celltypes))
        rownames(A2) <- paste0("Dim", seq_len(ranks[2]))
        colnames(A2) <- unique(names(celltypes))
        dimnames(out$S@data) <- list(
            Lpattern=paste0("Dim", seq_len(ranks[1])),
            Rpattern=paste0("Dim", seq_len(ranks[2])),
            LR=Pair.name
        )
        if (nrow(A1) * nrow(A2) == 1){
            core <- t(sum(out$S@data))
        }else{
            core <- modeSum(out$S, m=3, drop=TRUE)@data
            if(ranks[1] == 1 || ranks[2] == 1){
                dim(core) <- c(ranks[1], ranks[2])
            }
        }
        index <- .core2table_2(core)
        if(is.vector(index)){
            index <- t(index)
        }
        return(list(index=index, ligand=A1,
            receptor=A2, lrpair=out$S,
            recerror=out$RecError, relchange=out$RelChange))
    }else{
        tnsr_cc <- modeSum(tnsr, m=3, drop=TRUE)@data
        dimnames(tnsr_cc) <- list(unique(names(celltypes)),
            unique(names(celltypes)))
        return(list(cellcellpattern=tnsr_cc, cellcelllrpairpattern=tnsr))
    }
}

.cellCellDecomp.Third <- function(input, LR, celltypes, ranks, rank, centering,
        mergeas, outerfunc, comb, num.sampling, num.perm, decomp,
        thr1, thr2, thr3, L1_A, L2_A, verbose){
    # ranks-check
    max.rank <- length(unique(celltypes))
    if(ranks[1] > max.rank || ranks[2] > max.rank){
        stop("ranks must be defined less than number of celltypes")
    }
    if(centering){
        fout <- .celltypemergedtensor(input, LR, celltypes, mergeas, outerfunc)
    }else{
        if(comb == "random"){
            fout <- .fastPossibleCombination(input, LR, celltypes,
                num.sampling, outerfunc)
        }else if(comb == "all"){
            fout <- .slowPossibleCombination(input, LR, celltypes, outerfunc)
        }
    }
    # rank check
    check1 <- dim(fout$tnsr)[2] * dim(fout$tnsr)[3] < ranks[1]
    check2 <- dim(fout$tnsr)[3] * dim(fout$tnsr)[1] < ranks[2]
    check3 <- dim(fout$tnsr)[1] * dim(fout$tnsr)[2] < ranks[3]
    if(check1){
        stop("Please specify the ranks[1] as an smaller value")
    }
    if(check2){
        stop("Please specify the ranks[2] as an smaller value")
    }
    if(check3){
        stop("Please specify the ranks[3] as an smaller value")
    }
    if(dim(fout$tnsr)[1] >= 100 || dim(fout$tnsr)[2] >= 100){
        stop(paste0("Extreamly Large tensor will be generated!\n",
            "This problem will be solved by the next version of scTensor."))
    }
    tnsr <- as.tensor(fout$tnsr)
    Pair.name <- fout$pairname
    if(decomp){
        message(paste0(paste(dim(tnsr), collapse=" * "), " Tensor is created"))
        out <- try(NTD(X=tnsr, L1_A=L1_A, L2_A=L2_A,
            rank=ranks, num.iter=30,
            verbose=verbose, algorithm="Frobenius"))
        if(is(out)[1] == "try-error"){
            out <- try(NTD(X=tnsr, L1_A=L1_A, L2_A=L2_A,
                rank=ranks, num.iter=30,
                verbose=verbose, algorithm="Frobenius"))
        }
        A1 <- out$A[[1]]
        A2 <- out$A[[2]]
        A3 <- out$A[[3]]
        rownames(A1) <- paste0("Dim", seq_len(ranks[1]))
        colnames(A1) <- unique(names(celltypes))
        rownames(A2) <- paste0("Dim", seq_len(ranks[2]))
        colnames(A2) <- unique(names(celltypes))
        rownames(A3) <- paste0("Dim", seq_len(ranks[3]))
        colnames(A3) <- Pair.name
        core <- out$S@data
        index <- .core2table(core)
        if(is.vector(index)){
            index <- t(index)
        }
        lrpairout <- .cellCellReconst(A1, A2, A3, index, type="tensor",
            names=list(colnames(A1), colnames(A2), colnames(A3)),
            collapse=TRUE)
        return(list(index=index, ligand=A1,
            receptor=A2, lrpair=A3, cellcellpattern=lrpairout,
            recerror=out$RecError, relchange=out$RelChange))
    }else{
        tnsr_cc <- modeSum(tnsr, m=3, drop=TRUE)@data
        dimnames(tnsr_cc) <- list(unique(names(celltypes)),
            unique(names(celltypes)))
        return(list(cellcellpattern=tnsr_cc, cellcelllrpairpattern=tnsr))
    }
}

.cellCellDecomp.Second <- function(input, LR, celltypes, ranks, rank,
    centering, mergeas, outerfunc, comb, num.sampling, num.perm,
    decomp, thr1, thr2, thr3, L1_A, L2_A, verbose){
    # ranks-check
    max.rank <- length(unique(celltypes))
    if(rank > max.rank){
        stop("ranks must be defined less than number of celltypes")
    }
    if(centering){
        fout <- .celltypemergedtensor(input, LR, celltypes, mergeas, outerfunc)
    }else{
        if(comb == "random"){
            fout <- .fastPossibleCombination(input, LR, celltypes,
                num.sampling, outerfunc)
        }else if(comb == "all"){
            fout <- .slowPossibleCombination(input, LR, celltypes, outerfunc)
        }
    }
    tnsr <- as.tensor(fout$tnsr)
    Pair.name <- fout$pairname
    mat.tnsr <- modeSum(tnsr, m=3, drop=TRUE)@data
    dimnames(mat.tnsr) <- list(unique(names(celltypes)),
        unique(names(celltypes)))
    if(decomp){
        out <- try(NMF(mat.tnsr, J=rank, num.iter=30, algorithm="Frobenius"))
        colnames(out$U) <- paste0("Dim", seq_len(rank))
        colnames(out$V) <- paste0("Dim", seq_len(rank))
        sizeU <- apply(out$U, 2, function(x){sqrt(sum(x^2))})
        sizeV <- apply(out$V, 2, function(x){sqrt(sum(x^2))})
        value <- sizeU * sizeV
        index <- rank - base::rank(value) + 1
        return(list(ligand=out$U[,index], receptor=out$V[,index],
        cellcellpattern=.cellCellReconst(out$U[,index], out$V[,index])))
    }else{
        return(list(cellcellpattern=mat.tnsr))
    }
}

.cellCellDecomp.Pearson <- function(input, LR, celltypes, ranks, rank,
    centering, mergeas, outerfunc, comb, num.sampling, num.perm,
    decomp, thr1, thr2, thr3, L1_A, L2_A, verbose){
    out <- .cellType(input, celltypes)
    out <- cor(out, method="pearson")
    out <- as.matrix(out)
    return(list(cellcellpattern=out))
}

.cellCellDecomp.Spearman <- function(input, LR, celltypes, ranks, rank,
    centering, mergeas, outerfunc, comb, num.sampling, num.perm,
    decomp, thr1, thr2, thr3, L1_A, L2_A, verbose){
    out <- .cellType(input, celltypes)
    out <- cor(out, method="spearman")
    out <- as.matrix(out)
    return(list(cellcellpattern=out))
}

.cellCellDecomp.Distance <- function(input, LR, celltypes, ranks, rank,
    centering, mergeas, outerfunc, comb, num.sampling, num.perm,
    decomp, thr1, thr2, thr3, L1_A, L2_A, verbose){
    out <- .cellType(input, celltypes)
    out <- 1 / dist(t(out))
    out <- as.matrix(out)
    diag(out) <- 1
    return(list(cellcellpattern=out))
}

.cellCellDecomp.Pearson.LR <- function(input, LR, celltypes, ranks, rank,
    centering, mergeas, outerfunc, comb, num.sampling, num.perm,
    decomp, thr1, thr2, thr3, L1_A, L2_A, verbose){
    ct <- .cellType(input, celltypes)
    ct <- .divLR(ct, LR)
    L <- ct[[1]]
    R <- ct[[2]]
    Adj <- matrix(0, nrow=nrow(L), ncol=nrow(R))
    rownames(Adj) <- rownames(L)
    colnames(Adj) <- rownames(R)
    for(i in seq_len(nrow(L))){
        l <- as.character(rownames(L)[i])
        r <- as.character(LR[which(LR$GENEID_L == l), "GENEID_R"])
        Adj[l, intersect(colnames(Adj), r)] <- 1
    }
    out <- t(L) %*% Adj %*% R
    out <- cor(out, method="pearson")
    out <- as.matrix(out)
    return(list(cellcellpattern=out))
}

.cellCellDecomp.Spearman.LR <- function(input, LR, celltypes, ranks, rank,
    centering, mergeas, outerfunc, comb, num.sampling, num.perm,
    decomp, thr1, thr2, thr3, L1_A, L2_A, verbose){
    ct <- .cellType(input, celltypes)
    ct <- .divLR(ct, LR)
    L <- ct[[1]]
    R <- ct[[2]]
    Adj <- matrix(0, nrow=nrow(L), ncol=nrow(R))
    rownames(Adj) <- rownames(L)
    colnames(Adj) <- rownames(R)
    for(i in seq_len(nrow(L))){
        l <- as.character(rownames(L)[i])
        r <- as.character(LR[which(LR$GENEID_L == l), "GENEID_R"])
        Adj[l, intersect(colnames(Adj), r)] <- 1
    }
    out <- t(L) %*% Adj %*% R
    out <- cor(out, method="spearman")
    out <- as.matrix(out)
    return(list(cellcellpattern=out))
}

.cellCellDecomp.Distance.LR <- function(input, LR, celltypes, ranks, rank,
    centering, mergeas, outerfunc, comb, num.sampling, num.perm,
    decomp, thr1, thr2, thr3, L1_A, L2_A, verbose){
    ct <- .cellType(input, celltypes)
    ct <- .divLR(ct, LR)
    L <- ct[[1]]
    R <- ct[[2]]
    Adj <- matrix(0, nrow=nrow(L), ncol=nrow(R))
    rownames(Adj) <- rownames(L)
    colnames(Adj) <- rownames(R)
    for(i in seq_len(nrow(L))){
        l <- as.character(rownames(L)[i])
        r <- as.character(LR[which(LR$GENEID_L == l), "GENEID_R"])
        Adj[l, intersect(colnames(Adj), r)] <- 1
    }
    out <- t(L) %*% Adj %*% R
    out <- 1 / dist(t(out))
    out <- as.matrix(out)
    diag(out) <- 1
    return(list(cellcellpattern=out))
}

.cellCellDecomp.PossibleCombination <- function(input, LR, celltypes, ranks,
    rank, centering, mergeas, outerfunc, comb, num.sampling, num.perm,
    decomp, thr1, thr2, thr3, L1_A, L2_A, verbose){
    if(thr1 <= 0 || thr2 <= 0){
        warning("None of cell-cell interaction will be detected.")
    }
    lr <- .divLR(input, LR)
    L <- as.matrix(lr[[1]])
    R <- as.matrix(lr[[2]])

    L[which(L < thr1)] <- 0
    R[which(R < thr1)] <- 0
    L[which(L >= thr1)] <- 1
    R[which(R >= thr1)] <- 1

    tnsr_bin <- array(0, dim=c(length(unique(names(celltypes))),
        length(unique(names(celltypes))), 0))
    Pair.name <- c()
    comb <- as.matrix(expand.grid(unique(names(celltypes)),
        unique(names(celltypes))))
    for(i in seq_len(nrow(LR))){
        l <- which(LR$GENEID_L[i] == rownames(L))
        r <- which(LR$GENEID_R[i] == rownames(R))
        if(length(l)==1 && length(r) == 1){
            pre_tnsr_bin <- outer(as.vector(L[l,]), as.vector(R[r,]))
            pre_tnsr_bin <- apply(comb, 1, function(x){
                sum(pre_tnsr_bin[which(names(celltypes) == x[1]),
                    which(names(celltypes) == x[2])])
                })
            dim(pre_tnsr_bin) <- c(length(unique(celltypes)),
                length(unique(celltypes)))
            pre_tnsr_bin[which(pre_tnsr_bin < thr2)] <- 0
            pre_tnsr_bin[which(pre_tnsr_bin >= thr2)] <- 1
            tnsr_bin <- abind(tnsr_bin, pre_tnsr_bin, along=3)
            Pair.name <- c(Pair.name,
                paste(LR$GENEID_L[i], LR$GENEID_R[i], sep="_"))
        }
    }
    dimnames(tnsr_bin) <- list(unique(names(celltypes)),
        unique(names(celltypes)), Pair.name)
    tnsr_cc <- modeSum(as.tensor(tnsr_bin), m=3, drop=TRUE)@data
    dimnames(tnsr_cc) <- list(unique(names(celltypes)),
        unique(names(celltypes)))
    return(list(cellcellpattern=tnsr_cc, cellcelllrpairpattern=tnsr_bin))
}


.celltypemergedtensor <- function(input, LR, celltypes, mergeas, outerfunc){
    ct <- .cellType(input, celltypes, mergeas)
    ct <- .divLR(ct, LR)
    L <- ct[[1]]
    R <- ct[[2]]
    tnsr <- array(0, dim=c(ncol(L), ncol(R), 0))
    Pair.name <- c()
    for(i in seq_len(nrow(LR))){
        l <- which(LR$GENEID_L[i] == rownames(L))
        r <- which(LR$GENEID_R[i] == rownames(R))
        if(length(l)==1 && length(r) == 1){
            tnsr <- abind(tnsr,
                outer(as.vector(L[l,]), as.vector(R[r,]), outerfunc), along=3)
            Pair.name <- c(Pair.name,
                paste(LR$GENEID_L[i], LR$GENEID_R[i], sep="_"))
        }
    }
    list(tnsr=tnsr, pairname=Pair.name)
}

.celltypemergedtensor_2 <- function(input, LR, celltypes, mergeas, outerfunc){
    ct <- .cellType(input, celltypes, mergeas)
    ct <- .divLR(ct, LR)
    L <- ct[[1]]
    R <- ct[[2]]
    outer(as.vector(L), as.vector(R), outerfunc)
}

.labelpermutation <- function(observed, input, l, r, num.perm,
    celltypes, mergeas, outerfunc){
    perm <- array(0, dim=c(dim(observed), 0))
    for(j in seq_len(num.perm)){
        tmp <- .celltypemergedtensor_2(input,
            data.frame(GENEID_L=l, GENEID_R=r, stringsAsFactors=FALSE),
            sample(celltypes), mergeas, outerfunc)
        tmp <- observed - tmp
        tmp[which(tmp >= 0)] <- 0
        tmp[which(tmp < 0)] <- 1
        perm <- abind(perm, tmp)
    }
    modeSum(as.tensor(perm), m=3, drop=TRUE)@data / num.perm
}

.cellCellDecomp.LabelPerm.LR <- function(input, LR, celltypes, ranks, rank,
    centering, mergeas, outerfunc, comb, num.sampling, num.perm,
    decomp, thr1, thr2, thr3, L1_A, L2_A, verbose){
    fout <- .celltypemergedtensor(input, LR, celltypes,
        mergeas, outerfunc)
    tnsr <- as.tensor(fout$tnsr)
    Pair.name <- fout$pairname
    pval <- array(0, dim=c(dim(tnsr)[1:2], 0))
    for(i in seq_along(Pair.name)){
        cat(paste0(i, " / ", length(Pair.name), "\r"))
        l <- strsplit(Pair.name[i], "_")[[1]][1]
        r <- strsplit(Pair.name[i], "_")[[1]][2]
        tmp <- .labelpermutation(tnsr[,,i]@data, input,
            l, r, num.perm, celltypes, mergeas, outerfunc)
        pval <- abind(pval, tmp)
    }
    pval[which(is.nan(pval))] <- 1
    dimnames(pval) <- list(unique(names(celltypes)),
        unique(names(celltypes)), Pair.name)
    tnsr_cc <- modeSum(tnsr, m=3, drop=TRUE)@data
    dimnames(tnsr_cc) <- list(unique(names(celltypes)),
        unique(names(celltypes)))
    return(list(cellcellpattern=tnsr_cc, cellcelllrpairpattern=tnsr,
        pval=pval))
}

# SingleCellSignalR
.celltypemergedtensor.ca <- function(input, LR, celltypes, mergeas, outerfunc){
    ct <- .cellType(input, celltypes, mergeas)
    ct <- .divLR(ct, LR)
    L <- ct[[1]]
    R <- ct[[2]]
    tnsr <- array(0, dim=c(ncol(L), ncol(R), 0))
    Pair.name <- c()
    for(i in seq_len(nrow(LR))){
        l <- which(LR$GENEID_L[i] == rownames(L))
        r <- which(LR$GENEID_R[i] == rownames(R))
        if(length(l)==1 && length(r) == 1){
            tnsr <- abind(tnsr,
                outer(as.vector(L[l,]), as.vector(R[r,]), outerfunc), along=3)
            Pair.name <- c(Pair.name,
                paste(LR$GENEID_L[i], LR$GENEID_R[i], sep="_"))
        }
    }
    tnsr[which(is.nan(tnsr))] <- 0
    tnsr <- sqrt(tnsr) / (mean(rbind(L, R)) + sqrt(tnsr))
    list(tnsr=tnsr, pairname=Pair.name)
}

.celltypemergedtensor.ca_2 <- function(input, LR, meanC, celltypes,
    mergeas, outerfunc){
    ct <- .cellType(input, celltypes, mergeas)
    ct <- .divLR(ct, LR)
    L <- ct[[1]]
    R <- ct[[2]]
    tmp <- outer(as.vector(L), as.vector(R), outerfunc)
    sqrt(tmp) / (meanC + sqrt(tmp))
}

.meanC <- function(input, celltypes, mergeas, LR){
    ct <- .cellType(input, celltypes, mergeas)
    ct <- .divLR(ct, LR)
    L <- ct[[1]]
    R <- ct[[2]]
    mean(rbind(L, R))
}

.labelpermutation.ca <- function(observed, input, LR, l, r, num.perm,
    celltypes, mergeas, outerfunc){
    meanC <- .meanC(input, celltypes, mergeas, LR)
    perm <- array(0, dim=c(dim(observed), 0))
    for(j in seq_len(num.perm)){
        tmp <- .celltypemergedtensor.ca_2(input,
            data.frame(GENEID_L=l, GENEID_R=r, stringsAsFactors=FALSE),
            meanC, sample(celltypes), mergeas, outerfunc)
        tmp <- observed - tmp
        tmp[which(tmp >= 0)] <- 0
        tmp[which(tmp < 0)] <- 1
        perm <- abind(perm, tmp)
    }
    modeSum(as.tensor(perm), m=3, drop=TRUE)@data / num.perm
}

.cellCellDecomp.CabelloAguilar <- function(input, LR, celltypes, ranks,
    rank, centering, mergeas, outerfunc, comb, num.sampling, num.perm,
    decomp, thr1, thr2, thr3, L1_A, L2_A, verbose){
    fout <- .celltypemergedtensor.ca(input, LR, celltypes,
        mergeas, outerfunc)
    tnsr <- as.tensor(fout$tnsr)
    Pair.name <- fout$pairname
    pval <- array(0, dim=c(dim(tnsr)[1:2], 0))
    for(i in seq_along(Pair.name)){
        cat(paste0(i, " / ", length(Pair.name), "\r"))
        l <- strsplit(Pair.name[i], "_")[[1]][1]
        r <- strsplit(Pair.name[i], "_")[[1]][2]
        tmp <- .labelpermutation.ca(tnsr[,,i]@data, input,
            LR, l, r, num.perm, celltypes, mergeas, outerfunc)
        pval <- abind(pval, tmp)
    }
    pval[which(is.nan(pval))] <- 1
    dimnames(pval) <- list(unique(names(celltypes)),
        unique(names(celltypes)), Pair.name)
    tnsr_cc <- modeSum(tnsr, m=3, drop=TRUE)@data
    dimnames(tnsr_cc) <- list(unique(names(celltypes)),
        unique(names(celltypes)))
    return(list(cellcellpattern=tnsr_cc, cellcelllrpairpattern=tnsr,
        pval=pval))
}

# Halpern et al.（LECs）
.Zscaling <- function(x){
    (x - mean(x)) / sd(x)
}

.celltypemergedtensor.hl <- function(input, LR, celltypes, mergeas, outerfunc){
    ct <- .cellType(input, celltypes, mergeas)
    ct <- .divLR(ct, LR)
    L <- ct[[1]]
    R <- ct[[2]]
    # Z-scaling
    Z_L <- t(apply(L, 1, .Zscaling))
    Z_R <- t(apply(R, 1, .Zscaling))
    tnsr <- array(0, dim=c(ncol(L), ncol(R), 0))
    Pair.name <- c()
    for(i in seq_len(nrow(LR))){
        l <- which(LR$GENEID_L[i] == rownames(L))
        r <- which(LR$GENEID_R[i] == rownames(R))
        if(length(l)==1 && length(r) == 1){
            tnsr <- abind(tnsr,
                sqrt(outer(as.vector(Z_L[l,])^2, as.vector(Z_R[r,])^2, outerfunc)), along=3)
            Pair.name <- c(Pair.name,
                paste(LR$GENEID_L[i], LR$GENEID_R[i], sep="_"))
        }
    }
    tnsr[which(is.nan(tnsr))] <- 0
    list(tnsr=tnsr, pairname=Pair.name)
}

.celltypemergedtensor.hl_2 <- function(input, LR, celltypes, mergeas, outerfunc){
    ct <- .cellType(input, celltypes, mergeas)
    ct <- .divLR(ct, LR)
    Z_L <- .Zscaling(ct[[1]])
    Z_R <- .Zscaling(ct[[2]])
    sqrt(outer(as.vector(Z_L)^2, as.vector(Z_R)^2, outerfunc))
}

.labelpermutation.hl <- function(observed, input, l, r, num.perm,
    celltypes, mergeas, outerfunc){
    perm <- array(0, dim=c(dim(observed), 0))
    for(j in seq_len(num.perm)){
        tmp <- .celltypemergedtensor.hl_2(input,
            data.frame(GENEID_L=l, GENEID_R=r, stringsAsFactors=FALSE),
            sample(celltypes), mergeas, outerfunc)
        tmp <- observed - tmp
        tmp[which(tmp >= 0)] <- 0
        tmp[which(tmp < 0)] <- 1
        perm <- abind(perm, tmp)
    }
    modeSum(as.tensor(perm), m=3, drop=TRUE)@data / num.perm
}

.cellCellDecomp.Halpern <- function(input, LR, celltypes, ranks, rank,
    centering, mergeas, outerfunc, comb, num.sampling, num.perm,
    decomp, thr1, thr2, thr3, L1_A, L2_A, verbose){
    fout <- .celltypemergedtensor.hl(input, LR, celltypes,
        mergeas, outerfunc)
    fout$tnsr[which(is.nan(fout$tnsr))] <- 0
    tnsr <- as.tensor(fout$tnsr)
    Pair.name <- fout$pairname
    pval <- array(0, dim=c(dim(tnsr)[1:2], 0))
    for(i in seq_along(Pair.name)){
        cat(paste0(i, " / ", length(Pair.name), "\r"))
        l <- strsplit(Pair.name[i], "_")[[1]][1]
        r <- strsplit(Pair.name[i], "_")[[1]][2]
        tmp <- .labelpermutation.hl(tnsr[,,i]@data, input,
            l, r, num.perm, celltypes, mergeas, outerfunc)
        pval <- abind(pval, tmp)
    }
    pval[which(is.nan(pval))] <- 1
    dimnames(pval) <- list(unique(names(celltypes)),
        unique(names(celltypes)), Pair.name)
    tnsr_cc <- modeSum(tnsr, m=3, drop=TRUE)@data
    dimnames(tnsr_cc) <- list(unique(names(celltypes)),
        unique(names(celltypes)))
    return(list(cellcellpattern=tnsr_cc, cellcelllrpairpattern=tnsr,
        pval=pval))
}

.flist <- list(
    # 3-Order, 2-mode decomposition : MultilinearCX
    "cx" = .cellCellDecomp.Third_3,
    # 3-Order, 2-mode decomposition : NTD2
    "ntd2" = .cellCellDecomp.Third_2,
    # 3-Order : NTD
    "ntd" = .cellCellDecomp.Third,
    # 2-Order : NMF
    "nmf" = .cellCellDecomp.Second,
    # Other method (Possible Combination)
    "pcomb" = .cellCellDecomp.PossibleCombination,
    # Other method (Pearson's Correlation Coefficient without LR pair)
    "pearson" = .cellCellDecomp.Pearson,
    # Other method (Spearman's Correlation Coefficient without LR pair)
    "spearman" = .cellCellDecomp.Spearman,
    # Other method (Euclidian Distance without LR pair)
    "distance" = .cellCellDecomp.Distance,
    # Other method (Pearson's Correlation Coefficient with LR pair)
    "pearson.lr" = .cellCellDecomp.Pearson.LR,
    # Other method (Spearman's Correlation Coefficient with LR pair)
    "spearman.lr" = .cellCellDecomp.Spearman.LR,
    # Other method (Euclidian Distance with LR pair)
    "distance.lr" = .cellCellDecomp.Distance.LR,
    # Other method (Celltype Label Permutation with LR pair)
    "label.permutation" = .cellCellDecomp.LabelPerm.LR,
    # Other method (SingleCellSignalR)
    "cabello.aguilar" = .cellCellDecomp.CabelloAguilar,
    # Other method（LECs）
    "halpern" = .cellCellDecomp.Halpern
)

.geneInformation_taxid <- function(sce, ah, taxid, LR){
    targetGeneID <- as.character(unique(c(LR$GENEID_L, LR$GENEID_R)))
    # Gene symbol
    if("SYMBOL" %in% AnnotationDbi::columns(ah) && !is.null(ah)){
        message("Related gene names are retrieved from AnnotationHub...")
        GeneName <- AnnotationDbi::select(ah, columns=c("SYMBOL", "ENTREZID"),
            keytype="ENTREZID", keys=targetGeneID)
        GeneName <- unique(GeneName)
        nonna1 <- which(!is.na(GeneName[,1]))
        nonna2 <- which(!is.na(GeneName[,2]))
        GeneName <- GeneName[intersect(nonna1, nonna2), ]
        # Bipartite Matching
        g <- graph.data.frame(as.data.frame(GeneName), directed=FALSE)
        V(g)$type <- bipartite_mapping(g)$type
        g <- max_bipartite_match(g)
        target <- as.character(unique(GeneName[, "ENTREZID"]))
        GeneName <- data.frame(
            ENTREZID=target,
            SYMBOL=as.character(g$matching[target]),
            stringsAsFactors = FALSE)
    }else{
        GeneName <- NULL
    }

    # GENENAME
    if("GENENAME" %in%  AnnotationDbi::columns(ah) && !is.null(ah)){
        message("Related gene descriptions are retrieved from AnnotationHub...")
        Description <- AnnotationDbi::select(ah, columns=c("GENENAME", "ENTREZID"),
            keytype="ENTREZID", keys=targetGeneID)
        Description <- Description[, c("ENTREZID", "GENENAME")]
    }else{
        Description <- NULL
    }

    # GO
    if("GO" %in%  AnnotationDbi::columns(ah) && !is.null(ah)){
        message("Related GO IDs are retrieved from AnnotationHub...")
        GO <- AnnotationDbi::select(ah, columns=c("GO", "ENTREZID"),
            keytype="ENTREZID", keys=targetGeneID)
        GO <- GO[, c("ENTREZID", "GO")]
    }else{
        GO <- NULL
    }

    # ENSG
    if("ENSEMBL" %in%  AnnotationDbi::columns(ah) && !is.null(ah)){
        message("Related Ensembl Gene IDs are retrieved from AnnotationHub...")
        ENSG <- AnnotationDbi::select(ah, columns=c("ENSEMBL", "ENTREZID"),
            keytype="ENTREZID", keys=targetGeneID)
        ENSG <- ENSG[, c("ENTREZID", "ENSEMBL")]
    }else{
        ENSG <- NULL
    }

    # ENSP
    if("ENSEMBLPROT" %in%  AnnotationDbi::columns(ah) && !is.null(ah)){
        message("Related Ensembl Protein IDs are retrieved from AnnotationHub...")
        ENSP <- AnnotationDbi::select(ah, columns=c("ENSEMBLPROT", "ENTREZID"),
            keytype="ENTREZID", keys=targetGeneID)
        ENSP <- ENSP[, c("ENTREZID", "ENSEMBLPROT")]
    }else{
        ENSP <- NULL
    }

    # UniProtKB
    if("UNIPROT" %in%  AnnotationDbi::columns(ah) && !is.null(ah)){
        message("Related UniProtKB IDs are retrieved from AnnotationHub...")
        UniProtKB <- AnnotationDbi::select(ah, columns=c("UNIPROT", "ENTREZID"),
            keytype="ENTREZID", keys=targetGeneID)
        UniProtKB <- UniProtKB[, c("ENTREZID", "UNIPROT")]
    }else if("UNIPROTID" %in%  AnnotationDbi::columns(ah) && !is.null(ah)){
        message("Related UniProtKB IDs are retrieved from AnnotationHub...")
        UniProtKB <- AnnotationDbi::select(ah, columns=c("UNIPROTID", "ENTREZID"),
            keytype="ENTREZID", keys=targetGeneID)
        UniProtKB <- UniProtKB[, c("ENTREZID", "UNIPROTID")]
        colnames(UniProtKB) <- c("ENTREZID", "UNIPROT")
    }else{
        UniProtKB <- NULL
    }

    # MeSH
    ah <- AnnotationHub()
    mcah <- mcols(ah)
    msg <- gsub("LRBaseDb", "MeSHDb", ah[metadata(sce)$ahid]$title)
    ahid <- mcah@rownames[which(mcah@listData$title == msg)]
    if(length(ahid) != 0){
        message(paste0("Related MeSH IDs are retrieved from ",
            "AnnotationHub..."))
        MeSHfile <- ah[[ahid]]
        MeSHobj <- MeSHDbi::MeSHDb(MeSHfile)
        MeSH <- MeSHDbi::select(MeSHobj, columns=c("MESHID", "GENEID"),
            keytype="GENEID",
            keys=targetGeneID)
    }else{
        MeSH <- NULL
    }

    # Reactome
    Reactome <- toTable(reactomeEXTID2PATHID)
    targetReactome <- unlist(lapply(targetGeneID,
        function(x){which(Reactome$gene_id == x)}))
    if(length(targetReactome) != 0){
        message(paste0("Related Reactome IDs are retrieved from ",
            "reactome.db package..."))
        Reactome <- Reactome[targetReactome, ]
    }else{
        Reactome <- NULL
    }

    # Output
    list(GeneName=GeneName, Description=Description, GO=GO,
        ENSG=ENSG, ENSP=ENSP, UniProtKB=UniProtKB,
        Reactome=Reactome, MeSH=MeSH)
}

.geneInformation <- function(sce, ah, spc, LR){
    targetGeneID <- as.character(unique(c(LR$GENEID_L, LR$GENEID_R)))
    if(!is.null(ah)){
        # Gene Name
        message("Related gene names are retrieved from AnnotationHub...")
        GeneName <- AnnotationDbi::select(ah, columns=c("SYMBOL", "ENTREZID"),
            keytype="ENTREZID", keys=targetGeneID)
        GeneName <- unique(GeneName)
        nonna1 <- which(!is.na(GeneName[,1]))
        nonna2 <- which(!is.na(GeneName[,2]))
        GeneName <- GeneName[intersect(nonna1, nonna2), ]
        # Bipartite Matching
        g <- graph.data.frame(as.data.frame(GeneName), directed=FALSE)
        V(g)$type <- bipartite_mapping(g)$type
        g <- max_bipartite_match(g)
        target <- as.character(unique(GeneName[, "ENTREZID"]))
        GeneName <- data.frame(
            ENTREZID=target,
            SYMBOL=as.character(g$matching[target]),
            stringsAsFactors = FALSE)

        # Description
        message("Related gene descriptions are retrieved from AnnotationHub...")
        Description <- AnnotationDbi::select(ah, columns=c("GENENAME", "ENTREZID"),
            keytype="ENTREZID", keys=targetGeneID)
        Description <- Description[, c("ENTREZID", "GENENAME")]

        # GO
        message("Related GO IDs are retrieved from AnnotationHub...")
        GO <- AnnotationDbi::select(ah, columns=c("GO", "ENTREZID"),
            keytype="ENTREZID", keys=targetGeneID)
        GO <- GO[, c("ENTREZID", "GO")]

        # ENSG
        if(spc %in% c("Hsa", "Mmu", "Rno", "Cel", "Dre")){
            message("Related Ensembl Gene IDs are retrieved from AnnotationHub...")
            ENSG <- AnnotationDbi::select(ah, columns=c("ENSEMBL", "ENTREZID"),
                keytype="ENTREZID", keys=targetGeneID)
            ENSG <- ENSG[, c("ENTREZID", "ENSEMBL")]
        }else{
            ENSG <- NULL
        }

        # ENSP
        if(spc %ni% c("Ath", "Pab", "Xtr", "Ssc")){
            message("Related Ensembl Protein IDs are retrieved from AnnotationHub...")
            ENSP <- AnnotationDbi::select(ah, columns=c("ENSEMBLPROT", "ENTREZID"),
                keytype="ENTREZID", keys=targetGeneID)
            ENSP <- ENSP[, c("ENTREZID", "ENSEMBLPROT")]
        }else{
            ENSP <- NULL
        }

        # UniProtKB
        if(spc %ni% c("Ath", "Xtr")){
            message("Related UniProtKB IDs are retrieved from AnnotationHub...")
            UniProtKB <- AnnotationDbi::select(ah, columns=c("UNIPROT", "ENTREZID"),
                keytype="ENTREZID", keys=targetGeneID)
            UniProtKB <- UniProtKB[, c("ENTREZID", "UNIPROT")]
        }else{
            UniProtKB <- NULL
        }
    }else{
        GeneName <- NULL
        Description <- NULL
        GO <- NULL
        ENSG <- NULL
        ENSP <- NULL
        UniProtKB <- NULL
    }

    # MeSH
    MeSHname <- paste0("MeSH.", gsub(".eg.db.sqlite", "",
        strsplit(metadata(sce)$lrbase, "LRBase.")[[1]][3]), ".eg.db")
    MeSH.load <- eval(parse(text=paste0("try(requireNamespace('", MeSHname, "', quietly=TRUE), silent=TRUE)")))
    if(!MeSH.load){
        eval(parse(text=paste0("try(BiocManager::install('",
            MeSHname, "', update=FALSE, ask=FALSE), silent=TRUE)")))
    }
    MeSH.load2 <- eval(parse(text=paste0("try(require('", MeSHname, "', quietly=TRUE), silent=TRUE)")))
    if(MeSH.load2){
        eval(parse(text=paste0("library(", MeSHname, ")")))
        message(paste0("Related MeSH IDs are retrieved from ",
            "MeSH.XXX.eg.db-type package..."))
        MeSHobj <- eval(parse(text=MeSHname))
        MeSH <- MeSHDbi::select(MeSHobj, columns=c("MESHID", "GENEID"),
            keytype="GENEID",
            keys=targetGeneID)
    }else{
        MeSH <- NULL
    }

    if(spc != "Pab"){
        # Reactome
        message(paste0("Related Reactome IDs are retrieved from ",
            "reactome.db package..."))
        Reactome <- toTable(reactomeEXTID2PATHID)
        targetReactome <- unlist(lapply(targetGeneID,
            function(x){which(Reactome$gene_id == x)}))
        Reactome <- Reactome[targetReactome, ]
    }else{
        Reactome <- NULL
    }
    # Output
    list(GeneName=GeneName, Description=Description, GO=GO,
        ENSG=ENSG, ENSP=ENSP, UniProtKB=UniProtKB,
        Reactome=Reactome, MeSH=MeSH)
}

#
# cellCellSetting
#
setGeneric("cellCellSetting", function(sce, lrbase, label, lr.evidence="known", color=NULL){
    standardGeneric("cellCellSetting")})

setMethod("cellCellSetting", signature(sce="SingleCellExperiment"),
    function(sce, lrbase, label, lr.evidence="known", color=NULL){
        userobjects <- deparse(substitute(sce))
        .cellCellSetting(userobjects, lrbase, label, lr.evidence, color, sce)})

.cellCellSetting <- function(userobjects, lrbase, label, lr.evidence, color, ...){
    sce <- list(...)[[1]]
    # class-check
    if(is(lrbase)[1] != "LRBaseDb"){
        stop("Please specify the lrbase as LRBaseDbi-class object
        such as LRBase.Hsa.eg.db")
    }
    # size-check
    if(length(label) != ncol(assay(sce))){
        stop("Please specify the label as the vector which has
            same length of nrow(assay(sce))")
    }
    if(is.null(color)){
        unique.label <- unique(label)
        base.color <- .ggdefault_cols(length(unique.label))
        color <- label
        for(i in seq_along(base.color)){
            color[which(color == unique.label[i])] <- base.color[i]
        }
    }

    # size-check
    if(length(color) != ncol(assay(sce))){
        stop("Please specify the color as the vector which has
            same length of nrow(assay(sce))")
    }
    # size-check
    if(length(unique(color)) != length(unique(label))){
        stop("Please specify the kind of elements containing
            in color and label as same")
    }

    # Rowname-check
    rn <- rownames(assay(sce))
    if(length(rn) != length(unique(rn))){
        stop("Please specify the row names of the input matrix is unique")
    }
    # Only matrix is accepted
    for(i in names(assays(sce))){
        if(!is.matrix(assays(sce)[[i]])){
            message("The input data is coverted to matrix format by as.matrix")
            assays(sce)[[i]] <- as.matrix(assays(sce)[[i]])
        }
    }
    # NA-check
    NA1 <- length(which(is.na(colnames(assay(sce)))))
    NA2 <- length(which(is.na(label)))
    NA3 <- length(which(is.na(color)))
    if(NA1 != 0){
        stop("At least one NA element is in colnames(assay(sce))\nPlease remove it.")
    }
    if(NA2 != 0){
        stop("At least one NA element is in label\nPlease remove it.")
    }
    if(NA3 != 0){
        stop("At least one NA element is in color\nPlease remove it.")
    }
    # Overwrite
    metadata(sce) <- list(lrbase=lrbase$conn@dbname,
        ahid=names(lrbase$dbfile),
        lr.evidence=lr.evidence, label=label, color=color)
    assign(userobjects, sce, envir=.GlobalEnv)
}

#
# cellCellRanks
#
setGeneric("cellCellRanks", function(sce, centering=TRUE,
    mergeas=c("mean", "sum"), outerfunc=c("*", "+"), comb=c("random", "all"),
    num.sampling=100, num.perm=1000, assayNames="counts", verbose=FALSE,
    num.iter1=5, num.iter2=5, num.iter3=NULL){
    standardGeneric("cellCellRanks")})

setMethod("cellCellRanks",
    signature(sce="SingleCellExperiment"),
    function(sce, centering=TRUE,
    mergeas=c("mean", "sum"), outerfunc=c("*", "+"), comb=c("random", "all"),
    num.sampling=100, num.perm=1000, assayNames="counts", verbose=FALSE,
    num.iter1=5, num.iter2=5, num.iter3=NULL){
        # Argument Check
        mergeas <- match.arg(mergeas)
        outerfunc <- match.arg(outerfunc)
        comb <- match.arg(comb)
        .cellCellRanks(centering, mergeas, outerfunc, comb, num.sampling, num.perm,
            assayNames, verbose, num.iter1, num.iter2, num.iter3, sce)
    })

.cellCellRanks <- function(centering, mergeas, outerfunc, comb, num.sampling, num.perm,
    assayNames, verbose, num.iter1, num.iter2, num.iter3, ...){
    # value-check
    if(num.iter1 < 0){
        stop("Please specify the num.iter1 as positive integer")
    }
    if(num.iter2 < 0){
        stop("Please specify the num.iter2 as positive integer")
    }
    # Import from sce object
    sce <- list(...)[[1]]
    # Import expression matrix
    input <- .importAssays(sce, assayNames)
    lr.evidence <- metadata(sce)$lr.evidence
    LR <- .extractLR(sce, lr.evidence, c("GENEID_L", "GENEID_R"))
    celltypes <- metadata(sce)$color
    names(celltypes) <- metadata(sce)$label
    l <- length(unique(celltypes))
    # Tensor is generated
    tnsr <- .cellCellDecomp.Third(input, LR, celltypes, ranks=c(1,1,1),
        rank=1, centering, mergeas, outerfunc, comb, num.sampling,
        num.perm, decomp=FALSE, thr1=log2(5), thr2=25, thr3=0.95, verbose)$cellcelllrpairpattern
    # Limit
    l1 <- min(dim(tnsr)[1], dim(tnsr)[2]*dim(tnsr)[3])
    l2 <- min(dim(tnsr)[2], dim(tnsr)[3]*dim(tnsr)[1])

    # NMF in matricised tensors in each mode
    out1 <- NMF(cs_unfold(tnsr, m=1)@data, runtime=num.iter1, rank.method="rss", J=1:l1)
    out2 <- NMF(cs_unfold(tnsr, m=2)@data, runtime=num.iter2, rank.method="rss", J=1:l2)

    # Reconsturction Error
    rss1 <- unlist(lapply(seq(l1), function(x, out1){
        eval(parse(text=paste0("mean(out1$Trial$Rank", x, "$original)")))
    }, out1=out1))
    rss2 <- unlist(lapply(seq(l2), function(x, out2){
        eval(parse(text=paste0("mean(out2$Trial$Rank", x, "$original)")))
    }, out2=out2))

    # Estimated rank
    rank1 <- min(which((max(rss1) - rss1) / (max(rss1) - min(rss1)) > 0.8))
    rank2 <- min(which((max(rss2) - rss2) / (max(rss2) - min(rss2)) > 0.8))

    if(!is.null(num.iter3)){
        # Limit
        l3 <- min(30, dim(tnsr)[3], dim(tnsr)[1]*dim(tnsr)[2])
        # NMF in matricised tensors in each mode
        out3 <- NMF(cs_unfold(tnsr, m=3)@data, runtime=num.iter3, rank.method="rss", J=1:l3)
        # Reconsturction Error
        rss3 <- unlist(lapply(seq(l3), function(x, out3){
            eval(parse(text=paste0("mean(out3$Trial$Rank", x, "$original)")))
        }, out3=out3))
        # Estimated rank
        rank3 <- min(which((max(rss3) - rss3) / (max(rss3) - min(rss3)) > 0.8))
        list(
            RSS=list(
                rss1=rss1,
                rss2=rss2,
                rss3=rss3),
            selected=c(rank1, rank2, rank3))
    }else{
        list(RSS=list(
                rss1=rss1,
                rss2=rss2),
            selected=c(rank1, rank2))
    }
}

#
# cellCellDecomp
#
setGeneric("cellCellDecomp", function(sce,
    algorithm=c("ntd2", "ntd", "nmf", "cx", "pearson",
    "spearman", "distance", "pearson.lr", "spearman.lr", "distance.lr",
    "pcomb", "label.permutation", "cabello.aguilar", "halpern"), ranks=c(3,3),
    rank=3, thr1=log2(5), thr2=25, thr3=0.95, L1_A=0, L2_A=0, verbose=FALSE,
    centering=TRUE, mergeas=c("mean", "sum"), outerfunc=c("*", "+"),
    comb=c("random", "all"), num.sampling=100, num.perm=1000,
    assayNames="counts", decomp=TRUE){
    standardGeneric("cellCellDecomp")})

setMethod("cellCellDecomp", signature(sce="SingleCellExperiment"),
    function(sce,
        algorithm=c("ntd2", "ntd", "nmf", "cx", "pearson", "spearman",
        "distance", "pearson.lr", "spearman.lr", "distance.lr", "pcomb",
        "label.permutation", "cabello.aguilar", "halpern"), ranks=c(3,3),
        rank=3, thr1=log2(5), thr2=25, thr3=0.95, L1_A=0, L2_A=0,
        verbose=FALSE, centering=TRUE, mergeas=c("mean", "sum"),
        outerfunc=c("*", "+"), comb=c("random", "all"),
        num.sampling=100, num.perm=1000, assayNames="counts", decomp=TRUE){
        # Argument Check
        algorithm = match.arg(algorithm)
        mergeas = match.arg(mergeas)
        outerfunc = match.arg(outerfunc)
        comb = match.arg(comb)

        userobjects <- deparse(substitute(sce))
        .cellCellDecomp(userobjects, algorithm, ranks, rank,
            thr1, thr2, thr3, L1_A, L2_A, verbose, centering, mergeas,
            outerfunc, comb, num.sampling,
            num.perm, assayNames, decomp, sce)})

.cellCellDecomp <- function(userobjects, algorithm, ranks, rank,
    thr1, thr2, thr3, L1_A, L2_A, verbose, centering, mergeas, outerfunc,
    comb, num.sampling, num.perm, assayNames, decomp, ...){
    # Import from sce object
    sce <- list(...)[[1]]
    # Import expression matrix
    input <- .importAssays(sce, assayNames)
    lr.evidence <- metadata(sce)$lr.evidence
    LR <- .extractLR(sce, lr.evidence, c("GENEID_L", "GENEID_R"))
    celltypes <- metadata(sce)$color
    names(celltypes) <- metadata(sce)$label

    # Gene Symbol-check
    genesymbols <- grep("^[A-Z]", rownames(input))
    if(length(genesymbols) != 0){
        message(paste0("Input data matrix may contains ",
            length(genesymbols),
            " gene symbols because the name contains some alphabets.\n",
            "scTensor uses only NCBI Gene IDs for now.\n",
            "Here, the gene symbols are removed and remaining ",
            nrow(input) - length(genesymbols),
            " NCBI Gene IDs are used for scTensor next step."
            ))
        input <- input[setdiff(seq_len(nrow(input)), genesymbols), ]
    }
    # thr-check
    if(comb == "random" && (num.sampling <= 0)){
        warning("None of cell-cell interaction will be detected.")
    }
    # Corresponding function of the algorithm
    f <- .flist[[algorithm]]
    if(is.null(f)){
        stop("Please specify the appropriate algorithm\n")
    }else{
        res.sctensor <- f(input, LR, celltypes, ranks, rank, centering,
            mergeas, outerfunc, comb, num.sampling, num.perm, decomp,
            thr1, thr2, thr3, L1_A, L2_A, verbose)
    }

    # Data size
    if (algorithm %in% c("cx", "ntd2", "ntd")){
        datasize <- c(ncol(res.sctensor[[2]]), ncol(res.sctensor[[3]]),
            ncol(res.sctensor[[4]]))
        recerror <- res.sctensor$recerror
        relchange <- res.sctensor$relchange
    }else{
        datasize <- NULL
        recerror <- NULL
        relchange <- NULL
    }

    # Overwrite
    metadata(sce) <- list(lrbase=metadata(sce)$lrbase,
        ahid=metadata(sce)$ahid,
        lr.evidence=metadata(sce)$lr.evidence,
        color=metadata(sce)$color, label=metadata(sce)$label,
        algorithm=algorithm, sctensor=res.sctensor, ranks=ranks,
        datasize=datasize, recerror=recerror, relchange=relchange)
    assign(userobjects, sce, envir=.GlobalEnv)
}

#
# cellCellReport
#
setGeneric("cellCellReport", function(sce, reducedDimNames,
    out.dir=tempdir(), html.open=FALSE,
    title="The result of scTensor",
    author="The person who runs this script", assayNames="counts", thr=100,
    top="full", p=0.05, upper=20,
    goenrich=TRUE, meshenrich=TRUE, reactomeenrich=TRUE,
    doenrich=TRUE, ncgenrich=TRUE, dgnenrich=TRUE, nbins=40){
    standardGeneric("cellCellReport")})

setMethod("cellCellReport", signature(sce="SingleCellExperiment"),
    function(sce, reducedDimNames, out.dir, html.open, title, author,
        assayNames, thr, top, p, upper, goenrich, meshenrich,
        reactomeenrich, doenrich, ncgenrich, dgnenrich, nbins){
        .cellCellReport(reducedDimNames, out.dir,
            html.open, title, author, assayNames, thr, top, p, upper,
            goenrich, meshenrich, reactomeenrich,
            doenrich, ncgenrich, dgnenrich, nbins, sce)})

.cellCellReport <- function(reducedDimNames,
    out.dir=tempdir(), html.open=FALSE,
    title="The result of scTensor",
    author="The person who runs this script", assayNames="counts",
    thr=100, top="full", p=0.05, upper=20,
    goenrich=TRUE, meshenrich=TRUE, reactomeenrich=TRUE,
    doenrich=TRUE, ncgenrich=TRUE, dgnenrich=TRUE, nbins=40, ...){
    # Import from sce object
    sce <- list(...)[[1]]
    # algorithm-check
    if(metadata(sce)$algorithm %ni% c("cx", "ntd2", "ntd")){
        stop(paste0("cellCellReport can be performed by the result of",
            " cellCellDecomp in which the algorithm is ",
            "specified as 'cx', 'ntd2' or 'ntd' for now."))
    }
    # The Directory for saving the analytical result
    dir.create(paste0(out.dir, "/figures"),
        showWarnings = FALSE, recursive = TRUE)
    dir.create(paste0(out.dir, "/figures/Ligand"),
        showWarnings = FALSE, recursive = TRUE)
    dir.create(paste0(out.dir, "/figures/Receptor"),
        showWarnings = FALSE, recursive = TRUE)
    dir.create(paste0(out.dir, "/figures/Tagcloud"),
        showWarnings = FALSE, recursive = TRUE)
    # File copy
    if(metadata(sce)$algorithm == "ntd"){
        file.copy(
            from = system.file("extdata", "Workflow.png", package = "scTensor"),
            to = paste0(out.dir, "/Workflow.png"),
            overwrite = TRUE)
    }
    if(metadata(sce)$algorithm %in% c("cx", "ntd2")){
        file.copy(
            from = system.file("extdata", "Workflow_2.png", package = "scTensor"),
            to = paste0(out.dir, "/Workflow_2.png"),
            overwrite = TRUE)
    }

    # HTML Report
    if(metadata(sce)$algorithm == "ntd"){
        .cellCellReport.Third(sce, thr, upper, assayNames, reducedDimNames, out.dir, author, title, p, top,
            goenrich, meshenrich, reactomeenrich,
            doenrich, ncgenrich, dgnenrich, nbins)
    }
    if(metadata(sce)$algorithm %in% c("cx", "ntd2")){
        .cellCellReport.Third_2(sce, thr, upper, assayNames, reducedDimNames, out.dir, author, title, p, top,
            goenrich, meshenrich, reactomeenrich,
            doenrich, ncgenrich, dgnenrich, nbins)
    }
    # HTML Open
    message(paste0("################################################\n",
        "Data files are saved in\n",
        out.dir, "\n################################################\n"))
    if(html.open){
        browseURL(paste0(out.dir, "/index.html"))
    }
}

#
# cellCellSimulate-related functions
#
setGeneric("getParam", function(object, name){
    standardGeneric("getParam")})
setGeneric("setParam<-", function(object, name, value){
    standardGeneric("setParam<-")})
setGeneric("show", function(object){
    standardGeneric("show")})

#
# cellCellSimulate
#
cellCellSimulate <- function(params = newCCSParams(), verbose = TRUE){
    # Class Check
    assertClass(params, classes = "CCSParams")
    # Get the parameters
    if(verbose){message("Getting the values of params...")}
    nGene <- getParam(params, "nGene")
    nCell <- getParam(params, "nCell")
    cciInfo <- getParam(params, "cciInfo")
    lambda <- getParam(params, "lambda")
    # Set random seed
    if(verbose){message("Setting random seed...")}
    seed <- getParam(params, "seed")
    # Simulation data
    if(verbose){message("Generating simulation data...")}
    # Generate Simulation data
    out <- .simulateDropoutCounts(nGene, nCell, cciInfo, lambda, seed)
    input <- out$simcount
    LR <- out$LR
    celltypes <- out$celltypes
    LR_CCI <- out$LR_CCI
    # Output
    if(verbose){message("Done!")}
    list(input=input, LR=LR, celltypes=celltypes, LR_CCI=LR_CCI)
}

context("cellCellFunctions")

library(SingleCellExperiment)
library(LRBase.Hsa.eg.db)

data(GermMale)
data(labelGermMale)
data(tsneGermMale)

# SingleCellExperiment-class
sce <- SingleCellExperiment(assays = list(counts = GermMale))
reducedDims(sce) <- SimpleList(TSNE=tsneGermMale$Y)

expect_true(is.null(metadata(sce)$lrbase))
expect_true(is.null(metadata(sce)$color))
expect_true(is.null(metadata(sce)$label))

# Setting
cellCellSetting(sce, LRBase.Hsa.eg.db, labelGermMale, names(labelGermMale))

expect_false(is.null(metadata(sce)$lrbase))
expect_false(is.null(metadata(sce)$color))
expect_false(is.null(metadata(sce)$label))

---
title: "How to interpret the HTML report generated by `cellCellReport` function"
author:
- name: Koki Tsuyuzaki
  affiliation: Laboratory for Bioinformatics Research,
    RIKEN Center for Biosystems Dynamics Research
- name: Manabu Ishii
  affiliation: Laboratory for Bioinformatics Research,
    RIKEN Center for Biosystems Dynamics Research
- name: Itoshi Nikaido
  affiliation: Laboratory for Bioinformatics Research,
    RIKEN Center for Biosystems Dynamics Research
  email: k.t.the-answer@hotmail.co.jp
package: scTensor
output:
  BiocStyle::html_document
vignette: |
  %\VignetteIndexEntry{scTensor: 2. Interpretation of HTML report}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

# Introduction

Here, we explain the way to interpret of HTML report generated by `cellCellReport`.
If `cellCellDecomp` is properly finished, we can perform `cellCellReport`
function to output the HTML report.
The results can be confirmed by typing `example(cellCellReport)`.
The report will be generated in the temporary directory (it costs 5 to 10 minutes).
The output directory contains some files and directories as follows.

- **index.{Rmd,html}** : The main HTML report
- **reanalysis.RData** : The R binary file for using in the HTML report and reanalysis of scTensor
- **Workflow_2.png** : The figure for the section 1. About scTensor Algorithm in the HTML report
- **figures** : The directory containing some figures for the HTML report
- **ligand.{Rmd,html}** : The HTML report for the section 6. Gene-wise Hypergraph in the HTML report
- **ligand_all.{Rmd,html}** : The HTML report for the section 6. Gene-wise Hypergraph in the HTML report
- **receptor.{Rmd,html}** : The HTML report for the section 6. Gene-wise Hypergraph in the HTML report
- **receptor_all.{Rmd,html}** : The HTML report for the section 6. Gene-wise Hypergraph in the HTML report
- **pattern_X_Y.{Rmd,html}** : The HTML report for For the section 7. (Ligand-Cell, Receptor-Cell, ) -related L-R Pairs in the HTML report

Here, look at the index.html.

![Figure1 : HTML report of `cellCellReport`](Report_HEADER.png)

# Interpretation of "1. About scTensor Algorithm"

In the HTML report, the 1st item describes the overview of `r Biocpkg("scTensor")` and other CCI-related packages.

![Figure2: 1. About scTensor Algorithm](Report_1.png)

# Interpretation of "2. Global statistics and plots"

The 2nd item describes all the R objects saved in **reanalysis.RData**,
which contains the result of `r Biocpkg("scTensor")`.
This file is saved in the output directory (**out.dir**) specified in `cellCellReport`,
and the user also can re-analyze the result of `r Biocpkg("scTensor")`.

![Figure3: 2. Global statistics and plots](Report_2.jpg)

Using `r CRANpkg("plotly")` package, `cellCellReport` generates some interactive plots.
For example, in item 2.1, the number of cells in each cell type can be confirmed when the cursor moved on the box.

![Figure4: 2.1 Number of cells in each celltype](Report_2_1.jpg)

In item 2.2, the number of expressed genes in each cell type (Non-zero genes) can be confirmed when the cursor moved on the box.
![Figure5: 2.2 Number of expressed genes in each cell type (Non-zero genes)](Report_2_2.jpg)

In item 2.3, the two-dimensional plot user specified can be confirmed.
![Figure6: 2.3 Two dimensional plot of all cells](Report_2_3.jpg)

In item 2.4, the distribution of core tensor values and the value of each (Ligand-Cell-type, Receptor-Cell-type, LR-pair) pattern can be confirmed.

The red bars mean that these values are selected by the threshold (**thr** parameters) in `cellCellReport`.

Note that the thr can be specified from 0 to 100,
the large thr value will generate too many HTML files (cf. 8. (Ligand-Cell, Receptor-Cell, LR-pair) Patterns)
and takes a long time.

![Figure7 : 2.4 Distribution of core tensor values](Report_2_4.png)

The 3-order CCI-tensor consisting of Cell_L $\times$ Cell_R $\times$ LR-pair (LR) are decomposed by `r CRANpkg("nnTensor")`,
in which the tensor is iteratively matricised to mode-1 (Ligand-Cell direction) and mode-2 (Receptor-Cell direction).

In each direction, NMF is performed and the strength of each directional pattern is summarized in the bar plots.

For example, in item 2.5, the distribution of mode-1 matricised tensor can be confirmed.

![Figure8: 2.5 Distribution of mode-1 matricised tensor (Ligand-Cell Direction) (1/2)](Mode1.jpg)

![Figure9: 2.5 Distribution of mode-1 matricised tensor (Ligand-Cell Direction) (2/2)](Report_2_5.jpg)

Likewise, in item 2.6, the distribution of mode-2 matricised tensor can be confirmed,

![Figure10 : 2.6 Distribution of mode-2 matricised tensor (Receptor-Cell Direction) (1/2)](Mode2.jpg)

![Figure11 : 2.6 Distribution of mode-2 matricised tensor (Receptor-Cell Direction) (2/2)](Report_2_6.jpg)

# Interpretation of "3. Ligand-Cell Patterns"

In the 3rd item, using the heatmap of `r CRANpkg("plotly")`, the user can interactively confirm the detail of Ligand-Cell Patterns extracted by `r CRANpkg("nnTensor")`.

![Figure14 : 3. Ligand-Cell Patterns](Report_3.jpg)

# Interpretation of "4. Receptor-Cell Patterns"

Likewise, in the 4th item, the user can interactively confirm the detail of Receptor-Cell Patterns.

![Figure15 : 4. Receptor-Cell Patterns](Report_4.jpg)

# Interpretation of "5. CCI-wise Hypergraph"

In the 6th item describes, the strength between Ligand-Cell Patterns and Receptor-Cell Patterns (CCI-strength),
by the summation of the core tensor with the mode-3 direction,
a matrix consisting of the number of Ligand-Cell Patterns $\times$ the number of Receptor-Cell Patterns.

![Figure18 : 6. CCI-wise Hypergraph (1/2)](Mode3Sum.jpg)
![Figure19 : 6. CCI-wise Hypergraph (2/2)](Report_5.png)

# Interpretation of "6. Gene-wise Hypergraph"

In the 7th item, the relationship between LR-pairs, which coexpressed in any LR-pair pattern at least one time.
Ligand genes are described as red nodes,
receptor genes are described as blue nodes,
and corresponding LR-pair patterns are described as the color of edges.
Using `r CRANpkg("visNetwork")` package,
these interactions can be interactively visualized.

![Figure20 : 7. Gene-wise Hypergraph](Report_6.png)

Under the gene-wise hypergraph, four hyperlinks are embedded.

In the 1st link, the details of the gene-wise hypergraph can be confirmed as a corresponding table in a ligand gene-centric manner.
This page can work as a reverse lookup search by "Ctrl + F";
by typing the gene name of ligand that the user is interested in,
the partner receptors, which are coexpressed in some LR-pair patterns,
also can be found.
![Figure21: Details of Ligand Gene-centric Overview (selected)](Ligand_selected.png)


In the 2nd link, the user can find all the partner receptors,
even if the partner receptors are not coexpressed in any LR-pair pattern,
and if they are not included in the data matrix.
![Figure22: Details of Ligand Gene-centric Overview (all)](Ligand_all.png)

Likewise, the receptor gene-centric reverse search page is embedded in the 3rd link,
![Figure23: Details of Receptor Gene-centric Overview (selected)](Receptor_selected.png)

and, in the 4th link, all the partner ligand genes are included.
![Figure24: Details of Receptor Gene-centric Overview (all) (1/2)](Receptor_all_HEADER.png)
![Figure25 : Details of Receptor Gene-centric Overview (all) (2/2)](Receptor_all.png)

# Interpretation of "7. (Ligand-Cell, Receptor-Cell, LR-pair) Patterns"

In the 8th item, the details of (Ligand-Cell, Receptor-Cell, LR-pair) Patterns are ordered by the size of the core tensor, and the link of each pattern is embedded.

(Note that the number of links is dependent on the **thr** parameter of `cellCellReport`.)
![Figure26: 8. (Ligand-Cell, Receptor-Cell, LR-pair) Patterns](Report_7.png)

For example, the 1st link describes the details of (3,2,) Pattern, which means the relationship of *1*st pattern of Ligand-Cell patterns, *1*st pattern of Receptor-Cell patterns, and *5*th pattern of LR-pair patterns.
![Figure27 : Details of (3,2,) Pattern (1/3)](Details_32_HEADER.png)

In this pattern, only one LR-pair is coexpressed (INSL3 and GNG11).
The hyperlinks to many databases and PubMed are also embedded.
The degree of the size of the LR-pair in the LR-pair pattern is quantified as P-value and Q-value.
![Figure28 : Details of (3,2,) Pattern (2/3)](Details_32_Pair.png)

Under the LR-pair list,
the results of many enrichment analysis are also embedded such as Gene Ontology (BP/MF/CC), Reactome, MeSH...etc.
![Figure29 : Details of (3,2,) Pattern (3/3)](Details_32_EA_HEADER.png)

User can confirm the detail of the result of `r Biocpkg("scTensor")`, and perform the biological interpretation.

# Session information {.unnumbered}

```{r sessionInfo, echo=FALSE}
sessionInfo()
```

---
title: "How to perform CCI simulation by `cellCellSimulate` function"
author:
- name: Koki Tsuyuzaki
  affiliation: Laboratory for Bioinformatics Research,
    RIKEN Center for Biosystems Dynamics Research
- name: Manabu Ishii
  affiliation: Laboratory for Bioinformatics Research,
    RIKEN Center for Biosystems Dynamics Research
- name: Itoshi Nikaido
  affiliation: Laboratory for Bioinformatics Research,
    RIKEN Center for Biosystems Dynamics Research
  email: k.t.the-answer@hotmail.co.jp
package: scTensor
output:
  BiocStyle::html_document
vignette: |
  %\VignetteIndexEntry{scTensor: 3. Simulation of CCI}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

# Introduction

Here, we explain the way to generate CCI simulation data.
`r Biocpkg("scTensor")` has a function `cellCellSimulate`
to generate the simulation data.

The simplest way to generate such data is `cellCellSimulate` with default parameters.

```{r cellCellSimulate_Default, echo=TRUE}
suppressPackageStartupMessages(library("scTensor"))
sim <- cellCellSimulate()
```

This function internally generate the parameter sets by `newCCSParams`,
and the values of the parameter can be changed, and specified as the input of `cellCellSimulate` by users as follows.

```{r cellCellSimulate_Setting, echo=TRUE}
# Default parameters
params <- newCCSParams()
str(params)

# Setting different parameters
# No. of genes : 1000
setParam(params, "nGene") <- 1000
# 3 cell types, 20 cells in each cell type
setParam(params, "nCell") <- c(20, 20, 20)
# Setting for Ligand-Receptor pair list
setParam(params, "cciInfo") <- list(
    nPair=500, # Total number of L-R pairs
    # 1st CCI
    CCI1=list(
        LPattern=c(1,0,0), # Only 1st cell type has this pattern
        RPattern=c(0,1,0), # Only 2nd cell type has this pattern
        nGene=50, # 50 pairs are generated as CCI1
        fc="E10"), # Degree of differential expression (Fold Change)
    # 2nd CCI
    CCI2=list(
        LPattern=c(0,1,0),
        RPattern=c(0,0,1),
        nGene=30,
        fc="E100")
    )
# Degree of Dropout
setParam(params, "lambda") <- 10
# Random number seed
setParam(params, "seed") <- 123

# Simulation data
sim <- cellCellSimulate(params)
```

The output object **sim** has some attributes as follows.

Firstly, **sim$input** contains a synthetic gene expression matrix.
The size can be changed by **nGene** and **nCell** parameters described above.

```{r input, echo=TRUE}
dim(sim$input)
sim$input[1:2,1:3]
```

Next, **sim$LR** contains a ligand-receptor (L-R) pair list.
The size can be changed by **nPair** parameter of **cciInfo**,
and the differentially expressed (DE) L-R pairs
are saved in the upper side of this matrix.
Here, two DE L-R patterns are specified as **cciInfo**,
and each number of pairs is 50 and 30, respectively.

```{r LR, echo=TRUE}
dim(sim$LR)
sim$LR[1:10,]
sim$LR[46:55,]
sim$LR[491:500,]
```

Finally, **sim$celltypes** contains a cell type vector.
Since **nCell** is specified as "c(20, 20, 20)" described above,
three cell types are generated.

```{r celltypes, echo=TRUE}
length(sim$celltypes)
head(sim$celltypes)
table(names(sim$celltypes))
```

# Session information {.unnumbered}

```{r sessionInfo, echo=FALSE}
sessionInfo()
```

---
title: "Roadmap to prepare the input matrix for `scTensor`"
author:
- name: Koki Tsuyuzaki
  affiliation: Laboratory for Bioinformatics Research,
    RIKEN Center for Biosystems Dynamics Research
- name: Manabu Ishii
  affiliation: Laboratory for Bioinformatics Research,
    RIKEN Center for Biosystems Dynamics Research
- name: Itoshi Nikaido
  affiliation: Laboratory for Bioinformatics Research,
    RIKEN Center for Biosystems Dynamics Research
  email: k.t.the-answer@hotmail.co.jp
package: scTensor
output:
  BiocStyle::html_document
vignette: |
  %\VignetteIndexEntry{scTensor: 1. Data format and ID conversion}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

This vignette has been changed in BioC 3.14, when each data package (LRBase.XXX.eg.db) is deprecated and the way to provide LRBase data has changed to AnnotationHub-style.

# Introduction
We explain the way to create a matrix, in which the row names are **NCBI Gene ID (ENTREZID)**, for specifying an input of `r Biocpkg("scTensor")`.
Typical `r Biocpkg("scTensor")` workflow can be described as below.

```{r scTensor with NCBI Gene ID, eval=FALSE}
library("scTensor")
library("AnnotationHub")
library("LRBaseDbi")

# Input matrix
input <- ...
sce <- SingleCellExperiment(assays=list(counts = input))
# Celltype vector
label <- ...
# LRBase.XXX.eg.db
ah <- AnnotationHub()
dbfile <- query(ah, c("LRBaseDb", "Homo sapiens"))[[1]]
LRBase.Hsa.eg.db <- LRBaseDbi::LRBaseDb(dbfile)
# Setting
cellCellSetting(sce, LRBase.Hsa.eg.db, label)
```

In `r Biocpkg("scTensor")`, the row names of the input matrix is limited only NCBI Gene ID to cooperate with the other R packages (cf. data("Germline")). Since the user has many different types of the data matrix, here we introduce some situations and the way to convert the row names of the user's matrix as NCBI Gene ID.

# Step.1: Create a gene-level expression matrix

First of all, we have to prepare the expression matrix (gene $\times$ cell).
There are many types of single-cell RNA-Seq (scRNA-Seq) technologies and the situation will be changed by the used experimental methods and quantification tools described below.

## Case I: Gene-level quantification
In Plate-based scRNA-Seq experiment (i.e. Smart-Seq2, Quart-Seq2, CEL-Seq2, MARS-Seq,...etc), the FASTQ file is generated in each cell. After the mapping of reads in each FASTQ file to the reference genome, the same number of BAM files will be generated.
By using some quantification tools such as [featureCounts](http://bioinf.wehi.edu.au/featureCounts/), or [HTSeq-count](https://htseq.readthedocs.io/en/release_0.11.1/count.html), user can get the expression matrix and used as the input of `r Biocpkg("scTensor")`. These tools simply count the number of reads in union-exon in each gene.
One downside of these tools is that such tools do not take "multimap" of not unique reads into consideration and the quantification is not accurate.
Therefore, we recommend the transcript-level quantification and gene-level summarization explained below.

## Case II: Transcript-level quantification
Some quantification tools such as [RSEM](https://deweylab.github.io/RSEM/), [Sailfish](https://www.cs.cmu.edu/~ckingsf/software/sailfish/), [Salmon](https://combine-lab.github.io/salmon/), [Kallisto](https://pachterlab.github.io/kallisto/), and [StringTie](http://ccb.jhu.edu/software/stringtie/index.shtml) use the reference transcriptome instead of genome, and quantify the expression level in each transcript. After the quantification, the transcript-level expression can be summarized to gene-level by using `summarizeToGene` function of `r Biocpkg("tximport")`. [The paper of tximport](https://f1000research.com/articles/4-1521) showed that the transcript-level expression summalized to gene-level is more accurate than the gene-level expression calculated by featureCounts.

Note that if you use the reference transcriptome of [GENCODE](https://www.gencodegenes.org/human/stats.html), this step becomes slightly complicated. Although the number of transcripts of GENCODE and that of [Ensembl](https://ensembl.org/Homo_sapiens/Info/Annotation) is almost the same,
and actually, most of the transcript is duplicated in these two databases,
the gene identifier used in GENCODE looks complicated like "ENST00000456328.2|ENSG00000223972.5|OTTHUMG00000000961.2|OTTHUMT00000362751.1|DDX11L1-202|DDX11L1|1657|processed_transcript|".
In such a case, firstly only Ensembl Transcript ID should be extracted (e.g. ENST00000456328.2), removed the version (e.g. ENST00000456328), summarized to Ensembl Gene ID by tximport (e.g. ENSG00000223972), and then converted to NCBI Gene ID (e.g. 100287102) by each organism package such as `r Biocpkg("Homo.sapiens")`.

## Case III: UMI-count
In the droplet-based scRNA-Seq experiment (i.e. Drop-Seq, inDrop RNA-Seq, 10X Chromium), unique molecular identifier (UMI) is introduced for avoiding the bias of PCR amplification, and after multiplexing by cellular barcode, digital gene expression (DGE) matrix is generated by counting the number of types of UMI mapped in each gene.

When user perform Drop-seq, [Drop-Seq tool](https://github.com/broadinstitute/Drop-seq) can generate the DGE matrix.

Another tool [Alevin](https://salmon.readthedocs.io/en/latest/alevin.html), which is a subcommand of Salmon is also applicable to Drop-seq data. In such case [tximport] with option "type = 'alevin'" can import the result of Alevin into R and summarize the DGE matrix.

When the user performs 10X Chromium, using [Cell Ranger](https://support.10xgenomics.com/single-cell-gene-expression/software/pipelines/latest/what-is-cell-ranger) developed by 10X Genomics is straightforward.

Although Cell Ranger is implemented by Python, starting from the files generated by Cell Ranger (e.g. filtered_gene_bc_matrices/{hg19,mm10}/{barcodes.tsv,genes.tsv,matrix.mtx}), `r CRANpkg("Seurat")` can import these files to an R object.

For example, according to the tutorial of Seurat ([Seurat - Guided Clustering Tutorial](https://satijalab.org/seurat/v3.0/pbmc3k_tutorial.html)), PBMC data of Cell Ranger can be imported by the `Read10X` function and DGE matrix of UMI-count is available by the output of `CreateSeuratObject` function.

```{r Seurat, eval=FALSE}
if(!require(Seurat)){
    BiocManager::install("Seurat")
    library(Seurat)
}

# Load the PBMC dataset
pbmc.data <- Read10X(data.dir = "filtered_gene_bc_matrices/hg19/")

# Initialize the Seurat object with the raw (non-normalized data).
pbmc <- CreateSeuratObject(counts = pbmc.data,
  project = "pbmc3k", min.cells = 3, min.features = 200)
```

**Note that the matrix is formatted as a sparse matrix of `r CRANpkg("Matrix")` package (MM: Matrix market), but the `r Biocpkg("scTensor")` assumes dense matrix for now.**
By using `as.matrix` function,
the sparse matrix is easily converted to a dense matrix as follows.

```
# Sparse matrix to dense matrix
for_sc <- as.matrix(pbmc.data)
```

# Step.2: Convert the row names of a matrix as NCBI Gene ID (ENTREZID)

Even after creating the gene-level expression matrix in Step.1,
many kinds of gene-level gene identifiers can be assigned as row names of the matrix such as Ensembl Gene ID, RefSeq, or Gene Symbol.
Again, only NCBI Gene ID can be used as row names of the input matrix of `r Biocpkg("scTensor")`.
To do such a task, we originally implemented a function `convertRowID` function of `r Biocpkg("scTGIF")`.
The only user has to prepare for using this function is the 1. input matrix (or data.frame) filled with only numbers, 2. current gene-level gene identifier in each row of the input matrix, and 3. corresponding table containing current gene-level gene identifier (left) and corresponding NCBI Gene ID (right).
The usage of this function is explained below.

## Case I: Ensembl Gene ID to NCBI Gene ID
In addition to 1. and 2., the user has to prepare the 3. corresponding table.
Here we introduce two approaches to assign the user's Ensembl Gene ID to NCBI Gene ID.
First approarch is using [Organism DB](https://bioconductor.org/packages/release/BiocViews.html#___OrganismDb) packages such as `r Biocpkg("Homo.sapiens")`, `r Biocpkg("Mus.musculus")`, and `r Biocpkg("Rattus.norvegicus")`.

Using the `select` function of Organism DB, the corresponding table can be retrieved like below.

```{r Ensembl with Organism DB, echo=TRUE}
suppressPackageStartupMessages(library("scTensor"))
if(!require(Homo.sapiens)){
    BiocManager::install("Homo.sapiens")
    suppressPackageStartupMessages(library(Homo.sapiens))
}
if(!require(scTGIF)){
    BiocManager::install("scTGIF")
    suppressPackageStartupMessages(library(scTGIF))
}

# 1. Input matrix
input <- matrix(1:20, nrow=4, ncol=5)
# 2. Gene identifier in each row
rowID <- c("ENSG00000204531", "ENSG00000181449",
  "ENSG00000136997", "ENSG00000136826")
# 3. Corresponding table
LefttoRight <- select(Homo.sapiens,
  column=c("ENSEMBL", "ENTREZID"),
  keytype="ENSEMBL", keys=rowID)
# ID conversion
(input <- convertRowID(input, rowID, LefttoRight))
```

Second approarch is using `r Biocpkg("AnnotationHub")` package.

Although only three Organism DB packages are explicitly developed,
even if the data is generated from other species (e.g. Zebrafish, Arabidopsis thaliana),
similar database is also available from `r Biocpkg("AnnotationHub")`,
and `select` function can be performed like below.

```{r Ensembl with AnnotationHub, echo=TRUE}
suppressPackageStartupMessages(library("AnnotationHub"))

# 1. Input matrix
input <- matrix(1:20, nrow=4, ncol=5)
# 3. Corresponding table
ah <- AnnotationHub()
# Database of Human
hs <- query(ah, c("OrgDb", "Homo sapiens"))[[1]]
LefttoRight <- select(hs,
  column=c("ENSEMBL", "ENTREZID"),
  keytype="ENSEMBL", keys=rowID)
(input <- convertRowID(input, rowID, LefttoRight))
```

## Case II: Gene Symbol to NCBI Gene ID
When using cellranger or `r CRANpkg("Seurat")` to quantify UMI-count (cf. Step1, Case III),
the row names of the input matrix might be Gene Symbol,
and have to be converted to NCBI Gene ID.
As well as the Case I described above,
[Organism DB](https://bioconductor.org/packages/release/BiocViews.html#___OrganismDb)
and `r Biocpkg("AnnotationHub")` will support such a task like below.

```{r Gene Symbol with Organism DB, echo=TRUE}
# 1. Input matrix
input <- matrix(1:20, nrow=4, ncol=5)
# 2. Gene identifier in each row
rowID <- c("POU5F1", "SOX2", "MYC", "KLF4")
# 3. Corresponding table
LefttoRight <- select(Homo.sapiens,
  column=c("SYMBOL", "ENTREZID"),
  keytype="SYMBOL", keys=rowID)
# ID conversion
(input <- convertRowID(input, rowID, LefttoRight))
```

```{r Gene Symbol with AnnotationHub, echo=TRUE}
# 1. Input matrix
input <- matrix(1:20, nrow=4, ncol=5)
# 3. Corresponding table
ah <- AnnotationHub()
# Database of Human
hs <- query(ah, c("OrgDb", "Homo sapiens"))[[1]]
LefttoRight <- select(hs,
  column=c("SYMBOL", "ENTREZID"),
  keytype="SYMBOL", keys=rowID)
(input <- convertRowID(input, rowID, LefttoRight))
```

# Step.3: Normalize the count matrix

Finally, we introduce some situations to perform some normalization methods of gene expression matrix.

If a user converts a Seurat object to a SingleCellExperient object by using `as.SingleCellExperiment`,
the result of the `NormalizeData` function (log counts) is inherited to the SingleCellExperient object as follows;

```{r Seurat normalization, eval=FALSE}
pbmc2 <- NormalizeData(pbmc, normalization.method = "LogNormalize",
    scale.factor = 10000)
sce <- as.SingleCellExperiment(pbmc2)
assayNames(sce) # counts, logcounts
```

If the user want to use `r Biocpkg("scater")` package,
`calculateCPM` or `normalize` function can calculate the normalized expression values as follows; (see also [the vignette of scater](https://bioconductor.org/packages/release/bioc/vignettes/scater/inst/doc/vignette-intro.html)).

```{r Scater normalization, eval=FALSE}
if(!require(scater)){
    BiocManager::install("scater")
    library(scater)
}
sce <- SingleCellExperiment(assays=list(counts = input))
cpm(sce) <- calculateCPM(sce)
sce <- normalize(sce)
assayNames(sce) # counts, normcounts, logcounts, cpm
```

Any original normalization can be stored in the sce.
For example, we can calculate the value of count per median (CPMED) as follows;

```{r Original normalization, eval=FALSE}
# User's Original Normalization Function
CPMED <- function(input){
    libsize <- colSums(input)
    median(libsize) * t(t(input) / libsize)
}
# Normalization
normcounts(sce) <- log10(CPMED(counts(sce)) + 1)
```

We recommend using the normcounts slot to save such original normalization values.
After the normalization, such values can be specified by `assayNames` option in `cellCellRanks` `cellCellDecomp` and `cellCellReport` functions.

# Session information {.unnumbered}

```{r sessionInfo, echo=FALSE}
sessionInfo()
```

---
title: "Detection and visualization of cell-cell interactions using `LRBase` and `scTensor`"
author:
- name: Koki Tsuyuzaki
  affiliation: Laboratory for Bioinformatics Research,
    RIKEN Center for Biosystems Dynamics Research
- name: Manabu Ishii
  affiliation: Laboratory for Bioinformatics Research,
    RIKEN Center for Biosystems Dynamics Research
- name: Itoshi Nikaido
  affiliation: Laboratory for Bioinformatics Research,
    RIKEN Center for Biosystems Dynamics Research
  email: k.t.the-answer@hotmail.co.jp
package: scTensor
output:
  BiocStyle::html_document
vignette: |
  %\VignetteIndexEntry{scTensor}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

This vignette has been changed in BioC 3.14, when each data package (LRBase.XXX.eg.db) is deprecated and the way to provide LRBase data has changed to AnnotationHub-style.

# Specification change of `LRBase` and `scTensor` from BioC 3.14 (Nov. 2021)
This section is for the users of previous LRBase.XXX.eg.db-type packages and scTensor. The specifications of the LRBase.XXX.eg.db and scTensor have changed significantly since BioC 3.14. Specifically, the distribution of all LRBase.XXX.eg.db-type packages will be abolished, and the policy has been switched to one where the data is placed on a cloud server called AnnotationHub, and users are allowed to retrieve the data only when they really need it. The following are the advantages of this AnnotationHub-style.

- The installation time of the entire Bioconductor packages will be reduced.
- Old data will be archived.
- Data reproducibility is ensured (e.g. the version of the data can be specified, such as "v002").

# Introduction
## About Cell-Cell Interaction (CCI) databases

Due to the rapid development of single-cell RNA-Seq (scRNA-Seq) technologies, wide variety of cell types such as multiple organs of a healthy person, stem cell niche and cancer stem cell have been found. Such complex systems are composed of communication between cells (cell-cell interaction or CCI).

Many CCI studies are based on the ligand-receptor (L-R)-pair list of FANTOM5 project^[Jordan A. Ramilowski, A draft network of ligand-receptor-mediated multicellular signaling in human, Nature Communications, 2015] as the evidence of CCI (http://fantom.gsc.riken.jp/5/suppl/Ramilowski_et_al_2015/data/PairsLigRec.txt). The project proposed the L-R-candidate genes by following two basises.

1. **Subcellular Localization**
    1. Known Annotation (UniProtKB and HPRD) : The term **"Secreted"** for
    candidate ligand genes and **"Plasma Membrane"** for
    candidate receptor genes
    2. Computational Prediction (LocTree3 and PolyPhobius)
2. **Physical Binding of Proteins** : Experimentally validated PPI
    (protein-protein interaction) information of HPRD and STRING

The project also merged the data with previous L-R database such as
IUPHAR/DLRP/HPMR and filter out the list without PMIDs. The recent L-R databases such as CellPhoneDB and SingleCellSignalR also manually curated L-R pairs, which are not listed in IUPHAR/DLRP/HPMR. In Bader Laboratory, many putative L-R databases are predicted by their standards. In our framework, we expanded such L-R databases for **134** organisms based on the ortholog relationships. For the details, check the summary of rikenbit/lrbase-workflow^[https://github.com/rikenbit/lrbase-workflow#summary], which is the Snakemake workflow to create LRBase data in each bi-annual update of Bioconductor.

## `LRBase` and `scTensor` framework

Our L-R databases (`LRBase`) are provided a cloud server called AnnotationHub, and users are allowed to retrieve the data only when they really need it. Downloaded data is stored as a cache file on our local machines by the `r Biocpkg("BiocFileCache")` mechanism. Then, the data is converted to LRBase object by `r Biocpkg("LRBaseDbi")`. We also developed `r Biocpkg("scTensor")`, which is a method to detect CCI and the CCI-related L-R pairs simultaneously. This document provides the way to use `r Biocpkg("LRBaseDbi")`, LRBase objects, and `r Biocpkg("scTensor")` (Figure 1).

![Figure 1 : Workflow of L-R-related packages](Workflow_2.png)

# Usage
## LRBase objects (ligand-receptor database for 134 organisms)

To create the LRBase of 134 organisms, we introduced 36 approarches including known/putative L-R pairing.
Please see the evidence code of lrbase-workflow^[https://github.com/rikenbit/lrbase-workflow].

### Data retrieval from `AnnotationHub`

First of all, we download the data of LRBase from AnnotationHub.
`AnnotationHub::AnnotationHub` retrieve the metadata of all the data stored in cloud server.

```{r AHub1, echo=TRUE}
library("AnnotationHub")
ah <- AnnotationHub()
mcols(ah)
```

Specifying some keywords in `query()`, we can find LRBase data in AnnotationHub.

```{r AHub2, echo=TRUE}
dbfile <- query(ah, c("LRBaseDb", "Homo sapiens"))[[1]]
```

AnnotationHub also keeps old data as an archive, so please make sure you have the latest version (e.g. "v002" or higher) when you search for LRBaseDb.

Then, we can convert `dbfile` into LRBase object by using `LRBaseDbi`.

```{r AHub3, echo=TRUE}
library("LRBaseDbi")
LRBase.Hsa.eg.db <- LRBaseDbi::LRBaseDb(dbfile)
```

### columns, keytypes, keys, and select

Some data access functions are available for LRBase objects.
Any data table are retrieved by 4 functions defined by
`r Biocpkg("AnnotationDbi")`; `columns`, `keytypes`, `keys`, and `select` and commonly implemented by `r Biocpkg("LRBaseDbi")` package. `columns` returns the rows which we can retrieve in LRBase objects. `keytypes` returns the rows which can be used as the optional parameter in `keys` and select functions against LRBase objects. `keys` function returns the value of keytype. `select` function returns the rows in particular columns, which are having user-specified keys. This function returns the result as a dataframe. See the vignette of `r Biocpkg("AnnotationDbi")` for more details.

```{r Access, echo=TRUE}
columns(LRBase.Hsa.eg.db)
keytypes(LRBase.Hsa.eg.db)
key_HSA <- keys(LRBase.Hsa.eg.db, keytype="GENEID_L")
head(select(LRBase.Hsa.eg.db, keys=key_HSA[1:2],
            columns=c("GENEID_L", "GENEID_R"), keytype="GENEID_L"))
```

### Other functions

Other additional functions like `species`, `nomenclature`, and `listDatabases` are available. In each LRBase.XXX.eg.db-type package, `species` function returns the common name and `nomenclature` returns the scientific name. `listDatabases` function returns the source of data. `dbInfo` returns the information of the package. `dbfile` returns the directory where sqlite file is stored. `dbschema` returns the schema of the database. `dbconn` returns the connection to the sqlite database.

```{r Other1, echo=TRUE}
lrNomenclature(LRBase.Hsa.eg.db)
species(LRBase.Hsa.eg.db)
lrListDatabases(LRBase.Hsa.eg.db)
lrVersion(LRBase.Hsa.eg.db)

dbInfo(LRBase.Hsa.eg.db)
dbfile(LRBase.Hsa.eg.db)
dbschema(LRBase.Hsa.eg.db)
dbconn(LRBase.Hsa.eg.db)
```

Combined with `dbGetQuery` function of `r CRANpkg("RSQLite")` package,
more complicated queries also can be submitted.

```{r Other2, echo=TRUE}
suppressPackageStartupMessages(library("RSQLite"))
dbGetQuery(dbconn(LRBase.Hsa.eg.db),
  "SELECT * FROM DATA WHERE GENEID_L = '9068' AND GENEID_R = '14' LIMIT 10")
```

## `scTensor` (CCI-tensor construction, decomposition, and HTML reporting)

Combined with LRBase object and user's gene expression matrix of scRNA-Seq, `r Biocpkg("scTensor")` detects CCIs and generates HTML reports for exploratory data inspection. The algorithm of `r Biocpkg("scTensor")` is as follows.

Firstly, `r Biocpkg("scTensor")` calculates the celltype-level mean vectors, searches the corresponding pair of genes in the row names of the matrix, and extracted as two vectors.

Next, the cell type-level mean vectors of ligand expression and that of receptor expression are multiplied as outer product and converted to cell type $\times$ cell type matrix. Here, the multiple matrices can be represented as a three-order "tensor" (Ligand-Cell * Receptor-Cell * L-R-Pair). `r Biocpkg("scTensor")` decomposes the tensor into a small tensor (core tensor) and two factor matrices. Tensor decomposition is very similar to the matrix decomposition like PCA (principal component analysis). The core tensor is similar to the eigenvalue of PCA; this means that how much the pattern is outstanding. Likewise, three matrices are similar to the PC scores/loadings of PCA; These represent which ligand-cell/receptor-cell/L-R-pair are informative. When the matrices have negative values, interpreting which direction (+/-) is important and which is not, is a difficult and laboring task. That's why, `r Biocpkg("scTensor")`
performs non-negative Tucker2 decomposition (NTD2), which is non-negative version of tensor decomposition (cf. `r CRANpkg("nnTensor")`).

Finally, the result of NTD2 is summarized as an HTML report. Because most of the plots are visualized by `r CRANpkg("plotly")` package, the precise information of the plot can be interactively confirmed by user's on-site web browser. The two factor matrices can be interactively viewed and which cell types and which L-R-pairs are likely to be interacted each other. The mode-3 (LR-pair direction) sum of the core tensor is calculated and visualized as Ligand-Receptor Patterns. Detail of (Ligand-Cell, Receptor-Cell, L-R-pair) Patterns are also visualized.

### Creating a `SingleCellExperiment` object

Here, we use the scRNA-Seq dataset of male germline cells and somatic cells$^{3}$[GSE86146](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE86146) as demo data. For saving the package size, the number of genes is strictly reduced by the standard of highly variable genes with a threshold of the p-value are 1E-150 (cf. [Identifying highly variable genes](http://pklab.med.harvard.edu/scw2014/subpop_tutorial.html)). That's why we won't argue about the scientific discussion of the data here.

We assume that user has a scRNA-Seq data matrix containing expression count data summarised at the level of the gene. First, we create a `r Biocpkg("SingleCellExperiment")` object containing the data. The rows of the object correspond to features, and the columns correspond to cells. The gene identifier is limited as NCBI Gene ID for now.

To improve the interpretability of the following HTML report, we highly recommend that user specifies the two-dimensional data of input data (e.g. PCA, t-SNE, or UMAP). Such information is easily specified by `reducedDims` function of `r Biocpkg("SingleCellExperiment")` package and is saved to reducedDims slot of `SingleCellExperiment` object (Figure \@ref(fig:cellCellSetting)).

```{r SCE1, echo=TRUE}
suppressPackageStartupMessages(library("scTensor"))
suppressPackageStartupMessages(library("SingleCellExperiment"))
```

```{r SCE2, fig.cap="Germline, Male, GSE86146", echo=TRUE, fig.width=10, fig.height=10}
data(GermMale)
data(labelGermMale)
data(tsneGermMale)

sce <- SingleCellExperiment(assays=list(counts = GermMale))
reducedDims(sce) <- SimpleList(TSNE=tsneGermMale$Y)
plot(reducedDims(sce)[[1]], col=labelGermMale, pch=16, cex=2,
  xlab="Dim1", ylab="Dim2", main="Germline, Male, GSE86146")
legend("topleft", legend=c(paste0("FGC_", 1:3), paste0("Soma_", 1:4)),
  col=c("#9E0142", "#D53E4F", "#F46D43", "#ABDDA4", "#66C2A5", "#3288BD", "#5E4FA2"),
  pch=16)
```

### Parameter setting: `cellCellSetting`

To perform the tensor decomposition and HTML report, user is supposed to specify

- 1. LRBaseDb object (e.g. LRBase.Hsa.eg.db)
- 2. cell type vector of each cell (e.g. names(labelGermMale))

to `SingleCellExperiment` object.

```{r cellCellSetting, echo=TRUE}
cellCellSetting(sce, LRBase.Hsa.eg.db, names(labelGermMale))
```

The corresponding information is registered to the metadata slot of `SingleCellExperiment` object by `cellCellSetting` function.

### CCI-tensor construction and decomposition: `cellCellDecomp`

After `cellCellSetting`, we can perform tensor decomposition by `cellCellDecomp`. Here the parameter `ranks` is specified as dimension of core tensor. For example, c(2, 3) means The data tensor is decomposed to 2 ligand-patterns and 3 receptor-patterns.

```{r cellCellDecomp, echo=TRUE}
set.seed(1234)
cellCellDecomp(sce, ranks=c(2,3))
```

Although user has to specify the rank to perform cellCellDecomp, we implemented a simple rank estimation function based on the eigenvalues distribution of PCA in the matricised tensor in each mode in `cellCellRank`. `rks$selected` is also specified as rank parameter of `cellCellDecomp`.

```{r cellCellRank, echo=TRUE}
(rks <- cellCellRanks(sce))
rks$selected
```

### HTML Report: `cellCellReport`

If `cellCellDecomp` is properly finished, we can perform `cellCellReport` function to output the HTML report like below. Please type `example(cellCellReport)` and the report will be generated in the temporary directory (it costs 5 to 10 minutes). After `cellCellReport`, multiple R markdown files, compiled HTML files, figures, and R binary file containing the result of analysis are saved to `out.dir` (Figure 2). For more details, open the `index.html` by your web browser. Combined with cloud storage service such as Amazon Simple Storage Service (S3), it can be a simple web application and multiple people like collaborators can confirm the same report simultaneously.

![Figure2 : cellCellReport function of scTensor](Report.png)

# Session Information {.unnumbered}

```{r sessionInfo, echo=FALSE}
sessionInfo()
```

---
title: "How to reanalyze the results of scTensor"
author:
- name: Koki Tsuyuzaki
  affiliation: Laboratory for Bioinformatics Research,
    RIKEN Center for Biosystems Dynamics Research
- name: Manabu Ishii
  affiliation: Laboratory for Bioinformatics Research,
    RIKEN Center for Biosystems Dynamics Research
- name: Itoshi Nikaido
  affiliation: Laboratory for Bioinformatics Research,
    RIKEN Center for Biosystems Dynamics Research
  email: k.t.the-answer@hotmail.co.jp
package: scTensor
output:
  BiocStyle::html_document
vignette: |
  %\VignetteIndexEntry{scTensor: 4. Reanalysis of the results of scTensor}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

# Summary of the output objects of scTensor

Here, we introduced the objects saved in reanalysis.RData.

```{r reanalysis.RData, eval=FALSE}
library("scTensor")
load("reanalysis.RData")
```

After performing `cellCellReport`, some R objects are saved in the reanalysis.RData as follows;

- **sce** : SingleCellExperiment object
    - **metadata(sce)$lrbase** : The file pass to the database file of LRBase
    - **metadata(sce)$color** : The color vector specified by `cellCellSetting`
    - **metadata(sce)$label** : The label vector specified by `cellCellSetting`
    - **metadata(sce)$algorithm** : The algorithm for performing `r Biocpkg("scTensor")`
    - **metadata(sce)$sctensor** : The results of `r Biocpkg("scTensor")`
        - **metadata(sce)\$sctensor\$ligand** : The factor matrix (Ligand)
        - **metadata(sce)\$sctensor\$receptor** : The factor matrix (Receptor)
        - **metadata(sce)\$sctensor\$lrpair** : The core tensor
    - **metadata(sce)$datasize** : The data size of CCI tensor
    - **metadata(sce)$ranks** : The number of lower dimension in each direction of CCI tensor
    - **metadata(sce)$recerror** : Reconstruction Error of NTD
    - **metadata(sce)$relchange** : Relative Change of NTD
- **input** : The gene expression matrix <# Genes * # Cells>
- **twoD** : The result of 2D dimensional reduction (e.g. t-SNE)
- **LR** : The Ligand-Receptor corresponding table extracted from LRBase.XXX.eg.db
- **celltypes** : The celltype label and color scheme
- **index** : The core tensor values
- **corevalue** : The core tensor values (normalized)
- **selected** : The selected corevalue position with thr threshold "thr"
- **ClusterL** : The result of analysis in each L vector
- **ClusterR** : The result of analysis in each R vector
- **out.vecLR** : The result of analysis in LR pairs
- **g** : The igraph object to visualize ligand-receptor gene network

# Execution of scTensor with the different options

Using the `reanalysis.RData`, some users may want to perform `r Biocpkg("scTensor")` with different parameters.

For example, some users want to perform `cellCellDecomp` with different ranks, perform `cellCellReport` with omitting some enrichment analysis, provide the results to their collaborators.

To do such tasks, just type like belows.

```{r Reanalysis, eval=FALSE}
library("AnnotationHub")
library("LRBaseDbi")

# Create LRBase object
ah <- AnnotationHub()
dbfile <- query(ah, c("LRBaseDb", "Homo sapiens"))[[1]]
LRBase.Hsa.eg.db <- LRBaseDbi::LRBaseDb(dbfile)

# Register the file pass of user's LRBase
metadata(sce)$lrbase <- dbfile(LRBase.Hsa.eg.db)

# CCI Tensor Decomposition
cellCellDecomp(sce, ranks=c(6,5), assayNames="normcounts")

# HTML Report
cellCellReport(sce, reducedDimNames="TSNE", assayNames="normcounts",
    title="Cell-cell interaction within Germline_Male, GSE86146",
    author="Koki Tsuyuzaki", html.open=TRUE,
    goenrich=TRUE, meshenrich=FALSE, reactomeenrich=FALSE,
    doenrich=FALSE, ncgenrich=FALSE, dgnenrich=FALSE)
```

# Session information {.unnumbered}

```{r sessionInfo, echo=FALSE}
sessionInfo()
```

\docType{methods}
\name{setParam}
\alias{setParam}
\alias{setParam<-}
\alias{setParam,CCSParams-method}
\alias{setParam<-,CCSParams-method}
\alias{setParam,CCSParams,ANY-method}
\alias{setParam<-,CCSParams,ANY-method}
\title{Set a parameter}
\usage{
setParam(object, name) <- value
\S4method{setParam}{CCSParams}(object, name, value)
}
\arguments{
\item{object}{object to set parameter in.}

\item{name}{name of the parameter to set.}

\item{value}{value to set the paramter to.}
}
\value{
Object with new parameter value.
}
\description{
Function for setting parameter values.
}
\examples{
params <- newCCSParams()

setParam(params, "nGene") <- 20000
setParam(params, "nCell") <- c(12, 43, 323)
setParam(params, "cciInfo") <- list(nPair=2000,
                                   CCI1=list(
                                       LPattern=c(1,0,0),
                                       RPattern=c(0,1,1),
                                       nGene=100,
                                       fc="E10"),
                                   CCI2=list(
                                       LPattern=c(0,0,1),
                                       RPattern=c(1,1,1),
                                       nGene=200,
                                       fc="E10"),
                                   CCI3=list(
                                       LPattern=c(1,1,1),
                                       RPattern=c(1,0,1),
                                       nGene=300,
                                       fc="E10")
                                   )
setParam(params, "lambda") <- 0.1
setParam(params, "seed") <- 111
}

\name{m}
\alias{m}
\docType{data}
\title{
The gene-wise mean vector of Quartz-Seq data.
}
\description{
This data is internally used in cellCellSimulate function.
}
\usage{data(m)}
\examples{
data(m)
}
\keyword{datasets}

\name{newCCSParams}
\alias{newCCSParams}
\title{New Params}
\usage{
newCCSParams()
}
\arguments{
\item{}{Nothing.}
}
\value{
New Params object.
}
\description{
Create a new CCSParams object.
}
\examples{
params <- newCCSParams()
}

\name{cellCellRanks}
\docType{methods}
\alias{cellCellRanks,SingleCellExperiment-method}
\alias{cellCellRanks}

\title{
Rank estimation of the CCI-tensor
}
\description{
SVD is performed in each mode.
}
\usage{
cellCellRanks(sce, centering=TRUE,
    mergeas=c("mean", "sum"), outerfunc=c("*", "+"), comb=c("random", "all"),
    num.sampling=100, num.perm=1000, assayNames = "counts", verbose=FALSE,
    num.iter1=5, num.iter2=5, num.iter3=NULL)
}

\arguments{
\item{sce}{
A object generated by instantization of SingleCellExperiment-class.
}
\item{centering}{
When the value is TRUE, input matrix is summarized as celltype-level vectors
(Default: TRUE).
}
\item{mergeas}{
When the centering is TRUE, "mean" (celltype-level mean vector)
or "sum" (celltype-level sum vector) is calculated (Default: "mean").
}
\item{outerfunc}{
When the centering is TRUE, "*" (Kronecker product) or "+" (Kronecker sum) or is calculated (Default: "+").
}
\item{comb}{
When the centering is FALSE, "random" (random cell-cell pairing)
or "all" (all possible cell-cell pairing) is calculed (Default: "random").
}
\item{num.sampling}{
The number of random sampling used (Default: 100).
}
\item{num.perm}{
The number of the permutation in label permutation test (Default: 1000).
}
\item{assayNames}{
The unit of gene expression for using scTensor (e.g. normcounts, cpm...etc) (Default: "counts").
}
\item{verbose}{
The verbose parameter for nnTensor::NTD (Default: FALSE).
}
\item{num.iter1}{
The number of iteration to estimate the rank of mode-1 matricised data tensor (Default: 5).
}
\item{num.iter2}{
The number of iteration to estimate the rank of mode-2 matricised data tensor (Default: 5).
}
\item{num.iter3}{
The number of iteration to estimate the rank of mode-3 matricised data tensor (Default: NULL).
}
}

\value{
RSS: A list with three elements, in which each element means the average reconstructed error in each rank.
selected: A vector with three elements, in which each element means the estimated ranks in mode-1, 2 and 3 matricization.
}
\seealso{
\code{\link[SingleCellExperiment]{SingleCellExperiment}}.
}

\author{Koki Tsuyuzaki}

\examples{
showMethods("cellCellRanks")
}

\keyword{methods}

\name{v}
\alias{v}
\docType{data}
\title{
The gene-wise variance vector of Quartz-Seq data.
}
\description{
This data is internally used in cellCellSimulate function.
}
\usage{data(v)}
\examples{
data(v)
}
\keyword{datasets}

\name{CCSParams-class}
\docType{class}
\alias{CCSParams-class}
\title{Class "CCSParams"}

\description{
The parameter object to be specified against cellCellSimulate function.
}

\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("CCSParams", ...)}.
}

\section{Slots}{
   \describe{
    \item{nGene:}{The number of genes.}
    \item{nCell:}{The number of cells.}
    \item{cciInfo:}{The parameter to describe the CCI.}
    \item{lambda:}{The parameter for dropout simulation.}
    \item{seed:}{The seed for using random numbers.}
  }
}

\section{Methods}{
  \describe{
    \item{newCCSParams}{Generator of CCSParams object.}
    \item{getParam}{Getter function of the slot in CCSParams object.}
    \item{setParam<-}{Setter function of the slot in CCSParams object.}
  }
}

\seealso{
  \code{\link{newCCSParams}}, \code{\link{getParam}}, \code{\link{setParam<-}}
}

\keyword{classes}

\name{scTensor-package}
\alias{scTensor-package}
\alias{scTensor}
\docType{package}
\title{
\packageTitle{scTensor}
}
\description{
\packageDescription{scTensor}
}
\details{

The DESCRIPTION file:
\packageDESCRIPTION{scTensor}
\packageIndices{scTensor}
}
\author{
\packageAuthor{scTensor}

Maintainer: \packageMaintainer{scTensor}
}

\keyword{ package }
\seealso{\code{\link{GermMale}},\code{\link{labelGermMale}},
\code{\link{tsneGermMale}},\code{\link{cellCellSetting}},
\code{\link{cellCellDecomp}},\code{\link{cellCellReport}}}
\examples{
ls("package:scTensor")
}

\name{labelGermMale}
\alias{labelGermMale}
\docType{data}
\title{
The vector contains the celltype information and color scheme of GermMale
}
\description{
A vector with 852 length (cells).
}
\usage{data(labelGermMale)}
\details{
The Cluster label is downloaded from original paper page of Cell Stem Cell
(https://www.sciencedirect.com/science/article/pii/S1934590917300784)
}
\references{
Li L. and Dong J. and Yan L. and Yong J. et al. (2017)
Single-Cell RNA-Seq Analysis Maps Development of Human Germline Cells
and Gonadal Niche Interactions. \emph{Cell Stem Cell}, \bold{20(6)}: 858-873
}
\seealso{
\code{\link{GermMale}}, \code{\link{tsneGermMale}}.
}
\examples{
data(labelGermMale)
}
\keyword{datasets}

\name{cellCellSetting}
\docType{methods}
\alias{cellCellSetting,SingleCellExperiment-method}
\alias{cellCellSetting}

\title{
Parameter setting for scTensor
}
\description{
All parameters is saved to metadata slot of SingleCellExperiment object.
}
\usage{
cellCellSetting(sce, lrbase, label, lr.evidence="known", color=NULL)
}

\arguments{
\item{sce}{
A object generated by instantization of SingleCellExperiment-class.
}
\item{lrbase}{
Ligand-Receptor database (LRBase.XXX.eg.db-type package).
}
\item{label}{
Cellular label information for distingusishing
which cells belong to common celltypes.
}
\item{lr.evidence}{
The evidence code for L-R pair list (Default: "known").
When you specify "known", DLRP, IUPHAR, HPMR, CELLPHONEDB, SINGLECELLSIGNALR are searched, and other databases are searched, when you specify "putative".
You can also specify multiple databases at once (e.g. c("SWISSPROT_STRING", "TREMBL_STRING")).
cf. https://github.com/rikenbit/lrbase-workflow
}
\item{color}{
Color scheme for adding color against the cells (Default: NULL).
If the value is not specified, automatically the color vector is generated.
}
}

\value{
The result is saved to metadata slot of SingleCellExperiment object.
}
\seealso{
\code{\link[SingleCellExperiment]{SingleCellExperiment}}.
}

\author{Koki Tsuyuzaki}

\examples{
showMethods("cellCellSetting")
}

\keyword{methods}

\name{GermMale}
\alias{GermMale}
\docType{data}
\title{
The matrix which is used as test data of scTensor.
}
\description{
A matrix with 242 rows (genes) * 852 columns (cells).
}
\usage{data(GermMale)}
\details{
The data matrix is downloaded from GEO Series GSE86146
(https://www.ncbi.nlm.nih.gov/geo/download/?acc=GSE86146&format=file).
Only male data is extracted and then the gene symbol is converted to
NCBI Gene ID by Homo.sapiens package.

For saving the package size, the number of genes are strictlly reduced by the
standard of highlly variable genes with threshold of p-value is 1E-300.
}
\references{
Li L. and Dong J. and Yan L. and Yong J. et al. (2017)
Single-Cell RNA-Seq Analysis Maps Development of Human Germline Cells
and Gonadal Niche Interactions. \emph{Cell Stem Cell}, \bold{20(6)}: 858-873
}
\seealso{
\code{\link{labelGermMale}}, \code{\link{tsneGermMale}}.
}
\examples{
data(GermMale)
}
\keyword{datasets}

\name{cellCellDecomp}
\docType{methods}
\alias{cellCellDecomp,SingleCellExperiment-method}
\alias{cellCellDecomp}

\title{
Performing scTensor
}
\description{
All parameters is saved to metadata slot of SingleCellExperiment object.
}
\usage{
cellCellDecomp(sce, algorithm=c("ntd2", "ntd", "nmf", "cx", "pearson",
    "spearman", "distance", "pearson.lr", "spearman.lr", "distance.lr",
    "pcomb", "label.permutation", "cabello.aguilar", "halpern"), ranks=c(3,3), rank=3, thr1=log2(5), thr2=25, thr3=0.95, L1_A=0, L2_A=0, verbose=FALSE,
    centering=TRUE, mergeas=c("mean", "sum"), outerfunc=c("*", "+"),
    comb=c("random", "all"), num.sampling=100, num.perm=1000, assayNames = "counts", decomp=TRUE)
}

\arguments{
\item{sce}{
The object generated by instantization of SingleCellExperiment-class.
}
\item{algorithm}{
Algorithm for constrcting cell-cell similarity matrix.
"ntd2", "ntd", "nmf", "cx", "pearson", "spearman", "distance",
"pearson.lr", "spearman.lr", "distance.lr", "pcomb" or "label.permutation"
can be specified (Default: ntd2).
}
\item{ranks}{
The size of the core tensor decomposed by NTD.
Each element means (Number of Ligand-Cell Pattern,
Number of Receptor-Cell Pattern, Number of LR-pairs Pattern)
(Default: c(3,3)).
}
\item{rank}{
The number of low dimension of NMF (Default: 3).
}
\item{thr1}{
The threshold used by pcomb (Default: log2(5)).
}
\item{thr2}{
The threshold used by pcomb (Default: 25).
}
\item{thr3}{
The threshold used by cx (Default: 0.95).
}
\item{L1_A}{
The parameter to control the sparseness (Default: 0).
}
\item{L2_A}{
The parameter to control the outlier (Default: 0).
}
\item{verbose}{
The verbose parameter for nnTensor::NTD (Default: FALSE).
}
\item{centering}{
When the value is TRUE, input matrix is summarized as celltype-level vectors
(Default: TRUE).
}
\item{mergeas}{
When the centering is TRUE, "sum" (celltype-level sum vector)
or "mean" (celltype-level average vector) is calculated (Default: "sum").
}
\item{outerfunc}{
When the centering is TRUE, "+" (Kronecker sum) or "*" (Kronecker product)
is calculated (Default: "+").
}
\item{comb}{
When the centering is FALSE, "random" (random cell-cell pairing)
or "all" (all possible cell-cell pairing) is calculed (Default: "random").
}
\item{num.sampling}{
The number of random sampling used (Default: 100).
}
\item{num.perm}{
The number of the permutation in label permutation test (Default: 1000).
}
\item{assayNames}{
The unit of gene expression for using scTensor (e.g. normcounts, cpm...etc) (Default: "counts").
}
\item{decomp}{
When the value is TRUE, cell-cell interaction tensor is decomposed
(Default: TRUE).
}
}

\value{
The result is saved to metadata slot of SingleCellExperiment object.
}
\seealso{
\code{\link[SingleCellExperiment]{SingleCellExperiment}}.
}

\author{Koki Tsuyuzaki}

\examples{
showMethods("cellCellDecomp")
}

\keyword{methods}

\name{cellCellReport}
\docType{methods}
\alias{cellCellReport,SingleCellExperiment-method}
\alias{cellCellReport}

\title{
HTML report of the result of scTensor
}
\description{
The result is saved as HTML report which contains with multiple files.
}
\usage{
cellCellReport(sce, reducedDimNames,
    out.dir=tempdir(), html.open=FALSE,
    title="The result of scTensor",
    author="The person who runs this script", assayNames = "counts", thr=100,
    top="full", p=0.05, upper=20,
    goenrich=TRUE, meshenrich=TRUE, reactomeenrich=TRUE,
    doenrich=TRUE, ncgenrich=TRUE, dgnenrich=TRUE, nbins=40)
}

\arguments{
\item{sce}{
A object generated by instantization of SingleCellExperiment-class.
}
\item{reducedDimNames}{
The name of two-dimentional data saved in reducedDimNames slot of
SingleCellExperiment object.
}
\item{out.dir}{
The output directory for saving HTML report (out.dir: tempdir()).
}
\item{html.open}{
Whether the result of HTML report is opened when the calculation is finished
(Default: FALSE).
}
\item{title}{
The title of HTML report (Default: "The result of scTensor").
}
\item{author}{
The author of HTML report (Default: "The person who runs this script").
}
\item{assayNames}{
The unit of gene expression for using scTensor (e.g. normcounts, cpm...etc) (Default: "counts").
}
\item{thr}{
The threshold for selection of top pecentage of core tensor elements
(Default: 100 (1 to 100)).
}
\item{top}{
top genes in each (*,*,*)-pattern which are selected and summarized in the report (Default: "full")
}
\item{p}{
The threshold of p-value of the enrichment analysis (Default: 1E-2)
}
\item{upper}{
The maxium number of HTML reports generates (Default: 20)
}
\item{goenrich}{
Whether GO-Enrichment analysis is performed (Default: TRUE)
}
\item{meshenrich}{
Whether MeSH-Enrichment analysis is performed (Default: TRUE)
}
\item{reactomeenrich}{
Whether Reactome-Enrichment analysis is performed (Default: TRUE)
}
\item{doenrich}{
Whether DO-Enrichment analysis is performed (Default: TRUE)
}
\item{ncgenrich}{
Whether NCG-Enrichment analysis is performed (Default: TRUE)
}
\item{dgnenrich}{
Whether DGN-Enrichment analysis is performed (Default: TRUE)
}
\item{nbins}{
The number of bins used for the two dimensional plot of schex (Default: 40)
}
}

\value{
The result is saved as HTML report which contains with multiple files.
}
\seealso{
\code{\link[SingleCellExperiment]{SingleCellExperiment}}.
}

\author{Koki Tsuyuzaki}

\examples{
	if(interactive()){
	# Package Loading
	library("SingleCellExperiment")
	library("AnnotationHub")
	if(!require(LRBaseDbi)){
	    BiocManager::install("LRBaseDbi")
	    library(LRBaseDbi)
	}
	ah <- AnnotationHub()
	dbfile <- query(ah, c("LRBaseDb", "Homo sapiens"))[[1]]
	LRBase.Hsa.eg.db <- LRBaseDbi::LRBaseDb(dbfile)

	# Data Loading
	data(GermMale)
	data(labelGermMale)
	data(tsneGermMale)

	# SingleCellExperiment Object
	sce <- SingleCellExperiment(assays=list(counts = GermMale))
	reducedDims(sce) <- SimpleList(TSNE=tsneGermMale$Y)

	# User's Original Normalization Function
	CPMED <- function(input){
	    libsize <- colSums(input)
	    median(libsize) * t(t(input) / libsize)
	}
	# Normalization
	normcounts(sce) <- log10(CPMED(counts(sce)) + 1)

	# Registration of required information into metadata(sce)
	cellCellSetting(sce, LRBase.Hsa.eg.db, names(labelGermMale))

	# Rank Estimation
	rks <- cellCellRanks(sce, assayNames="normcounts")

	# CCI Tensor Decomposition
	set.seed(1234)
	cellCellDecomp(sce, ranks=rks$selected, assayNames="normcounts")

	# HTML Report
	options(device.ask.default = FALSE)
	cellCellReport(sce, reducedDimNames="TSNE",
        out.dir=tempdir(), html.open=FALSE,
        title="The result of scTensor",
        author="The person who runs this script",
        assayNames="counts", thr=100,
        top="full", p=0.05, upper=20,
        goenrich=TRUE, meshenrich=TRUE, reactomeenrich=TRUE,
        doenrich=TRUE, ncgenrich=TRUE, dgnenrich=TRUE, nbins=40)
    }else{
        showMethods("cellCellReport")
    }
}

\keyword{methods}

\name{tsneGermMale}
\alias{tsneGermMale}
\docType{data}
\title{
The result of Rtsne against GermMale
}
\description{
A List contains some parameters and the result of Rtsne function.
}
\usage{data(tsneGermMale)}
\details{
Rtsne is performed as follows.

library(Rtsne)
set.seed(123)
tsneGermMale <- Rtsne(dist(t(GermMale)), is_distance=TRUE, perplexity=40)
}
\references{
Li L. and Dong J. and Yan L. and Yong J. et al. (2017) Single-Cell RNA-Seq
Analysis Maps Development of Human Germline Cells and Gonadal Niche
Interactions. \emph{Cell Stem Cell}, \bold{20(6)}: 858-873
}
\seealso{
    \code{\link{labelGermMale}}, \code{\link{GermMale}}.
}
\examples{
data(tsneGermMale)
}
\keyword{datasets}

\docType{methods}
\name{getParam}
\alias{getParam}
\alias{getParam,CCSParams-method}
\title{Get a parameter}
\usage{
getParam(object, name)

\S4method{getParam}{CCSParams}(object, name)
}
\arguments{
\item{object}{object to get parameter from.}

\item{name}{name of the parameter to get.}
}
\value{
The extracted parameter value
}
\description{
Accessor function for getting parameter values.
}
\examples{
params <- newCCSParams()

getParam(params, "nGene")
getParam(params, "nCell")
getParam(params, "cciInfo")
getParam(params, "lambda")
getParam(params, "seed")
}

\name{cellCellSimulate}
\docType{methods}
\alias{cellCellSimulate,SingleCellExperiment-method}
\alias{cellCellSimulate}

\title{
Parameter Simulate for scTensor
}
\description{
All parameters is saved to metadata slot of SingleCellExperiment object.
}
\usage{
cellCellSimulate(params = newCCSParams(), verbose = TRUE)
}

\arguments{
\item{params}{
A parameter object generated by newCCSParams().
}
\item{verbose}{
Whether the message is outputted or not (Default: TRUE).
}
}

\value{
A list object containing simcount, LR, and celltype.
simcount is the synthetic count matrix, LR is the synthetic ligand-receptor pair list, and celltype is the vector to specity the celltype of the each column of simcount.
}

\author{Koki Tsuyuzaki}

\examples{
showMethods("cellCellSimulate")
}

\keyword{methods}
